//============================================================================
// FILE: ConstantDescriptor.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe des descripteurs constants
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _CONSTANTDESCRIPTOR_HH
#define _CONSTANTDESCRIPTOR_HH
// SYSTEM INCLUDES
//
//#include <>                                    //

// PROJECT INCLUDES
//
#include "Descriptor.h"

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name CDPFORTEXLes descripteurs constants
\begin{itemize} \item Directive d'inclusion :
[P]
\#include ConstantDescriptor.h
[/P]
\end{itemize}

Un descripteur d'entité est déclaré constant, c'est-à-dire installé comme une instance de la présente classe, lorsque sa valeur n'est pas censée évoluer au cours de la simulation. En réalité rien n'empêche de changer la valeur d'un descripteur constant, mais sont inactivés certains services tels que ceux rendus par la classe [C]DescValueMonitor[/C], ceux attachés à l'utilisation interactive du simulateur (notamment l'affichage de la dynamique des changements de valeur) et enfin la possibilité de sauvegarder sur fichier externe la chronique des changements de valeurs. \\

Lors de la création d'un descripteur constant, l'attribut 'valeur par défaut' est défini pour les différents types conformément au tableau suivant\index{valeur par défaut} :\\

Attention : cette valeur par défaut (renvoyée par [Y]GetDefaultValue[/Y]) n'est pas assignée automatiquement à la valeur du descripteur (renvoyée par [Y]Get..Value[/Y]). A cette fin, utiliser la méthode [Y]SetToDefaultValue[/Y] dans le constructeur de la sous-classe, si la valeur doit être celle par défaut.

\begin{tabular}{cccl}
{\tiny classe} & {\tiny valeur par défaut} & {\tiny valeur} & {\tiny type}\\
\hline
{\tiny [C]IntConstantDescriptor[/C]} & {\tiny 0} & {\tiny 0} & {\tiny valeur entière ({\em int)}} \\
{\tiny [C]FloatConstantDescriptor[/C]} & {\tiny 0.0} & {\tiny 0.0} & {\tiny valeur flottante ({\em float)}} \\
{\tiny [C]DoubleConstantDescriptor[/C]} & {\tiny 0.0} & {\tiny 0.0} & {\tiny valeur flottante double précision ({\em double)}} \\
{\tiny [C]StringConstantDescriptor[/C]} & {\tiny ""} & {\tiny ""} & {\tiny une chaÃ®ne de caractères ({\em char*)}} \\
{\tiny [C]EntityConstantDescriptor[/C]} & {\tiny NULL} & {\tiny NULL} & {\tiny {\em BasicEntity*}} \\
{\tiny [C]EntitySpecConstantDescriptor[/C]} & {\tiny NULL} & {\tiny NULL} & {\tiny {\em EntitySpec*}} \\
{\tiny [C]IntTabConstantDescriptor[/C]} & {\tiny un tableau de taille nulle} & {\tiny NULL} & {\tiny {\em pIntTab*}} \\
{\tiny [C]FloatTabConstantDescriptor[/C]} & {\tiny un tableau de taille nulle} & {\tiny NULL} & {\tiny {\em pFloatTab*}} \\
{\tiny [C]DoubleTabConstantDescriptor[/C]} & {\tiny un tableau de taille nulle} & {\tiny NULL} & {\tiny  {\em pDoubleTab*}} \\
{\tiny [C]StringTabConstantDescriptor[/C]} & {\tiny un tableau de taille nulle} & {\tiny un tableau de taille nulle} & {\tiny {\em TableauString*}} \\
{\tiny [C]EntityTabConstantDescriptor[/C]} & {\tiny un tableau de taille nulle} & {\tiny un tableau de taille nulle} & {\tiny {\em pEntityTab*}} \\
{\tiny [C]DescValueSpecTabConstantDescriptor[/C]} & {\tiny un tableau de taille nulle} & {\tiny NULL} & {\tiny {\em pDescValueSpecTab*}} \\
{\tiny [C]IntDomainConstantDescriptor[/C]} & {\tiny un tableau de taille nulle} & {\tiny un tableau de taille nulle} & {\tiny {\em pIntIntervalTab*}} \\
{\tiny [C]FloatDomainConstantDescriptor[/C]} & {\tiny un tableau de taille nulle} & {\tiny un tableau de taille nulle} & {\tiny {\em pFloatIntervalTab*}} \\
{\tiny [C]DoubleDomainConstantDescriptor[/C]} & {\tiny un tableau de taille nulle} & {\tiny un tableau de taille nulle} & {\tiny {\em pDoubleIntervalTab*}} \\
\end{tabular}

\\
*/
//@{ // classe de base + ss_classes typées 

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name La classe de base des descripteurs constants 
\\
Cette classe n'offre pas de services publics, à son niveau. Les services de manipulation de la valeur, entre autres, sont détenus par les sous-classes typées ([C]IntConstantDescriptor[/C], etc.).
*/
//@{
           /// 
class ConstantDescriptor : public Descriptor 
{
public:

// LIFECYCLE
          // Constructeur de base. N'est normalement appelé que par les sous-classes prédéfinies, et non dans les classes définies par l'utilisateur.
  ConstantDescriptor();    // _no swig_                    

// OPERATORS
         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const ConstantDescriptor&) const {return TRUE;};
  Boolean         operator > (const ConstantDescriptor&) const {return TRUE;};

// OPERATIONS

// pour InteractWithUser
   void      ValueToString(char str[50]);
              

/* *@name ACCESS */
// @{
// @}

// INQUIRY

protected:

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Sous-classes typ\'ees de descripteurs constants.
Toutes les sous-classes typées de descripteurs constants possèdent les mêmes services, qui ne se différencient que par le type de la valeur traitée. On les définit de façon commune de la manière suivante :\\

\begin{itemize}
\item Les constructeurs tels que [Y]IntConstantDescriptor()[/Y] construisent le descripteur en lui affectant un type (e.g. INT) et une catégorie (e.g. CONSTANT) pour la sous-classe [C]IntConstantDescriptor[/C]).
\item Les constructeurs tels que [Y]IntConstantDescriptor(UserDescriptorId)[/Y] construisent le descripteur en lui affectant un symbole de classe. 
\item Les constructeurs tels que [Y]IntConstantDescriptor(UserDescriptorId, char*)[/Y] construisent le descripteur en lui affectant un symbole et d'un nom de classe. Le nom est affiché lors de la trace de la création(voir page 'Fonctions globales/RÃŽle de ParseMainArguments').
\item Les constructeurs tels que [Y]IntConstantDescriptor(const IntConstantDescriptor\&)[/Y] sont les constructeurs par copie. L'entité décrite n'est pas copiée dans ce constructeur. La copie de l'entité décrite et son attachement à la copie du descripteur doivent être gérés par ailleurs (voir le constructeur par copie de [C]DescriptedEntity[/C]).
\item Les destructeurs tels que [Y]~IntConstantDescriptor[/Y] détruisent le domaine de valeurs admissibles éventuellement affecté.
\item Les opérateurs tels que [Y]operator = (const IntConstantDescriptor\&)[/Y] sont les opérateurs d'affectation. Seuls la valeur du descripteur à droite du signe =, et son domaine, sont affectés au descripteur à gauche du signe =.

\item void [Y]SetDomain(DescIntValueSpec*)[/Y] dote le descripteur d'un domaine de valeurs entières admissibles, celui passé en argument. {\em idem} pour les autres types. \index{méthodes!SetDomain - Const.Descriptor}
\item void [Y]SetSubDomain(int)[/Y] dote le descripteur d'un domaine vide de valeurs admissibles, puis lui ajoute une valeur (ici entière). {\em idem} pour les autres types. \index{méthodes!SetSubDomain - Const.Descriptor}
\item void [Y]SetSubDomain(int, int)[/Y] dote le descripteur d'un domaine vide de valeurs admissibles, puis lui ajoute un intervalle de valeurs (ici une suite de nombres entiers). {\em idem} pour les autres types. \index{méthodes!SetSubDomain - Const.Descriptor}
\item void [Y]AddSubDomain(int)[/Y] ajoute une valeur (ici entière) au domaine de valeurs admissibles. {\em idem} pour les autres types. Les valeurs numériques doivent être ajoutées en respectant l'ordre croissant. \index{méthodes!AddSubDomain - Const.Descriptor}
\item void [Y]AddSubDomain(float, float)[/Y] ajoute un intervalle de valeurs (ici flottantes) au domaine de valeurs admissibles. Si les intervalles sont ajoutés sans nécessairement respecter l'ordre croissant de leurs bornes inférieures, il faut utiliser la méthode [Y]SortOnLowBound[/Y] des sous-classes de [C]DescValueSpec[/C]. {\em idem} pour les autres types numériques.
\item [Y]GetDomain()[/Y] renvoie un pointeur sur l'instance de [C]DescValueSpec[/C] qui représente le domaine de valeurs admissibles. {\em idem} pour les autres types. \index{méthodes!GetDomain - Const.Descriptor}
\item bool [Y]IsInDomain(int)[/Y] renvoie {\em vrai} si et seulement si un domaine de valeurs admissibles a été spécifié et que la valeur passée en argument est dans ce domaine. \index{méthodes!IsInDomain - Const.Descriptor}
\item bool [Y]IsOutOfDomain(int)[/Y] renvoie {\em vrai} si et seulement si un domaine de valeurs admissibles a été spécifié et que la valeur passée en argument n'est pas dans ce domaine. \index{méthodes!IsOutOfDomain - Const.Descriptor}

\item int [Y]GetIntValue()[/Y] renvoie la valeur entière du descripteur. {\em idem} pour les autres types. \index{méthodes!Get..Value - Const.Descriptor}
\item int [Y]GetDefaultValue()[/Y] renvoie la valeur entière par défaut du descripteur. {\em idem} pour les autres types. \index{méthodes!GetDefaultValue - Const.Descriptor}

\item void [Y]SetValue(int)[/Y] affecte une valeur entière au descripteur. Lève l'exception [Y]OutOfIntDomain[/Y] si la valeur n'est pas admissible. {\em idem} pour les autres types. \index{méthodes!SetValue - Const.Descriptor}
\item void [Y]SetValue(char*)[/Y] fait une copie de la chaîne passée en argument, puis l'affecte au descripteur.
\item void [Y]SetDefaultValue(int)[/Y] établit que la valeur entière donnée en argument est la valeur par défaut du descripteur. {\em idem} pour les autres types. \index{méthodes!SetDefaultValue - Const.Descriptor}
\item void [Y]SetToDefaultValue()[/Y] établit que la valeur (définitive) est la valeur qui a été attribuée par défaut. \index{méthodes!SetToDefaultValue - Const.Descriptor}
\item void [Y]SetToDefaultValue(int)[/Y] établit que la valeur entière donnée en argument est la valeur par défaut du descripteur, et l'affecte à la valeur (définitive) du descripteur. {\em idem} pour les autres types. \index{méthodes!SetToDefaultValue({\em valeur}) - Const.Descriptor}
\end{itemize}
*/
//@{       // ss_classes typées

//-X///////////////X/////////////////////////////X////////////////////////////


/**@name Descripteurs constants à valeur numérique monodimensionnelle 
Par rapport aux autres, ces descripteurs sont simplement caractérisés par une unité.*/
//@{
          ///
class MonoDimensionalNumericalConstantDescriptor : public ConstantDescriptor
{
public:

// @Man: LIFECYCLE
// @{
          // Constructeur de base. N'est normalement appelé que par les sous-classes prédéfinies, et non dans les classes définies par l'utilisateur.
   MonoDimensionalNumericalConstantDescriptor();  // _no swig_    
          // Destructeur.
   ~MonoDimensionalNumericalConstantDescriptor();   // _no swig_  
// @}

// @Man: OPERATORS
// @{
         // Opération  d'affectation. Cette opération n'est en fait pas gérée par DIESE : l'exception [Y]UnAuthorizedAssignement[/Y] est levée.

   const MonoDimensionalNumericalConstantDescriptor&     operator=(const MonoDimensionalNumericalConstantDescriptor&);  // assignment operator
// @}

// OPERATIONS
                  

/**@name ACCESS */
//@{
         /// Affecte une unité au descripteur qui reçoit ce message.\index{méthodes!ClassUnity - MonoDimensionalNumericalConstantDescriptor}
   void              ClassUnity(char*);  // _no swig_
         /// Renvoie le unityaire associé au récepteur.
   char*             ClassUnity() const;  // _no swig_
//@}

protected:
   char*                 mDescriptorUnity;

private:
}; // _end class_

//@} // fin classe

/**@name Descripteurs constants à valeur enti\`ere */
//@{
          ///
class IntConstantDescriptor : public MonoDimensionalNumericalConstantDescriptor
{
public:

// EXCEPTIONS

/**@name LIFECYCLE */
//@{

         ///Affecte le type INT et la catégorie CONSTANT.
   IntConstantDescriptor();    // _no swig_
         /// 
   IntConstantDescriptor(UserDescriptorId);    // _no swig_    
         /// 
   IntConstantDescriptor(UserDescriptorId, char*);    // _no swig_    
         /// 
   IntConstantDescriptor(const IntConstantDescriptor&);    // _no swig_    
         ///
   ~IntConstantDescriptor();    // _no swig_
//@}

/**@name OPERATORS */
//@{
         /// 
   const
   IntConstantDescriptor&      operator=(const IntConstantDescriptor&);
//@} 


/**@name [SC]Opérations sur le domaine[/SC] */ 
//@{  
          /// 
   void SetDomain(DescIntValueSpec*);   
          ///  
   void SetSubDomain(int);  // _no swig_ 
          /// 
   void SetSubDomain(int, int);
          /// 
   void AddSubDomain(int);  // _no swig_ 
          /// 
   void AddSubDomain(int, int);
//@}
                           
/**@name [SC]Opérations sur la valeur[/SC] */
//@{
          //
   void RecordValue(int, int);
          /// 
   int      GetIntValue() const;
          /// 
   int      GetDefaultValue() const;
          /// 
   DescIntValueSpec* GetIntDomain() const;
          /// 
   bool              IsInDomain(int) const;
          /// 
   bool              IsOutOfDomain(int) const;

          /// 
   void     SetToDefaultValue();  // _no swig_ 
          /// 
   void     SetValue(int);
          /// 
     //void     SetDatedValue(int value, int date);
          /// 
   void     SetDefaultValue(int); 
          /// 
   void     SetToDefaultValue(int);

         /// Renvoie {\em vrai} si un corps a été assigné à la méthode de conversion en chaîne de caractères(par un appel à [Y]AssignConverse[/Y].\index{méthodes!HasConverseAssigned} 
   bool              HasConverseAssigned();  // _no swig_
         /// Affecte une fonction de conversion vers une chaîne de caractères.\index{méthodes!AssignConverse}
   void              AssignConverse(char* (*f)(int));  // _no swig_ 
//@}

// INQUIRY

protected:

   int             mValue;
   int             mDefaultValue;
   DescIntValueSpec* mDVSDomain;

   char*           (*ConverseToString)(int); 
   bool            mHasConverseAssigned; 

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name \ldots à valeur flottante */
//@{
           /// 
class FloatConstantDescriptor : public MonoDimensionalNumericalConstantDescriptor
{
public:

// EXCEPTIONS

/**@name LIFECYCLE */
//@{
         /// Affecte le type FLOAT et la catégorie CONSTANT.
   FloatConstantDescriptor();    // _no swig_
         /// 
   FloatConstantDescriptor(UserDescriptorId);    // _no swig_    
         /// 
   FloatConstantDescriptor(UserDescriptorId, char*);    // _no swig_    
         /// 
   FloatConstantDescriptor(const FloatConstantDescriptor&);    // _no swig_    
         ///
   ~FloatConstantDescriptor();    // _no swig_
//@}

/**@name OPERATORS */
//@{
         /// 
   const
   FloatConstantDescriptor&      operator=(const FloatConstantDescriptor&);
//@}

/**@name [SC]Opérations sur le domaine[/SC] */ 
//@{ 
          /// 
   void SetDomain(DescFloatValueSpec*);   
          ///  
   void SetSubDomain(float, float);
          /// 
   void AddSubDomain(float, float);
//@}
                           
/**@name [SC]Opérations sur la valeur[/SC] */
//@{
           //
   void RecordValue(int, float);
          /// 
   float      GetFloatValue() const;
          /// 
   float      GetDefaultValue() const;
          /// 
   DescFloatValueSpec* GetFloatDomain() const;
          /// 
   bool              IsInDomain(float) const;
          /// 
   bool              IsOutOfDomain(float) const;

          /// 
   void     SetToDefaultValue();  // _no swig_ 
          /// 
   void     SetValue(float);
          /// 
   void     SetDefaultValue(float); 
          /// 
   void     SetToDefaultValue(float); 
//@}

// INQUIRY

protected:

   float           mValue;
   float           mDefaultValue;
   DescFloatValueSpec* mDVSDomain;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

 /**@name \ldots à valeur flottante en double précision */
//@{
           /// 
class DoubleConstantDescriptor : public MonoDimensionalNumericalConstantDescriptor 
{
public:

// EXCEPTIONS

/**@name LIFECYCLE */
//@{
         /// Affecte le type DOUBLE et la catégorie CONSTANT.
   DoubleConstantDescriptor();    // _no swig_
         /// 
   DoubleConstantDescriptor(UserDescriptorId);    // _no swig_    
         /// 
   DoubleConstantDescriptor(UserDescriptorId, char*);    // _no swig_    
         /// 
   DoubleConstantDescriptor(const DoubleConstantDescriptor&);    // _no swig_ 
         ///
   ~DoubleConstantDescriptor();    // _no swig_
//@}

/**@name OPERATORS */
//@{
         /// 
   const
   DoubleConstantDescriptor&      operator=(const DoubleConstantDescriptor&);
//@}

/**@name [SC]Opérations sur le domaine[/SC] */ 
//@{  
          /// 
   void SetDomain(DescDoubleValueSpec*);   
          ///  
   void SetSubDomain(double, double); 
          /// 
   void AddSubDomain(double, double);
//@}
                           
/**@name [SC]Opérations sur la valeur[/SC] */ 
//@{ 
          //
   void RecordValue(int, double);
          /// 
   double      GetDoubleValue() const;
          /// 
   double      GetDefaultValue() const;
          /// 
   DescDoubleValueSpec* GetDoubleDomain() const;
          /// 
   bool              IsInDomain(double) const;
          /// 
   bool              IsOutOfDomain(double) const;

          /// 
   void     SetToDefaultValue();  // _no swig_ 
          /// 
   void     SetValue(double);
          /// 
   void     SetDefaultValue(double); 
          /// 
   void     SetToDefaultValue(double); 
//@}

// INQUIRY

protected:

   double           mValue;
   double           mDefaultValue;
   DescDoubleValueSpec* mDVSDomain;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

 /**@name \ldots dont la valeur est une entité */
//@{
           ///
class EntityConstantDescriptor : public ConstantDescriptor
{
public:

// EXCEPTIONS

/**@name LIFECYCLE */
//@{ 
         /// Affecte le type P\_ENTITY et la catégorie CONSTANT. 
   EntityConstantDescriptor();    // _no swig_
         /// 
   EntityConstantDescriptor(UserDescriptorId);    // _no swig_    
         /// 
   EntityConstantDescriptor(UserDescriptorId, char*);    // _no swig_    
         /// 
   EntityConstantDescriptor(const EntityConstantDescriptor&);    // _no swig_
         ///
   ~EntityConstantDescriptor();    // _no swig_
//@}

/**@name OPERATORS */
//@{ 
         /// 
   const
   EntityConstantDescriptor&      operator=(const EntityConstantDescriptor&);
//@}

/**@name [SC]Opérations sur le domaine[/SC] */  
//@{ 
          /// 
   void SetDomain(DescEntityValueSpec*);   
          ///  
   void SetSubDomain(BasicEntity*); 
          /// 
   void AddSubDomain(BasicEntity*); 
 
          /// Spécifie que la valeur du descripteur doit être une instance de la classe dont le symbole est passé en argument. Lève l'exception [Y]MisuseValueClassId[/Y] si on a déjà spécifié une classe. Par défaut d'une telle spécification, la classe de valeur sera O\_BASIC\_ENTITY. \index{méthodes!ValueClassId - Const.Descriptor}        
   void ValueClassId(UserClassId id);

          /// Ajoute un symbole de classe parmi ceux des classes qui peuvent être mère de l'entité. \index{méthodes!AddValueClassId - Const.Descriptor}
   void AddValueClassId(UserClassId id);

          /// Renvoie le symbole de la classe qui est la classe de valeur du descripteur. \index{méthodes!GetAddValueClassId - Const.Descriptor} 
     UserClassId GetValueClassId();

          /// Renvoie la table des symboles des classes de valeur du descripteur.                 
   UserClassIdTab ValueClassIdTab();
//@}
                           
/**@name [SC]Opérations sur la valeur[/SC] */
//@{ 
          //
   void              RecordValue(int, char*);  // + JPR 050805
          /// 
   BasicEntity*      GetEntityValue() const;
          // 
   BasicEntity*      GetDefaultValue() const;
          /// 
   DescEntityValueSpec* GetEntityDomain() const;
          /// 
   bool              IsInDomain(BasicEntity*) const;
          /// 
   bool              IsOutOfDomain(BasicEntity*) const;
 
          /// 
   void     SetValue(BasicEntity*);
          // 
   void     SetToDefaultValue();  // _no swig_
          // 
   void     SetDefaultValue(BasicEntity*); 
          // 
   void     SetToDefaultValue(BasicEntity*);
//@} 

// INQUIRY

protected:

   BasicEntity*         mValue;
   BasicEntity*         mDefaultValue;
   UserClassIdTab       mValueClassIdTab;
   DescEntityValueSpec* mDVSDomain;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

 /**@name \ldots dont la valeur est un tableau d'entités */
//@{
           ///
class EntityTabConstantDescriptor : public ConstantDescriptor
{
public:

/**@name LIFECYCLE */
//@{ 
         /// Affecte le type ENTITY\_TAB et la catégorie CONSTANT.
   EntityTabConstantDescriptor();    // _no swig_
         /// Précise l'incrément pour le tableau valeur du descripteur. Le premier argument est inutilisé.
   EntityTabConstantDescriptor(bool, int tabIncrement);    // _no swig_
         /// 
   EntityTabConstantDescriptor(UserDescriptorId);    // _no swig_    
         /// 
   EntityTabConstantDescriptor(UserDescriptorId, char*);    // _no swig_    
         /// {\bf Important :} la valeur de la copie est la copie de la valeur (pointeurs différents sur deux tableaux des mêmes pointeurs d'entités). 
   EntityTabConstantDescriptor(const EntityTabConstantDescriptor&);    // _no swig_    
         ///
   ~EntityTabConstantDescriptor();    // _no swig_
//@}

/**@name OPERATORS */
//@{ 
         /// 
   const
   EntityTabConstantDescriptor& operator=(const EntityTabConstantDescriptor&);
//@}


/**@name [SC]Opérations sur le domaine[/SC] */
//@{  
          /// Spécifie que les instances d'entités dans le tableau qui est la valeur du descripteur doivent être des instances de la classe dont le symbole est passé en argument. \index{méthodes!ValueClassId - Const.Descriptor}        
   void ValueClassId(UserClassId id);   // _no swig_   

          /// Ajoute un symbole de classe parmi ceux des classes qui peuvent être mère de l'entité. \index{méthodes!AddValueClassId - Const.Descriptor} 
   void AddValueClassId(UserClassId id);   // _no swig_  

          /// Renvoie le symbole de la classe qui est la classe des instances d'entités dans le tableau valeur. \index{méthodes!GetAddValueClassId - Const.Descriptor}
   UserClassId GetValueClassId();

          /// Renvoie la table des symboles des classes de valeur du descripteur.                 
   UserClassIdTab ValueClassIdTab(); 
//@}
 
/**@name [SC]Opérations sur la valeur[/SC] */  
//@{ 
          //
   void              RecordValue(int, char*);  // + JPR 050805
          /// 
   bool              IsInDomain(BasicEntity*) const;
          /// 
   bool              IsOutOfDomain(BasicEntity*) const;
          
          /// 
   pEntityTab*      GetEntityTabValue() const;  
          /// 
   BasicEntity*     GetValueElement(int k) const; 
          /// 
   void             SetValue(pEntityTab*);  
          /// 
   void             SetValueElement(BasicEntity*, int k);
          /// 
   void             AddValueElement(BasicEntity*); 
          /// 
   void             AddValueElements(pEntityTab*);  
          /// 
   void             RemoveValue(BasicEntity*); 
          /// 
   void             ClearValues();

          // 
   pEntityTab*      GetDefaultValue() const;

          // 
   void             SetToDefaultValue();  // _no swig_
          // 
   void             SetDefaultValue(pEntityTab*);
          // 
   void             SetToDefaultValue(pEntityTab*); 
//@}

// INQUIRY

protected:

   pEntityTab*         mValue;
   pEntityTab*         mDefaultValue;
   UserClassIdTab      mValueClassIdTab;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

 /**@name \ldots dont la valeur est une spécification d'ensemble d'entités */
//@{
           ///
class EntitySpecConstantDescriptor : public ConstantDescriptor
{
public:

// EXCEPTIONS

/**@name LIFECYCLE */
//@{ 
         /// Affecte le type P\_ENTITYSPEC et la catégorie CONSTANT. 
   EntitySpecConstantDescriptor();    // _no swig_
         /// 
   EntitySpecConstantDescriptor(UserDescriptorId);    // _no swig_    
         /// 
   EntitySpecConstantDescriptor(UserDescriptorId, char*);    // _no swig_    
         /// 
   EntitySpecConstantDescriptor(const EntitySpecConstantDescriptor&);    // _no swig_
         ///
   ~EntitySpecConstantDescriptor();    // _no swig_
//@}

/**@name OPERATORS */
//@{ 
         /// 
   const
   EntitySpecConstantDescriptor&      operator=(const EntitySpecConstantDescriptor&);
//@}
                           
/**@name [SC]Opérations sur la valeur[/SC] */
//@{ 
          /// 
   EntitySpec*      GetEntitySpecValue() const;
          // 
   EntitySpec*      GetDefaultValue() const;
 
          /// 
   void         SetValue(EntitySpec*);
          // 
   void     SetToDefaultValue();  // _no swig_
          // 
   void     SetDefaultValue(EntitySpec*); 
          // 
   void     SetToDefaultValue(EntitySpec*); 
//@}

// INQUIRY

protected:

   EntitySpec*             mValue;
   EntitySpec*             mDefaultValue;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

 /**@name \ldots dont la valeur est une chaÃ®ne de caractères */
//@{
           ///
class StringConstantDescriptor : public ConstantDescriptor
{
public:

// EXCEPTIONS

/**@name LIFECYCLE */
//@{ 
         /// Affecte le type STRING et la catégorie CONSTANT.
   StringConstantDescriptor();    // _no swig_
         /// 
   StringConstantDescriptor(UserDescriptorId);    // _no swig_    
         /// 
   StringConstantDescriptor(UserDescriptorId, char*);     // _no swig_   
         /// 
   StringConstantDescriptor(const StringConstantDescriptor&);    // _no swig_ 
         ///
   ~StringConstantDescriptor();    // _no swig_
//@}

/**@name OPERATORS */
//@{ 
         /// 
   const
   StringConstantDescriptor&      operator=(const StringConstantDescriptor&);
//@}

/**@name [SC]Opérations sur le domaine[/SC] */  
//@{     
          /// 
   void SetDomain(DescStringValueSpec*);   
          ///  
   void SetSubDomain(char*); 
          /// 
   void AddSubDomain(char*);
//@}
                           
/**@name [SC]Opérations sur la valeur[/SC] */ 
//@{ 
          //
   void       RecordValue(int, char*);
          /// 
   char*      GetStringValue() const;
          /// 
   char*      GetDefaultValue() const;
          /// 
   DescStringValueSpec* GetStringDomain() const;
          /// 
   bool       IsInDomain(char*) const;
          /// 
   bool       IsOutOfDomain(char*) const;

          /// 
   void     SetToDefaultValue();   // _no swig_
          /// 
   void     SetValue(char*);
          /// 
   void     SetDefaultValue(char*); 
          /// 
   void     SetToDefaultValue(char*); 
//@}

// INQUIRY

protected:

   char*             mValue;
   char*             mDefaultValue;
   DescStringValueSpec* mDVSDomain;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

 /**@name \ldots dont la valeur est un tableau d'intervalles d'entiers */
//@{
           ///
class IntDomainConstantDescriptor : public MonoDimensionalNumericalConstantDescriptor 
{
public:

/**@name LIFECYCLE */
//@{ 
         /// Affecte le type INTDOMAIN et la catégorie CONSTANT.
   IntDomainConstantDescriptor();    // _no swig_
         /// 
   IntDomainConstantDescriptor(UserDescriptorId);    // _no swig_    
         /// 
   IntDomainConstantDescriptor(UserDescriptorId, char*);    // _no swig_    
         /// 
   IntDomainConstantDescriptor(const IntDomainConstantDescriptor&);    // _no swig_
         /// 
   ~IntDomainConstantDescriptor();     // _no swig_
//@}

/**@name OPERATORS */
//@{ 
         /// 
   const
   IntDomainConstantDescriptor&      operator=(const IntDomainConstantDescriptor&);       
//@}      
     
/**@name [SC]Opérations sur la valeur[/SC] */   
//@{ 
          /// 
   pIntIntervalTab*      GetIntDomainValue() const;
          /// 
   pIntIntervalTab*      GetDefaultValue() const;

          /// 
   void     SetToDefaultValue();   // _no swig_
          /// 
   void     SetValue(pIntIntervalTab*);
          /// 
   void     AddValueElement(int);   // _no swig_
          /// 
   void     AddValueElement(int, int);
          /// 
   void     InsertValueElement(int, int);
          /// 
   void     SetDefaultValue(pIntIntervalTab*); 
          /// 
   void     SetToDefaultValue(pIntIntervalTab*); 
          /// Renvoie {\em vrai} si et seulement si {\em val} est une des valeurs du descripteur, c'est-à-dire si {\em val} est dans un des intervalles qui définissent la valeur du descripteur. \index{méthodes!IsInValue - IntDomainConstantDescriptor}
   bool     IsInValue(int val) const;
//@}

// INQUIRY

protected:

   pIntIntervalTab*           mValue;
   pIntIntervalTab*           mDefaultValue;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

 /**@name \ldots dont la valeur est un tableau d'intervalles de valeurs flottantes */
//@{
           ///
class FloatDomainConstantDescriptor : public MonoDimensionalNumericalConstantDescriptor 
{
public:

/**@name LIFECYCLE */
//@{ 
         /// Affecte le type FLOATDOMAIN et la catégorie CONSTANT.
   FloatDomainConstantDescriptor();   // _no swig_
         /// 
   FloatDomainConstantDescriptor(UserDescriptorId);    // _no swig_   
         /// 
   FloatDomainConstantDescriptor(UserDescriptorId, char*);   // _no swig_    
         /// 
   FloatDomainConstantDescriptor(const FloatDomainConstantDescriptor&);// _no swig_
         /// 
   ~FloatDomainConstantDescriptor();   // _no swig_
//@}

/**@name OPERATORS */
//@{ 
         /// 
   const
   FloatDomainConstantDescriptor&      operator=(const FloatDomainConstantDescriptor&); 
//@}
                           
/**@name [SC]Opérations sur la valeur[/SC] */   
//@{ 
          /// 
   pFloatIntervalTab*      GetFloatDomainValue() const;    // _no swig_
          /// 
   pFloatIntervalTab*      GetFloatDomainValue(Boolean);
          /// 
   pFloatIntervalTab*      GetDefaultValue() const;

          /// 
   void     SetToDefaultValue();   // _no swig_
          /// 
   void     SetValue(pFloatIntervalTab*);   // _no swig_ 
          /// 
   void     AddValueElement(float, float);
          /// 
   void     InsertValueElement(float, float);
          /// 
   void     SetValue(pFloatIntervalTab*, Boolean);
          /// 
   void     SetDefaultValue(pFloatIntervalTab*); 
          /// 
   void     SetToDefaultValue(pFloatIntervalTab*); 
          /// Renvoie {\em vrai} si et seulement si {\em val} est une des valeurs du descripteur, c'est-à-dire si {\em val} est dans un des intervalles qui définissent la valeur du descripteur. \index{méthodes!IsInValue - FloatDomainConstantDescriptor}
   bool     IsInValue(float val) const;
//@}

// INQUIRY

protected:

   pFloatIntervalTab*           mValue;
   pFloatIntervalTab*           mDefaultValue;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name \ldots dont la valeur est un tableau d'intervalles de flottants double précision */
//@{
           ///
class DoubleDomainConstantDescriptor : public MonoDimensionalNumericalConstantDescriptor 
{
public:

/**@name LIFECYCLE */
//@{ 
         /// Affecte le type DOUBLEDOMAIN et la catégorie CONSTANT.
   DoubleDomainConstantDescriptor();   // _no swig_
         /// 
   DoubleDomainConstantDescriptor(UserDescriptorId);    // _no swig_   
         /// 
   DoubleDomainConstantDescriptor(UserDescriptorId, char*);   // _no swig_    
         /// 
   DoubleDomainConstantDescriptor(const DoubleDomainConstantDescriptor&);// _no swig_
         /// 
   ~DoubleDomainConstantDescriptor();   // _no swig_
//@}

/**@name OPERATORS */
//@{ 
         /// 
   const
   DoubleDomainConstantDescriptor&      operator=(const DoubleDomainConstantDescriptor&); 
//@}
                           
/**@name [SC]Opérations sur la valeur[/SC] */ 
//@{   
          /// 
   pDoubleIntervalTab*      GetDoubleDomainValue() const;    // _no swig_
          /// 
   pDoubleIntervalTab*      GetDoubleDomainValue(Boolean);
          /// 
   pDoubleIntervalTab*      GetDefaultValue() const;

          /// 
   void     SetToDefaultValue();   // _no swig_
          /// 
   void     SetValue(pDoubleIntervalTab*);   // _no swig_ 
          /// 
   void     AddValueElement(double, double);
          /// 
   void     InsertValueElement(double, double);
          /// 
   void     SetValue(pDoubleIntervalTab*, Boolean);
          /// 
   void     SetDefaultValue(pDoubleIntervalTab*); 
          /// 
   void     SetToDefaultValue(pDoubleIntervalTab*); 
          /// Renvoie {\em vrai} si et seulement si {\em val} est une des valeurs du descripteur, c'est-à-dire si {\em val} est dans un des intervalles qui définissent la valeur du descripteur. \index{méthodes!IsInValue - DoubleDomainConstantDescriptor}
   bool     IsInValue(double val) const;
//@}

// INQUIRY

protected:

   pDoubleIntervalTab*           mValue;
   pDoubleIntervalTab*           mDefaultValue;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

 /**@name \ldots dont la valeur est un tableau de chaÃ®nes de caractères */
//@{
           ///
class StringTabConstantDescriptor : public ConstantDescriptor
{
public:
/**@name LIFECYCLE */
//@{ 
         /// Affecte le type ENTITY\_TAB et la catégorie CONSTANT.
   StringTabConstantDescriptor();   // _no swig_
         /// 
   StringTabConstantDescriptor(UserDescriptorId);   // _no swig_    
         /// 
   StringTabConstantDescriptor(UserDescriptorId, char*);   // _no swig_    
         /// {\bf Important :} la valeur de la copie est différente de la valeur du descripteur copie (pointeurs sur deux tableaux de chaînes).
   StringTabConstantDescriptor(const StringTabConstantDescriptor&);// _no swig_   
         ///
   ~StringTabConstantDescriptor();   // _no swig_
//@}

/**@name OPERATORS */
//@{ 
         /// {\bf Important :} la valeur à gauche est la même que la valeur à droite (pointeurs sur deux tableaux de chaînes). 
   const
   StringTabConstantDescriptor&      operator=(const StringTabConstantDescriptor&);     
//@}                        

/**@name [SC]Opérations sur la valeur[/SC] */
//@{ 
          /// 
   bool              IsInDomain(char*) const;
          /// 
   bool              IsOutOfDomain(char*) const;
          
   void              SetToDefaultValue();  // _no swig_  
          // 
   void              SetDefaultValue(TableauString*);  
          // 
   void              SetToDefaultValue(TableauString*);
          /// 
   void              SetValue(TableauString*);   // _no swig_
          /// Ajoute {\em str} à la liste de valeurs, seulement si {\em str} n'y est pas déjà présent.
   void              AddValueElement(char* str);   // _no swig_
          /// 
   void              RemoveValue(char*);   // _no swig_
          /// 
   void              ClearValues();
          // 
   void              DepthDeleteValue();           // + JPR 120116
          // 
   void              DepthDeleteDefaultValue();    // + JPR 120116


          // 
   TableauString*       GetDefaultValue() const;   
          /// 
   TableauString*       GetStringTabValue();    // _no swig_
//@}

// INQUIRY

protected:

   TableauString*         mValue;
   TableauString*         mDefaultValue;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

 /**@name \ldots dont la valeur est un tableau de domaines de valeurs */
//@{
           ///
class DescValueSpecTabConstantDescriptor : public ConstantDescriptor
{
public:

/**@name LIFECYCLE */
//@{ 
         /// Affecte le type DESCVALUESPEC\_TAB et la catégorie CONSTANT.
   DescValueSpecTabConstantDescriptor();    // _no swig_
         /// 
   DescValueSpecTabConstantDescriptor(UserDescriptorId);    // _no swig_    
         /// 
   DescValueSpecTabConstantDescriptor(UserDescriptorId, char*);    // _no swig_    
         /// {\bf Important :} la valeur de la copie est la copie de la valeur (pointeurs différents sur deux tableaux des mêmes pointeurs de [C]DescValueSpec[/C]). 
   DescValueSpecTabConstantDescriptor(const DescValueSpecTabConstantDescriptor&);    // _no swig_    
         ///
   ~DescValueSpecTabConstantDescriptor();    // _no swig_
//@}

/**@name OPERATORS */
//@{ 
         /// 
   const
   DescValueSpecTabConstantDescriptor& operator=(const DescValueSpecTabConstantDescriptor&);
//@}
                           
/**@name [SC]Opérations sur la valeur[/SC] */ 
//@{           
          /// Renvoie toujours {\em vrai}.
   bool               IsInDomain(DescValueSpec*) const;
          /// Renvoie toujours {\em faux}.
   bool               IsOutOfDomain(DescValueSpec*) const;
          /// 
   pDescValueSpecTab* GetDescValueSpecTabValue() const; 
          /// 
   void               SetValue(pDescValueSpecTab*); 
          /// 
   void               AddValueElement(DescValueSpec*); 
          /// 
   void               RemoveValue(DescValueSpec*); 
          /// 
   void               ClearValues();

          // 
   pDescValueSpecTab* GetDefaultValue() const;

          // 
   void               SetToDefaultValue();  // _no swig_
          // 
   void               SetDefaultValue(pDescValueSpecTab*);
          // 
   void               SetToDefaultValue(pDescValueSpecTab*); 
//@}

// INQUIRY

protected:

   pDescValueSpecTab*         mValue;
   pDescValueSpecTab*         mDefaultValue;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

 /**@name \ldots dont la valeur est un tableau d'entiers */
//@{
           ///
class IntTabConstantDescriptor : public ConstantDescriptor   // JPR 090421 
{
public:

/**@name LIFECYCLE */
//@{ 
         /// Affecte le type INT\_TAB et la catégorie CONSTANT.
   IntTabConstantDescriptor();    // _no swig_
         /// 
   IntTabConstantDescriptor(UserDescriptorId);    // _no swig_    
         /// 
   IntTabConstantDescriptor(UserDescriptorId, char*);    // _no swig_    
         /// 
   IntTabConstantDescriptor(const IntTabConstantDescriptor&);    // _no swig_    
         ///
   ~IntTabConstantDescriptor();    // _no swig_
//@}

/**@name OPERATORS */
//@{ 
         /// 
   const
   IntTabConstantDescriptor& operator=(const IntTabConstantDescriptor&);
//@}
                           
/**@name [SC]Opérations sur la valeur[/SC] */ 
//@{           
          /// Renvoie toujours {\em vrai}.
   bool               IsInDomain(int) const;
          /// Renvoie toujours {\em faux}.
   bool               IsOutOfDomain(int) const;
          /// 
   pIntTab*         GetIntTabValue() const;
          /// 
   int              GetIntValueElement(int k) const;  
          /// 
   void               SetValue(pIntTab*);  
          /// 
   void               SetValueElement(int k, int);  
          /// 
   void               AddValueElement(int);
          /// 
   void               ClearValues();

          // 
   pIntTab* GetDefaultValue() const;

          // 
   void               SetToDefaultValue();  // _no swig_
          // 
   void               SetDefaultValue(pIntTab*);
          // 
   void               SetToDefaultValue(pIntTab*); 
//@}

// INQUIRY

protected:

   pIntTab*         mValue;
   pIntTab*         mDefaultValue;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

 /**@name \ldots dont la valeur est un tableau de flottants (simple précision) */
//@{
           ///
class FloatTabConstantDescriptor : public ConstantDescriptor   // JPR 051114 
{
public:

/**@name LIFECYCLE */
//@{ 
         /// Affecte le type FLOAT\_TAB et la catégorie CONSTANT.
   FloatTabConstantDescriptor();    // _no swig_
         /// 
   FloatTabConstantDescriptor(UserDescriptorId);    // _no swig_    
         /// 
   FloatTabConstantDescriptor(UserDescriptorId, char*);    // _no swig_    
         /// 
   FloatTabConstantDescriptor(const FloatTabConstantDescriptor&);    // _no swig_    
         ///
   ~FloatTabConstantDescriptor();    // _no swig_
//@}

/**@name OPERATORS */
//@{ 
         /// 
   const
   FloatTabConstantDescriptor& operator=(const FloatTabConstantDescriptor&);
//@}
                           
/**@name [SC]Opérations sur la valeur[/SC] */ 
//@{           
          /// Renvoie toujours {\em vrai}.
   bool               IsInDomain(float) const;
          /// Renvoie toujours {\em faux}.
   bool               IsOutOfDomain(float) const;
          /// 
   pFloatTab*         GetFloatTabValue() const;
          /// 
   float              GetFloatValueElement(int k) const;  
          /// 
   void               SetValue(pFloatTab*);  
          /// 
   void               SetValueElement(int k, float);  
          /// 
   void               AddValueElement(float);
          /// 
   void               ClearValues();

          // 
   pFloatTab* GetDefaultValue() const;

          // 
   void               SetToDefaultValue();  // _no swig_
          // 
   void               SetDefaultValue(pFloatTab*);
          // 
   void               SetToDefaultValue(pFloatTab*); 
//@}

// INQUIRY

protected:

   pFloatTab*         mValue;
   pFloatTab*         mDefaultValue;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

 /**@name \ldots dont la valeur est un tableau de flottants (double précision) */
//@{
           ///
class DoubleTabConstantDescriptor : public ConstantDescriptor   // JPR 051114 
{
public:

/**@name LIFECYCLE */
//@{ 
         /// Affecte le type DOUBLE\_TAB et la catégorie CONSTANT.
   DoubleTabConstantDescriptor();    // _no swig_
         /// 
   DoubleTabConstantDescriptor(UserDescriptorId);    // _no swig_    
         /// 
   DoubleTabConstantDescriptor(UserDescriptorId, char*);    // _no swig_    
         /// 
   DoubleTabConstantDescriptor(const DoubleTabConstantDescriptor&);    // _no swig_    
         ///
   ~DoubleTabConstantDescriptor();    // _no swig_
//@}

/**@name OPERATORS */
//@{ 
         /// 
   const
   DoubleTabConstantDescriptor& operator=(const DoubleTabConstantDescriptor&);
//@}
                           
/**@name [SC]Opérations sur la valeur[/SC] */ 
//@{           
          /// Renvoie toujours {\em vrai}.
   bool               IsInDomain(double) const;
          /// Renvoie toujours {\em faux}.
   bool               IsOutOfDomain(double) const;
          /// 
   pDoubleTab*         GetDoubleTabValue() const;
          /// 
   double              GetDoubleValueElement(int k) const;  
          /// 
   void               SetValue(pDoubleTab*);  
          /// 
   void               SetValueElement(int k, double);  
          /// 
   void               AddValueElement(double);
          /// 
   void               ClearValues();

          // 
   pDoubleTab* GetDefaultValue() const;

          // 
   void               SetToDefaultValue();  // _no swig_
          // 
   void               SetDefaultValue(pDoubleTab*);
          // 
   void               SetToDefaultValue(pDoubleTab*); 
//@}

// INQUIRY

protected:

   pDoubleTab*         mValue;
   pDoubleTab*         mDefaultValue;

private:
}; // _end class_

//@} // fin classe

//@} // fin sous-classes typées de descripteurs constants

//@} // fin descripteurs constants

// INLINE METHODS


// EXTERNAL REFERENCES

#endif  // _CONSTANTDESCRIPTOR_HH
