//============================================================================
// FILE: CoreEvent.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des événements "moteur"
// C
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _COREEVENT_HH
#define _COREEVENT_HH

// SYSTEM INCLUDES
//
//#include <>                                    //

// PROJECT INCLUDES
//
#include "Event.h"

/**@name CDPFORTEXLes év\'enements pr\'ed\'efinis
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include CoreEvent.h
[/P]
\end{itemize}

Les classes présentées dans cette section sont des sous-classes de [C]Event[/C]. C'est normalement l'utilisateur qui crée des descendants de la classe de base, en fonction de son domaine d'application. Cependant, certains événements apparaissent naturellement dans suffisamment de domaines pour que DIESE fournissse les classes correspondantes : il s'agit de l'initialisation d'un processus continu, de son démarrage et de sa reprise en cas d'interruption. Ces classes ne sont pas d'une utilisation obligatoire : l'utilisateur peut chercher à obtenir des fonctionnalités voisines en les redéfinissant. 
*/ 
//@{

// X///////////////X/////////////////////////////X////////////////////////////

         /**@name  Evénement d'initialisation d'un processus continu.
Un événement de cette classe établit l'état initial du ou des composants du système touchés par le processus.\index{SetInitialState} Cette tâche est réalisée parce que tout événement de cette classe possède, par construction, un et un seul élément dans le tableau de ses conséquences : un couple constitué d'un pointeur sur le processus passé en argument du constructeur et de la directive INIT. Il n'a pas de postconséquence et n'est pas autogénérable. \\

L'insertion dans l'agenda d'une instance de [C]SetInitialState[/C] peut être réalisée en envoyant au processus concerné le message [P]PostInitialisationEvent(aDate)[/P], où {\em aDate} est la date (en nombre d'unités de temps simulé) à laquelle l'état des composants touchés par le processus doit être initialisé. Ce message provoque la construction d'un événement de la présente classe, puis son insertion dans l'agenda :
\begin{verbatim}
[P]  SetInitialState* pEvent = new SetInitialState(aDate, this);
  pCurrentSim->InsertInAgendaEvents(pEvent);[/P]
\end{verbatim}
*/
//@{
        ///
class SetInitialState : public Event
{
public:
          // Constructeur de base.
   SetInitialState();       
          /// Constructeur avec date et processus en jeu.       
   SetInitialState(int, ContinuousProcess*);               
 };
//@}

// X///////////////X/////////////////////////////X////////////////////////////

         /**@name Evénement d'initialisation et de démarrage d'un processus continu.
Un événement de cette classe établit l'état initial du ou des composants du système touchés par le processus et démarre le processus de changement d'état des composants.\index{SetInitialStateAndProceed} Un tel événement possède deux éléments dans le tableau de ses conséquences : un couple avec le pointeur sur le processus passé en argument du constructeur et la directive INIT, un second couple avec le même pointeur et la directive PROCEED.\\

L'insertion dans l'agenda d'une instance de [C]SetInitialStateAndProceed[/C] peut être réalisée en envoyant au processus concerné le message [P]PostInitAndProceedEvent(aDate)[/P], où {\em aDate} est la date (en nombre d'unités de temps simulé) à laquelle l'événement doit intervenir. Ce message demande au processus de construire l'événement (avec les conséquences INIT et PROCEED), puis réalise l'insertion :
\begin{verbatim}
[P]  SetInitialStateAndProceed* pEvent = new SetInitialStateAndProceed(aDate, this);
  pCurrentSim->InsertInAgendaEvents(pEvent);[/P]
\end{verbatim}
*/
//@{
        ///
class SetInitialStateAndProceed : public Event
{
public:
          /// Constructeur avec date et processus en jeu.       
   SetInitialStateAndProceed(int, ContinuousProcess*);               
 };
//@}

// X///////////////X/////////////////////////////X////////////////////////////

         /**@name Continuation de la simulation d'un processus continu.
Un événement de cette classe a pour effet d'envoyer une directive PROCEED au processus.\index{ProceedFromCurrentState} Cette tâche est réalisée parce que tout événement de cette classe possède, par construction, un et un seul élément dans le tableau de ses conséquences : un couple constitué d'un pointeur sur le processus passé en argument du constructeur et de la directive PROCEED. Il n'a pas de postconséquence et n'est pas autogénérable. \\

L'insertion dans l'agenda d'une instance de [C]ProceedFromCurrentState[/C] peut être réalisée en envoyant au processus concerné le message [P]PostProceedEvent(aDate)[/P], où {\em aDate} est la date (en nombre d'unités de temps simulé) à partir de laquelle l'état change. Ce message provoque la construction d'un événement de la présente classe, puis son insertion dans l'agenda :
\begin{verbatim}
[P]  ProceedFromCurrentState* pEvent = new ProceedFromCurrentState(aDate, this);
  pCurrentSim->InsertInAgendaEvents(pEvent);[/P]
\end{verbatim}
*/
//@{
        ///
class ProceedFromCurrentState : public Event
{
public:
          // Constructeur de base.
   ProceedFromCurrentState(); 
          /// Constructeur avec date et processus en jeu.                   
   ProceedFromCurrentState(int, ContinuousProcess*);       
 };
//@}

// X///////////////X/////////////////////////////X////////////////////////////

         /**@name Exécution d'un processus discret.
Un événement de cette classe a pour effet d'envoyer une directive EXEC au processus.\index{ DiscreteExecution} Cette tâche est réalisée parce que tout événement de cette classe possède, par construction, un et un seul élément dans le tableau de ses conséquences : un couple constitué d'un pointeur sur le processus passé en argument du constructeur et de la directive EXEC. Il n'a pas de postconséquence et n'est pas autogénérable. \\

L'insertion dans l'agenda d'une instance de [C]DiscreteExecution[/C] peut être réalisée en envoyant au processus concerné le message [P]PostExecutionEvent(aDate)[/P], où {\em aDate} est la date (en nombre d'unités de temps simulé) à partir de laquelle l'état change. Ce message provoque la construction d'un événement de la présente classe, puis son insertion dans l'agenda :
\begin{verbatim}
[P]  DiscreteExecution* pEvent = new DiscreteExecution(aDate, this);
  pCurrentSim->InsertInAgendaEvents(pEvent);[/P]
\end{verbatim}

*/
//@{
        ///
class DiscreteExecution : public Event
{
public:
          // Constructeur de base.
   DiscreteExecution(); 
          /// Constructeur avec date et processus en jeu.                   
   DiscreteExecution(int, DiscreteProcess*);       
 };
//@}

// X///////////////X/////////////////////////////X////////////////////////////

         // Fonction de démarrage du processus.
/*Cette fonction, exclusivement appelée lors de la gestion d'un événement de classe [C]SetInitialState[/C], installe dans l'agenda un événement de classe [C]ProceedFromCurrentState[/C], dont la date est celle de l'initialisation.
*/
Boolean            InsertProceedFromCurrentStateEvent(Method* pM);
//@}

//@} // fermeture dans Event.h

#endif  // COREEVENT_HH
