//============================================================================
// FILE: Simulation.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe générique des simulations
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _SIMULATION_HH
#define _SIMULATION_HH

// SYSTEM INCLUDES
//
//#include <string>  // 080319
////using std::string;

// PROJECT INCLUDES
//
#include "CoreTypes.h" 
#include "CoreSimTime.h"  
#include "Rng.h" 
  
#ifdef _INTERACTIVE_  
enum ExecMode {GUI_EVENTSTEP, GUI_USERSTEP, GUI_CONTINU, GUI_PAUSE, GUI_STOP, GUI_EXIT}; // _no swig_
#endif

class Event;

/**@name CDPFORTEXLa classe des simulations
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include Simulation.h
[/P] 
\end{itemize}

 Une fois le système modélisé, par l'écriture des sous-classes appropriées d'entités, de descripteurs, de processus et d'événements, la simulation du système repose simplement sur : 
\begin{enumerate}
\item La création d'une instance de la classe [C]Simulation[/C] par un constructeur de la classe, et la liaison de la variable globale [Y]pCurrentSim[/Y]\index{pCurrentSim}\label{pCurrentSim2} à l'adresse de cette instance : 
\begin{verbatim}
[P]  Simulation* pCurrentSim = new Simulation();[/P]
\end{verbatim}
L'horloge du temps simulé, un des attributs de la classe, est mise à zéro.

\item L'affectation explicite de valeurs à certains de ses attributs. 

L'unité pour le temps simulé est fixée par des instructions telles que :
\begin{verbatim}
[P]  pCurrentSim->ClockTimeUnit(SECOND);
  ClockTimeUnitQuantity(1);[/P]
\end{verbatim}

Il est aussi obligatoire d'insérer au moins un événement dans l'agenda\index{agenda (d'événements)}. Par exemple :
\begin{verbatim}
[P]  Event* pMyEvent = new Event();
  pCurrentSim->InsertInAgendaEvents(pMyEvent);[/P]
\end{verbatim}

Les événements sont rangés dans l'agenda selon l'ordre\index{ordre (des événements)} défini dans une fonction d'ordre sur les événements de l'agenda. La fonction standard [Y]BeforeByDatePriority[/Y] (voir page 'Fonctions globales/Fonctions diverses') est affectée par défaut, mais elle peut être surchargée par une fonction écrite par l'utilisateur :
\begin{verbatim}
[P]  pCurrentSim->AssignBeforeInAgendaMethod(myFunction);[/P]
\end{verbatim}

Il est aussi recommandé d'associer à la simulation, par la méthode [Y]SimulatedEntity[/Y], une "entité simulée", \index{entité simulée}\label{entiteSimulee}choisie parmi celles de plus haut niveau d'intégration, c'est-à-dire qui ne sont ni composant ni élément d'une autre entité. Cette entité est normalement celle qui représente le système à simuler. L'intérêt est de disposer de manière simple d'un point d'entrée dans le réseau des entités, à partir duquel on peut "naviguer" en utilisant des services prédéfinis :
\begin{verbatim}
[P]  Entity* pPS = ...;
  pCurrentSim->SimulatedEntity(pPS);
  Entity* pE_ref = pCurrentSim->SimulatedEntity(pPS);
  Entity* pMan = pE_ref->GetComponent(MANAGER);[/P]
\end{verbatim}

 Cette désignation devient nécessaire quand on utilise la version du simulateur équipée pour une {\bf utilisation interactive}.

\item L'envoi à cette instance du message d'exécution de la simulation\index{exécution de la simulation} :
\begin{verbatim}
[P]  pCurrentSim->Run(); // ou pCurrentSim->Run(n);[/P]
\end{verbatim}

où  {\small {\tt n}} est un nombre entier positif ou une variable entière qui peut être [Y]gMaxNbIterations[/Y]\index{gMaxNbIterations}. La méthode [Y]Run[/Y] contrôle la simulation en répétant la série d'instructions suivante, au maximum  {\small {\tt n}} fois et tant que (i) il y a au moins un événement dans l'agenda, (ii) la date de fin de simulation éventuellement spécifiée par l'utilisateur (méthode [Y]SetEndClock[/Y] ou [Y]SetEndDate[/Y]) n'est pas atteinte, (iii) un message explicite d'arrêt de la simulation (méthode [Y]Stop[/Y]) n'a pas été adressé :\\

\Tab4 [a] \Tab4 Dépilement de l'événement de tête de l'agenda, et avancement de l'horloge de la simulation\index{horloge de la simulation} à la date de cet événement. \\
\Tab4 [b] \Tab4 Application des conséquences de l'événement sur le système simulé, en termes de directives (ou actions) sur une liste de processus. Chaque couple {processus, action} est traité dans l'ordre d'apparition dans la liste. Si la précondition\index{précondition} de l'action est satisfaite, celle-ci est réalisée. Sinon, soit l'exécution est abandonnée, soit la même action sur le même processus est à nouveau programmée dans un événement ultérieur spécialement généré à cet effet par DIESE (l'action sur le processus est donc mise en attente). \\
\Tab4 [c] \Tab4 Déclenchement d'une fonction éventuellement définie par l'utilisateur pour coder des conséquences de l'événement qui ne peuvent pas s'exprimer en termes de directives sur des processus. Si une telle fonction (désignée par le terme de {\em postconséquence})\index{postconséquence} existe, elle ne sera déclenchée qu'une fois que tous les processus auront été effectivement traités, que ce soit lors de l'événement initial ou lors des événements reprogrammés à une date ultérieure pour cause de non-satisfaction de la précondition. \\
\Tab4 [d] \Tab4 Dans le cas où l'utilisateur l'a définie, une autre fonction est déclenchée qui génère un événement de la même classe pour une date ultérieure (on parle d'{\em autogénération}). Cette fonctionnalité permet de définir une série d'événements, sans les énumérer tous explicitement. Le nouvel événement généré est inséré dans l'agenda, par la fonction d'ordre évoquée plus haut. \\
\Tab4 [e] \Tab4 Enfin, l'événement dépilé est détruit (récupération de la mémoire qu'il occupait). En ce qui concerne les processus associés, les règles de suppression sont les suivantes. Un processus continu est détruit si l'événement dépilé comportait (normalement) une directive d'arrêt du processus. Un processus discret est détruit si l'événement dépilé comportait une directive d'exécution et que, soit celle-ci a effectivement été réalisée soit le processus a été définitivement abandonné (voir la méthode [Y]AssignPreconditionMethod[/Y] de la classe [C]Process[/C]). Dans les autres cas, le processus n'est pas détruit par le système.

\item Une fois terminée l'exécution de la méthode [Y]Run[/Y], la récupération des résultats de la simulation. DIESE fournit un service d'écriture, sur des supports externes, d'informations sur l'état du système, collationnées en cours de simulation en fonction de directives exprimées avant le lancement du calcul (voir la section 'Les spécifications de sortie').

\item La suppression de l'instance de simulation  : 
\begin{verbatim}
[P]  delete pCurrentSim;[/P] 
\end{verbatim}
Cette suppression entraîne celle des événements restant dans l'agenda et de tous les processus en cours ou en attente, mais pas celle des entités sur lesquelles ils portent, ni des descripteurs et des moniteurs attachés.\\
Et l'appel éventuel aux fonctions [Y]DeleteEntityInstances[/Y] et [Y]DeleteMonitors[/Y]\index{fonctions!DeleteEntityInstances}, pour libérer la mémoire occupée par les entités (et leurs descripteurs) et les moniteurs. Ceci peut être nécessaire, par exemple dans le cas d'un enchaînement d'un grand nombre de simulations portant sur des entités différentes. 

\end{enumerate}
*/ 
//@{
        ///
class Simulation 
{
public:

  /**@name LIFECYCLE */
//@{
          ///Constructeur de base. L'horloge est mise à 0, l'agenda d'événements est vide. La période de simulation commence à l'heure réelle courante. L'unité de temps simulé n'est pas indiquée. La fonction d'ordre sur les événements est [Y]BeforeByDatePriority[/Y]. La valeur maximale de l'horloge est le plus grand entier positif.
  Simulation(); // _no swig_                               
          /// Constructeur par copie profonde (y compris des événements de l'agenda).
  //Simulation(const Simulation&); // _no swig_     
  Simulation(Simulation&); // _no swig_     // pas const parce que go_head 
          /// Destructeur profond (y compris des événements de l'agenda). Comme le destructeur d'événements ne détruit pas les processus associés, c'est dans ce destructeur profond de simulation que les processus créés et non encore détruits sont détruits.                
   ~Simulation(); // _no swig_     
//@}                                

/**@name OPERATORS
\DTab4Pas d'opérateur d'affectation fourni, qui supposerait la coexistence de deux simulations. L'expression suivante a un effet imprévisible : [P]Simulation S = *pCurrentSim;[/P]
*/                           

/**@name [SC]Opérations sur l'agenda d'événements[/SC] */
//@{
          /// Exécute la simulation, en dépilant l'agenda d'événements jusqu'à ce qu'il soit vide, ou bien qu'un certain nombre de dépilements d'événements, donné en argument, aient eu lieu, ou bien que la date de fin de simulation soit dépassée.\index{méthodes!Run}
   void            Run(int);  // _no swig_  \label{defRun}
          /// Exécute la simulation, en dépilant l'agenda d'événements jusqu'à ce qu'il soit vide, ou bien jusqu'à ce que la date de fin de simulation soit dépassée, et tant qu'un message [Y]Stop[/Y] n'a pas été adressé à la simulation en cours.
   void            Run(); // _no swig_ 

          // Exécute le corps d'un dépilement de la simulation.
   void            RunCore(int&);   // _no swig_ 

          /// Renvoie le numéro d'ordre du dépilement en cours.\index{méthodes!GetIterationCounter}
  int              GetIterationCounter() const;  // + JPR 100628
   
          // Attribue le numéro d'identification de la simulation. 
  void             SetNumId(int); 
          /// Renvoie le numéro d'identification de la simulation. 
  int              GetNumId() const;   
  

          /// Après l'envoi de ce message, la simulation en cours s'arrêtera à la prochaine itération de [Y]Run[/Y]. L'agenda d'événements n'est pas modifié et les événements devront être désalloués, explicitement ou par un appel {\small {\tt delete pCurrentSim;}}.\index{méthodes!Stop}   
void               Stop(); 
 
          /// Affiche l'agenda d'événements sur la sortie standard. Les actions sur les processus sont affichées gâce à la fonction [Y]StringActionType[/Y] (voir page 'Fonctions globales/Fonctions diverses').\index{méthodes!DisplayAgendaEvents}
    void           DisplayAgendaEvents(); // _no swig_       
                        
          /// Insère un événement dans l'agenda d'événements, selon un ordre déterminé par l'application de la fonction [Y]AssignBeforeInAgendaMethod()[/Y]  \index{méthodes!InsertInAgendaEvents}.    
   Boolean         InsertInAgendaEvents(Event*);           
          /// Même rôle que la méthode précédent, mais la priorité et l'instant d'occurrence sont ceux fournis en deuxième et troisième arguments.
   Boolean         InsertInAgendaEvents(Event*, int, int); // _no swig_           
          /// Augmente d'une valeur 'delay' la date d'occurrence de l'evenement 'pEvent' et reprogramme celui-ci.\index{méthodes!DelayInAgendaEvents}.  // + JPR 091021  
   Boolean         DelayInAgendaEvents(Event* pEvent, int delay); // _no swig_    

          //  Crée un agenda d'événements vide. 
   void            CreateEmptyAgendaEvents();  // _no swig_ 

          ///  Renvoie l'événement qui vient d'être retiré de l'agenda d'événements, ou NULL si aucun événement n'a encore été retiré.    
   Event*          CurrentlyPoppedEvent() const;  // + JPR 020412 // _no swig_ 

          //  Renvoie TRUE si l'agenda est vide, FALSE sinon.
   Boolean         IsEmptyAgendaEvents(); // _no swig_ 

          /// Place un événement en dernière position dans l'agenda d'événements. Ce placement doit normalement respecter l'ordre qu'établit la méthode [Y]InsertInAgendaEvents[/Y], mais le moteur ne le vérifie pas.\index{méthodes!PushBackToAgendaEvents}
   void             PushBackToAgendaEvents(Event*);
          /// Enlève un événement de l'agenda d'événements. La mémoire allouée à l'événement n'est pas libérée (faire éventuellement un {\small {\tt delete}] sur son pointeur). L'événement doit être dans l'agenda.\index{méthodes!RemoveFromAgendaEvents}
void                RemoveFromAgendaEvents(Event*);
          /// Même effet que la méthode précédente. En outre, si le second argument eet TRUE, désalloue l'événement et ses processus, en profondeur.
 void               RemoveFromAgendaEvents(Event*, Boolean toRemove);
          /// Renvoie le nombre d'événements actuellement présents dans l'agenda.\index{méthodes!AgendaEventSize}
   int              AgendaEventSize();
          /// Affecte à l'instance de simulation en cours une fonction d'ordre sur les événements de l'agenda. La fonction d'ordre peut être le prédicat prédéfini [Y]BeforeByDatePriority[/Y] (voir la page 'Fonctions diverses'). Elle peut aussi être spécifiée par l'utilisateur, et pour cela déclarée et réalisée dans les fichiers [F]UserFunction.h[/F] et [F].cc[/F] (respectivement). Cette fonction d'ordre prend en arguments deux événements quelconques et renvoie TRUE si le premier doit précéder le second dans l'agenda. Le message [Y]AssignBeforeInAgendaMethod[/Y] est normalement envoyé par l'utilisateur dans le programme principal, juste après la création de l'instance de la simulation courante : [P]pCurrentSim->AssignBeforeInAgendaMethod(myFunction);[/P].\index{méthodes!AssignBeforeInAgendaMethod}\index{ordre (des événements)}
   void             AssignBeforeInAgendaMethod(Boolean (* function)
											   (Event* pEv1, Event* pEv2)); // _no swig_ 
         
  // Rendu public pour CoreFunction.cc GetNamedEvent() JPR 020412
  //pEventTab       AgendaEvents() const; 
  pEventList         AgendaEvents() const; // JPR 060803
  
  /// L'evénement de tête devient l'événement courant.
  void              GoHeadInAgenda();  // _no swig_  
  
  /// L'evénement suivant le courant devient l'événement courant.
  void               GoNextInAgenda();  // _no swig_  
  /// Renvoie {\em vrai} s'il existe un evénement courant.
  bool               IsCurrentInAgenda();  // _no swig_ 
  /// Retourne l'evénement courant dans l'agenda.
  Event*             GetCurrentInAgenda();  // _no swig_ 
  /// Enlève de l'agenda son evénement courant. L'evénement qui le suivait devient l'événement courant.
  void               RemoveCurrentInAgenda();  // _no swig_
  
  Event*             DepileAgendaEvents();
//@}

/**@name [SC]Gestion de l'horloge de la simulation[/SC] */
//@{                   
          /// Détermine l'unité de temps naturelle pour l'expression (par l'utilisateur) du temps simulé.
   void     ClockTimeUnit(TimeUnit); // _no swig_ 
          /// Renvoie l'unité de temps naturelle dans l'expression (par l'utilisateur) du temps simulé.
   TimeUnit ClockTimeUnit() const;
          /// Détermine le nombre d'unités de temps naturelles dans l'unité de temps simulé. \index{méthodes!ClockTimeUnitQuantity}
   void     ClockTimeUnitQuantity(int); // _no swig_ 
          /// Renvoie le nombre d'unités de temps naturelles dans l'unité de temps simulé.\index{méthodes!ClockTimeUnitQuantity}
   int      ClockTimeUnitQuantity() const;
  
          /// Renvoie la valeur courante de l'horloge de la simulation. A l'initialisation de la simulation, cette valeur est positionnée à 0. Par la suite et à tout instant, la valeur courante de l'horloge est un nombre entier d'unités de temps simulé depuis le début de la simulation.\index{méthodes!Clock}
   int      Clock() const;    
          // Affecte une valeur à l'horloge de la simulation.
   void     Clock(int); // _no swig_

          /// Affecte une valeur maximale à l'horloge de la simulation. Cette valeur sera aussi la durée de la simulation (puisque la valeur initiale de l'horloge est 0) si l'arrêt de la simulation n'est provoqué que par l'atteinte de cette valeur. 
   void     SetEndClock(int);      
          // Renvoie le nombre de secondes entre le début de la simulation et la fin prévue de la simulation. Cette valeur est calculée par DIESE lors de la définition par l'utilisateur de la date finale maximale de la simulation (méthode [Y]EndDate[/Y](\ldots).
   time_t   EndTime() const; // _no swig_       
          // Renvoie le nombre de secondes entre le 01/01/70 et le début de la simulation. 
   time_t   AbsoluteBeginningTime() const;        
          // Renvoie le nombre de secondes entre le 01/01/70 et la date de fin prévue de la simulation.
   time_t   AbsoluteEndTime() const; // _no swig_ 

          /// Renvoie la structure [C]Date[/C] correspondant à la date de début de la période simulée.\index{méthodes!BeginningDate}
   Date*    BeginningDate() const; 
          /// Détermine la date de début de la période simulée : jour, mois (ou le {\em ShortMonthName}), millésime (par exemple 2001).
   void     SetBeginningDate(int, int, int); // _no swig_      
          /// Détermine la date de début de la période simulée (jour, mois (ou le {\em ShortMonthName}), millésime (par exemple 2001), heure, minutes, secondes).
   void     SetBeginningDate(int, int, int, int, int, int);     
          /// Détermine la date de début de la période simulée (heure, minutes, secondes). Le jour est supposé être le jour où la simulation est effectuée.\index{méthodes!ThisDayBeginningDate}
   void     SetThisDayBeginningDate(int, int, int);   
          /// Renvoie la valeur qu'aura l'horloge lors de la fin prévue de la simulation. Cette valeur pourra ne pas être atteinte si l'agenda d'événements devient vide ou si le nombre maximum de dépilements spécifié par l'utilisateur a été réalisé.\index{méthodes!EndClock}
   int      EndClock() const; 
          /// Renvoie la structure [C]Date[/C] correspondant à la date de fin prévue de la simulation.
   Date*    EndDate() const; 
          /// Détermine la date de fin prévue de la période simulée : jour, mois (ou le {\em ShortMonthName}), millésime (par exemple 2001).
   void     SetEndDate(int, int, int);  // _no swig_     
          /// Détermine la date de fin prévue de la période simulée (jour, mois (ou le {\em ShortMonthName}), millésime (par exemple 2001), heure, minutes, secondes).\index{méthodes!EndDate}
   void     SetEndDate(int, int, int, int, int, int);     
          /// Calcule (et renvoie) la structure {\small {\tt tm}} correspondant à la date simulée courante (c'est-à-dire à la valeur courante de l'horloge).\index{méthodes!CurrentTmDate}
   tm*      CurrentTmDate() const;
//@}


/**@name [SC]Entité simulée, répertoires d'entrées/sorties, etc..[/SC] */
//@{ 

          /// Associe une entité à la simulation, qui devient l'"entité simulée".
   void     SimulatedEntity(BasicEntity*); // _no swig_ 
          /// Renvoie l'entité associée à la simulation.\index{méthodes!SimulatedEntity}
   BasicEntity*     SimulatedBasicEntity();    
          /// 
   Entity*          SimulatedEntity();
          /// Désigne le chemin d'accès aux fichiers de sortie s'il en existe. Le chemin est absolu, ou bien relatif par rapport au répertoire de lancement de la commande d'exécution. Bien noter que dernier n'est pas toujours celui qui contient le module exécutable. Dans le cas notable d'un lancemente utilisation au travers de l'interface graphique, le répertoire de lancement est celui dans lequel est lancée l'interface.\index{méthodes!SetOutDir}
  void               SetOutDir(char*);  // _no swig_ 
         /// Retourne le chemin d'accès aux fichiers de sortie, où une chaine vide s'il n'en existe pas.\index{méthodes!GetOutDir}
  char*              GetOutDir(); // _no swig_

          /// Désigne le chemin d'accès aux fichiers d'entrée s'il en existe. Le chemin est absolu, ou bien relatif par rapport au répertoire de lancement de la commande d'exécution. Bien noter que dernier n'est pas toujours celui qui contient le module exécutable. Dans le cas notable d'un lancemente utilisation au travers de l'interface graphique, le répertoire de lancement est celui dans lequel est lancée l'interface.\index{méthodes!SetInDir}
  void               SetInDir(char*);  // _no swig_ 
         /// Retourne le chemin d'accès aux fichiers d'entrée, où une chaine vide s'il n'en existe pas.\index{méthodes!GetInDir}
  char*              GetInDir(); // _no swig_

  RNG*               GetRandomGenerator(); // _no swig_   // + JPR 090730
 

#ifdef _SWIG_    
          /// Désigne le chemin d'accès au fichier des routines Perl. Le chemin est relatif par rapport au répertoire de lancement de la commande d'exécution, ou bien absolu.\index{méthodes!SetPerlFile}
  void               SetPerlFile(char*);  // _no swig_ 
         /// Retourne le chemin d'accès au fichier des routines Perl, où une chaine vide s'il n'en existe pas.\index{méthodes!GetPerlFile}
  char*              GetPerlFile();  // _no swig_ 
         /// Retourne {\em vrai} si et seulement si le chemin d'accès au fichier des routines Perl n'est pas une chaine vide.\index{méthodes!HasPerlFile}
  Boolean            HasPerlFile(); // _no swig_ 
#endif


#ifdef _INTERACTIVE_       
          // Renvoie {\em vrai} si la mise à jour permanante de l'arbre de structure a été démandée par l'IHM.
  Boolean            IsStructureUpdated(); // _no swig_        
          // Envoie la date courante à l'IHM user. Le paramètre est l'indice d'itération
  void               SendDateToIHM(int); // _no swig_  
  // les 3 méthodes suivantes : déplacé de 'Operations' à ici JPR 110831
          //  
   void            SendStringForWord(char*); // _no swig_ 
          //  
   void            SendStringForWord(char*, Boolean); // _no swig_ 
          //  
   int             InteractWithUser(int&); // _no swig_  
#endif

//@}

  
Boolean              ToBeStopped();


protected:
   //Event*          DepileAgendaEvents();

   int             mNumId;   // + JPR 080204
   int             mIterationCounter;   // + JPR 100628

   TimeDuration    mClockUnit;
   int             mClock; 
   int             mEndClock; 
   time_t          mEndTime;
   time_t          mAbsoluteBeginningTime; 
   time_t          mAbsoluteEndTime;
   Date*           mBeginningDate;
   Date*           mEndDate;
   Date*           mDate;                     // JPR 090127 : unused !!
   BasicEntity*    mSimulatedEntity;
   char*           mInDir;
   //std::string           mInDir;  // JPR 080319
   char*           mOutDir;
#ifdef _SWIG_ 
   char*           mPerlFile;
   Boolean         mHasPerlFile;
#endif

   Boolean         mToBeStopped;

   //pEventTab       mAgendaEvents;
   pEventList      mAgendaEvents;
   Event*          mCurrentlyPoppedEvent; // + JPR 020412
   Boolean         (* BeforeInAgenda) (Event*, Event*);
  
   int             mNbMaxIterations;    // + JPR 000306

   RNG*            mRandomGenerator;      

#ifdef _INTERACTIVE_  
   ExecMode        mExecMode;  // + JPR 000222
   int             mSlowDownFactor;  // + JPR 000222
   int             mDelayBeforeNextPause;  // + JPR 000222
   int             mLastPauseClock;  // + JPR 000222

   int             mStepForUpdateDate;     // + JPR 000303
   int             mNextTimeForUpdateDate;// + JPR 000303

   Boolean         mDisplayingAgenda;     // + JPR 000627
   int             mNbDisplayedEvents;    // + JPR 000627

   Boolean         mStructureUpdating;     // + JPR 000912
#endif

private:
  friend class ContinuousProcess; // pour pouvoir y exploiter l'agenda d'events
}; // _end class_

//@} // fin chapitre

// INLINE METHODS 
                         
// ACCESS


// EXTERNAL REFERENCES

#endif  // _SIMULATION_HH
