//============================================================================
// FILE: C_ActivityPrimitive.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des sous-classes d'activités primitives (C-DIESE)
// 
// AUTHORS: Baali/Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_ACTIVITY_PRIMITIVE_HH
#define _C_ACTIVITY_PRIMITIVE_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "C_Activity.h"
#include "C_CoreTypes.h"

//----------------------------------------------------------------------------
// PRIMITIVE_ACTIVITY classe mère et spécialisations
//----------------------------------------------------------------------------

/**@name Les activités primitives
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_ActivityPrimitive.h} \end{itemize}

\begin{enumerate}
\item {\bf Définition}

Un {\em plan nominal} définit un ensemble d'opérations élémentaires, structuré par des {\em opérateurs de composition}\index{opérateur de composition}, à effectuer sur le système. Ce sont les {\em instructions}\index{instruction}, instances de la classe {\bf PrimitiveActivity}, qui spécifient ces opérations élémentaires (en termes d'effet sur l'état du système) en même temps que les {\em ressources}\index{ressource} nécessaires à leur exécution. Parmi toutes les sous-classes d'{\bf Activity}, seules les {\em instructions} contiennent cette information.\\

La propriété essentielle d'une activité primitive, par rapport aux activités non primitives, est de ne pas posséder d'éléments. Si un {\em plan nominal} est vu comme une arborescence, les instructions en sont les feuilles.\\

On peut créer des instances directes de la classe {\bf PrimitiveActivity}, ou bien d'une de ses sous-classes. \\
\end{enumerate}
*/
//@{

//----------------------------------------------------------------------------
// PRIMITIVE_ACTIVITY classe mère
//----------------------------------------------------------------------------
/**@name Les activités primitives (instructions) 

\bf{descripteurs :} \\

{\scriptsize 
\noindent\begin{tabular}{lllll}
OPERATED\_OBJECT\_SPEC\_ATTRIBUTE & OPERATED\_OBJECT\_SPECIFICATION & NULL  \\
OPERATION\_SPEC\_ATTRIBUTE & OPERATION\_SPECIFICATION & NULL  \\
PERFORMER\_SPEC\_ATTRIBUTE & PERFORMER\_SPECIFICATION & NULL  \\
\end{tabular}
}\\
OPERATED\_OBJECT\_SPECIFICATION : classe des spécifications des objets opérés
OPERATION\_SPECIFICATION : classe des spécifications d'opération et ses ressources propres
PERFORMER\_SPECIFICATION : classe des spécifications d'opérateur

\bf{méthodes :} \\

{\scriptsize 
\noindent\begin{tabular}{lll}
\end{tabular} 
}
*/
//@{
          ///
//----------------------------------------------------------------------------
class PrimitiveActivity : public Activity  // _no swig_
{
public:
     
  /**@name LIFECYCLE */
  //@{
          /// Constructeur de base.
PrimitiveActivity();
          /// Constructeur par copie.
PrimitiveActivity(const PrimitiveActivity&);
          /// Destructeur. Détruit les spécifications et l'opération.
~PrimitiveActivity();
  //@}

/**@name OPERATIONS */
  //@{

           ///
void DeleteInDepth();  // + JPR 110211
           ///
void ClearSpecifications();
           ///
void DeleteSpecifications();  // + JPR 110321
           ///
void ResetDisjunction(int);  // _no swig_
           ///
void ResetDisjunction();  // _no swig_

//TableauInt* OpeningWindow();

int  GetMinBegDateFromDown();
int  GetMaxBegDateFromDown();
int  GetMinEndDateFromDown();
int  GetMaxEndDateFromDown();
           //
Operation* GetAllocatedOperation();
           //
bool IsAllocatedOperationPermanent();
           //
bool IsAllocatedOperationTerminated();
           // 
int GetNbInvolvedEntities();  // _no swig_
           // 
void SetInvolvedIntQuantity(int val);
           // 
int GetInvolvedIntQuantity();
           // 
void SetInvolvedFloatQuantity(float val);
           // 
float GetInvolvedFloatQuantity();
  //@} 

/**@name Tests d'exécutabilité */
//@{

           /// 
bool CheckSonsIfWaiting();  // _no swig_
           /// 
bool CheckSonsIfWaiting(Activity*);  // _no swig_
           /// 
bool CheckSonsIfOpen();  // _no swig_
           /// 
bool CheckSonsIfOpen(Activity*);  // _no swig_
           /// 
bool CheckSonsIfClosed();  // _no swig_
           /// 
bool CheckSonsIfClosed(Activity*);  // _no swig_

           /// 
bool CheckIfSonWaiting(Activity*);  // _no swig_
           /// 
bool CheckIfSonOpen(Activity*);  // _no swig_
           /// 
bool CheckIfSonClosed(Activity*);  // _no swig_

//@} 

/**@name Propagation de changement d'état */
//@{
           ///
void PropagateWaitingToSons(Activity*);  // _no swig_
           /// 
void PropagateOpenToSons(Activity*);  // _no swig_
           /// 
void PropagateClosedToSons(Activity*);  // _no swig_

           /// 
void UpdateIfSonWaiting(Activity*);  // _no swig_
           /// 
void UpdateIfSonOpen(Activity*);  // _no swig_
           /// 
void UpdateIfSonClosed(Activity*);  // _no swig_
           /// 
void UpdateIfSonNoLongerOpenable(Activity*);  // _no swig_

//@}

bool         IsOrMayBeUniqueLast(Activity*);
bool         HasOperationNotBegunAtMaxBegDate();
bool         HasOperationNotBegun();

           //
pEntityTab*  GetMAXSpecTab();

           //
Entity*  GetFirstMAXSpec();


protected :
 
};  // _end class_

//@}  // fin primitive activity

//----------------------------------------------------------------------------
// TRANSFER_ACTIVITY
//----------------------------------------------------------------------------
/**@name Les activités primitives de transfert

Une activivité de transfert est une activité primitive dont l'opération consiste à retirer des éléments, identifiés ou non comme des instances d'{\bf Entity}, d'une partie du système piloté pour les placer dans une autre partie de ce système.

\bf{descripteurs :} \\

{\scriptsize 
\noindent\begin{tabular}{lllll}
DESTINATION_SPEC\_ATTRIBUTE & OPERATED\_OBJECT\_SPECIFICATION & NULL  \\
\end{tabular}
}\\
OPERATED\_OBJECT\_SPECIFICATION : classe des spécifications des objets opérés

\bf{méthodes :} \\

{\scriptsize 
\noindent\begin{tabular}{lll}
\end{tabular} 
}
*/
//@{
          ///
//----------------------------------------------------------------------------
class TransferActivity : public PrimitiveActivity  // _no swig_
{
public:
     
  /**@name LIFECYCLE */
  //@{
          /// Constructeur de base.
TransferActivity();
          /// Constructeur par copie.
TransferActivity(const TransferActivity&);
          /// Destructeur. Détruit les spécifications et l'opération.
~TransferActivity();
  //@}

protected :
 
};  // _end class_

//@}  // fin  activity transfer

//@}  // fin primitive activity et spécialisations

#endif  // _C_ACTIVITY_PRIMITIVE_HH

 
