//============================================================================
// FILE: C_CoreException.h
//
// TYPE: C-LIBRARY
// DESCRIPTION: Pseudo-exception catching functions
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_COREEXCEPTION_HH
#define _C_COREEXCEPTION_HH

// SYSTEM INCLUDES
//


// PROJECT INCLUDES
//
#include "C_CoreTypes.h"


/**@name La gestion des situations d'erreur 
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include C_CoreException.h
[/P] 
\end{itemize} 
Un certain nombre de méthodes (au sens de C++), dans diverses classes, savent reconnaître des situations particulières (en général indésirables) portant sur leurs paramètres ou sur l'état des entités qu'elles manipulent. Dans une telle situation, la méthode en question {\em peut lever une exception}\index{exception}.\\

Le manuel de \Db justifie que le mécanisme standard de reconnaissance et de traitement d'exception de C++ a été remplacé par des appels à des fonctions globales qui réalisent le même effet. On rappelle qu'on distingue deux types de fonctions :\\
\begin{itemize}
\item celles qui provoquent l'arrêt du programme ({\em exceptions} proprement dit) : un message d'erreur est alors toujours affiché ;
\item celles qui ne font qu'afficher un {\em avertissement}, et encore seulement si la variable globale [Y]g_BD_TraceWarnings[/Y] a la valeur {\em vrai}.
\end{itemize} 

 Ces fonctions sont documentées ci-après pour ce qui concerne les exceptions soulevées par le code de \CDb et seulement lui.
 */
//@{

//===========================================================================
/**@name Erreurs de spécification du système décisionnel */
//@{ 

//---------------------------------------------------------------------------
/**@name Défauts de structure (système, plan, activités) 

{\large {\tt BadDecisionSystemSpecification}} : \\
Exception levée lors de l'interprétation du plan d'activités, quand le système décisionnel a été incomplètement spécifié. L'objet manquant est affiché.\\

De manière schématique, le système décisionnel (composant du système de production, l'entité de plus haut niveau d'intégration) est construite de la manièrs suivante, si on utilise la voie de l'externalisation sur un fichier :

[P]
\begin{verbatim}
+ I maConduite pPlan
    //...
;
+ I activitiesResourcesBlock pARB1
	<- C <I><, pPlan>;
    //....
;  
+ I activitiesResourcesBlockSet pARBSet
	<- E <I><,pARB1>;
    //....
;  
+ I strategy pStrategie
	<- C <I><,pARBSet>;
;
+ I monManager pSD             // composant 'stratégie'
	<- C <I><, pStrategie>;    // présent
	//<- C <I><, pStrategie>;  // absent --> BadDecisionSystemSpecification
;
+ I productionSystem pSP
	<- C <I><, pSD>;
    //....
;
ENTITE_SIMULEE <I><,pSP>;
\end{verbatim}
[/P]  

{\large {\tt NotInUniqueRootPlan}} : \\
Avertissement lancé lorsqu'on détecte qu'une activité {\em pA} est élément (à une profondeur quelconque) de deux activités qui sont en position de racine d'un plan (c'est-à-dire une activité qui n'est élement d'aucune autre).\label{defNotInUniqueRootPlan}

[P]
\begin{verbatim}
+ I maConduite1 pPlan1
    + I activityConjunction pA;
    //...
;
+ I maConduite2 pPlan2
    + E <I><, pA>;            // futur NotInUniqueRootPlan
    //...
;
+ I activitiesResourcesBlock pARB1
	<- C <I><, pPlan1>;
    //....
; 
+ I activitiesResourcesBlock pARB2
	<- C <I><, pPlan2>;
    //....
;  
+ I activitiesResourcesBlockSet pARBSet
	<- E <I><,pARB1>;
	<- E <I><,pARB2>;
;
\end{verbatim}
[/P]  

Cette situation est généralement non désirée, et la description du système décisionnel doit alors être révisée quand l'avertissement est lancé. Elle peut aussi correspondre à un cas particulier justifiable.

{\large {\tt StrategyNotReadyForInterpretation}} : \\
Exception levée juste avant l'interprétation du plan d'activités, lorsque celui-ci n'a pas été préparé pour ça : valuer à 1 le descripteur [C]ReadyToRun[/C] :

[P]
\begin{verbatim}
+ I maConduite1 pPlan1
    + I activityConjunction pA;
    readyToRun = 1;
    //...
; 
\end{verbatim}
[/P] 

{\large {\tt PreconditionOnReadyToRun}} : \\
Exception levée lorsqu'on déclare que le plan est prêt à être interprété, alors que l'activité racine ne possède déjà plus la situation SLEEPING ou possède déjà la valeur TRUE pour le descripteur [C]ReadyToRun[/C].\\
Cette situation est évitée en respectant le schéma du haut de cette page.\label{defPreCondOnReadyToRun} 

{\large {\tt BadEarliestLatestDates}} : \\
Exception levée lors de la création d'une activité (de quelque type que ce soit), lorsque : \\
(i) une date au plus tôt (début ou fin) est au-delà de la date au plus tard :
[P]
\begin{verbatim}
+ I primitiveActivity pA;
    minBegDate = 100;
    maxBegDate = 90;            // BadEarliestLatestDates
; 
\end{verbatim}
[/P] 
ou (ii) une date de début (au plus tôt ou au plus tard) est au-delà de la date de fin correspondante : 
[P]
\begin{verbatim}
+ I primitiveActivity pA;
    maxBegDate = 90;
    maxEndDate = 80;            // BadEarliestLatestDates
; 
\end{verbatim}
[/P]

{\large {\tt SpecAttachedToSeveralInstructions}} : \\
Avertissement lancé lorsqu'une spécification décrivant une activité primitive (objet opéré, \ldots) se trouve attachée à plus d'une activité primitive : 
[P]
\begin{verbatim}
+ I operatedObjectSpecification pOOSpec
    //...
;
+ I primitiveActivity pA
    operatedObjectSpecAttribute = pOOSpec;
    //...
;
+ I primitiveActivity pB
    operatedObjectSpecAttribute = pOOSpec; // SpecAttachedToSeveralInstructions
    //...
;
\end{verbatim}
[/P]  
Cette situation peut provoquer des effets indésirables si deux de ces activités sont dans la situation 'ouverte' en même temps. 

{\large {\tt ActivityWithoutOperatedObjectSpec}} : \\
Exception lancée lorsqu'on tente de développer la spécification d'objets opérés d'une activité primitive, alors que l'activité n'en comporte pas et que l'opération en jeu est du type TIME_GRANULATED_OPERATION : 
[P]
\begin{verbatim}
+ I myOperation pOT   // TimeGranulatedOperation
    //...
;
+ I operationSpecification pOTSpec
    operationClassId = MY_OPERATION;
;
+ I primitiveActivity pA
    operationSpecAttribute = pOTSpec;
    //operatedObjectSpecAttribute = pOOSpec;  // future ActivityWithoutOperatedObjectSpec
    //...
;
\end{verbatim}
[/P]   

{\large {\tt ActivityWithoutOperationSpec}} : \\
Exception lancée lorsqu'on tente de développer la spécification d'opération d'une activité primitive, alors que l'activité n'en comporte pas. 
[P]
\begin{verbatim}
+ I myOperation pOT   // spécialisation quelconque d'Operation
    //...
;
+ I operationSpecification pOTSpec
    operationClassId = MY_OPERATION;
;
+ I primitiveActivity pA
    noOperationSpec = 0;                // OperationSpec non facultative
    //operationSpecAttribute = pOTSpec; // future ActivityWithoutOperationSpec
    //...
;
\end{verbatim}
[/P]   

{\large {\tt BadNumberOfReplications}} : \\
Exception lancée lorsqu'on affecte à une activité itérative une valeur minimale pour le nombre d'itérations supérieure à la valeur maximale, ou vice versa : 
[P]
\begin{verbatim}
+ I primitiveActivity pA
    //...
; 
+ I activityIteration pIter;
    + E <I><, pA>;
    minNumberOfReplications = 5;
    maxNumberOfReplications = 4; // BadNumberOfReplications
; 
\end{verbatim}
[/P]  

{\large {\tt BadIterationSpecification}} : \\
Exception levée lorsqu'un plan contient une activité descendante dans plusieurs activités itératives différentes : 
[P]
\begin{verbatim}
+ I primitiveActivity pA
    //...
; 
+ I activityIteration pIter1;
    + E <I><, pA>;
; 
+ I activityIteration pIter2;
    + E <I><, pA>;               // BadIterationSpecification
; 
\end{verbatim}
[/P] 

{\large {\tt ActivityIncludedInDisparateOptional}} : \\
Exception levée quand le plan contient une activité qui est élément de plusieurs activités 'option' ([C]ActivityOptional[/C]) qui n'ont pas la même valeur du descripteur [C]IsOneShot[/C]. Ce descripteur est 1 si et seulement si la condition d'ouverture de l'activité optionnelle n'est testée qu'une fois avant que l'activité 'option' ne soit déclarée soit ouverte soit plus jamais ouvrable. 
[P]
\begin{verbatim}
+ I primitiveActivity pA
    //...
; 
+ I activityOptional pOpt1;
    isOneShot = 0;
    + E <I><, pA>;
; 
+ I activityOptional pOpt2;
    isOneShot = 1;
    + E <I><, pA>;               // ActivityIncludedInDisparateOptional
; 
\end{verbatim}
[/P] 

*/
//@{
void BadDecisionSystemSpecification(char* objet);  // + JPR 110922

void NotInUniqueRootPlan(Activity* A, pEntityTab*);
//void NotAUniqueRootActivity(Activity* A, pEntityTab*);
void StrategyNotReadyForInterpretation(Activity* pAct);  // + JPR 110922
void PreconditionOnReadyToRun(Activity* A, C_StateId oldSituation, int oldReady);
void BadEarliestLatestDates(Activity* A, int mB, int MB, int mE, int ME);
//Exception levée lorsque la spécification d'une activité primitire (PRIMITIVE\_ACTIVITY) ou d'une activité itérative (ACTIVITY\_ITERATION) ne comporte pas de condition d'ouverture ou de condition de fermeture. Il faut spécifier soit des dates au plus tôt et au plus tard, soit un prédicat ayant un corps non égal au pointeur NULL.
//void OpenCloseConditionUnspecified(Activity* A, int openClose);
void SpecAttachedToSeveralInstructions(Entity* newOOSpecValue, Entity* pInst, pEntityTab* pInstList);
void ActivityWithoutOperatedObjectSpec(PrimitiveActivity*);
void ActivityWithoutOperationSpec(PrimitiveActivity*);
void BadNumberOfReplications(ActivityIteration*, int, int);
void BadIterationSpecification(Activity* A);
void ActivityIncludedInDisparateOptional(Activity* pAct);

         // / Exception levée dans le processus de génération des opérations à exécuter si une activité de type 'StrongCostarting' ou 'StrongEquality' possède une composante qui n'est pas une activité primitive.
// void BadElementForUnsplittableSet(); // unused
//@}

//---------------------------------------------------------------------------
/**@name Défaut de spécification d'une activité : objet opéré, opération, 'performer'

Exceptions levées lorsqu'on recherche sans succès le symbole de la classe de référence d'une spécification d'objet opéré, d'opération, ou de performer, ou parce que ni symbole ni nom de classe de référence n'ont été attribués à la spécification. 

{\large {\tt NoInformationForOperatedObjectClass}} : 
[P]
\begin{verbatim}
+ I operatedObjectSpecification pOOSpec
    //entitySpecReferenceClassId = MY_ENTITY; // future NoInformationForOperatedObjectClass
    //...
;
+ I primitiveActivity pA
    operatedObjectSpecAttribute = pOOSpec;
    //...
;
\end{verbatim}
[/P]   

{\large {\tt NoInformationForOperationClass}} : 
[P]
\begin{verbatim}
+ I operationSpecification pOTSpec
    //operationClassId = MY_ENTITY;  // future ...
    //operationClassName = myEntity; // ... NoInformationForOperationClass
    //...
;
+ I primitiveActivity pA
    operationSpecAttribute = pOTSpec;
    //...
;
\end{verbatim}
[/P]    

{\large {\tt NoInformationForPerformerClass}} : 
[P]
\begin{verbatim}
+ I entitySpec pES
    //REFCLASS_ID = MY_ENTITY;  // future NoInformationForPerformerClass
    //...
;
+ I performerSpecification pPerfSpec
    PerformerEntitySpecAttribute = pES;
    //...
;
+ I primitiveActivity pA
    performerSpecAttribute = pPerfSpec;
    //...
;
\end{verbatim}
[/P]    

Une autre situation qui lève ces exceptions est celle où on a attribué un nom de classe de référence, mais que le symbole ne peut pas être inféré parce qu'aucune instance de cette classe n'a encore été créée (typiquement par absence d'invocation de [Y]CreateMetaClass[/Y] pour cette classe) : 
[P]
\begin{verbatim}
void App_InstallMetaClasses() {
  MetaClass* pMC;
  //pMC = CreateMetaClass(MY_ENTITY, "myEntity"); // future ...
  //pMC->SetParentClassId(ENTITY); // ... NoInformationForOperatedObjectClass
  //...
}

+ I operatedObjectSpecification pOOSpec
    entitySpecReferenceClassId = MY_ENTITY;
    //...
;
+ I primitiveActivity pA
    operatedObjectSpecAttribute = pOOSpec;
    //...
;
\end{verbatim}
[/P] 


*/
//@{
void NoInformationForOperatedObjectClass(OperatedObjectSpecification*, char*);
void NoInformationForOperatedObjectClass(OperatedObjectSpecification*);
           
void NoInformationForOperationClass(OperationSpecification*, char*);
void NoInformationForOperationClass(OperationSpecification*);
           
void NoInformationForPerformerClass(PerformerSpecification*, char*);
void NoInformationForPerformerClass(PerformerSpecification*);

//@}

//---------------------------------------------------------------------------
/**@name Changement non autorisé de l'état d'une activité 

{\large {\tt PreconditionOnState}} : \\
Exception levée lorsque le système est amené à tenter une transition d'état interdite.\label{defPreCondOnState}\\

Les seules transitions autorisées sont les suivantes :\\

\begin{tabular}{lll}
                           & {\tiny avant} & {\tiny après} \\ \hline\\
{\tiny TurnToWaiting} & {\tiny SLEEPING}                & {\tiny WAITING} \\
                      & {\tiny activité itérée CLOSED}  & {\tiny WAITING} \\
{\tiny TurnToOpen}    & {\tiny WAITING}                 & {\tiny OPEN} \\
{\tiny TurnToClosed}  & {\tiny OPEN}                    & {\tiny CLOSED} \\
                      & {\tiny activité itérative OPEN avec activité itérée SLEEPING ou WAITING}                                                                     & {\tiny CLOSED} \\
                      & {\tiny activité itérative avec activité itérée OPEN après la fin de l'itérative}                                                             & {\tiny CLOSED} \\
{\tiny TurnToNoLongerOpenable} & {\tiny WAITING}        & {\tiny NO_LONGER_OPENABLE} \\
\end{tabular}
*/
//@{
void PreconditionOnState(Activity* A, C_StateId oldValue, C_StateId newValue);
//@}
 
//---------------------------------------------------------------------------
  /**@name Terminaison anormale de l'interprétation du plan 

{\large {\tt PlanFailure}} : \\
Exception levée lorsque la mise en oeuvre du plan d'activités ne peut plus être poursuivie sans violer une contrainte définisant une des activités du plan. Cet échec survient dans les circonstances suivantes : \label{defPlanFailure}\\
\begin{itemize}
\item le plan global ne peut pas être initialisé (passage à WAITING impossible) ;
\item une activité en attente (WAITING) a une condition d'ouverture plus jamais satisfiable ;
\item au terme de sa fenêtre d'ouverture, une activité a son opération non encore commencée (typiquement, sa condition explicite de faisabilité n'a pas pu être satisfaite) ;
\item lorsqu'une activité ouverte (OPEN) voit son opération terminée (TERMINATED), sa condition explicite de fermeture n'est pas satisfaite ;
\item alors qu'on est avant la fin de la période de simulation, une activité ouverte (OPEN) a sa condition de fermeture plus jamais satisfiable ;
\item on a atteint le nombre maximum d'itérations d'une activité itérative ([C]ActivityIteration[/C]) avant sa date de fin au plus tôt
\end{itemize}

{\large {\tt UpdateStatusFailedInLoop}} : \\
Exception levée lorsque la méthode [Y]UpdateStatus[/Y] part en boucle sur des appels à [Y]Validate[Waiting Open Closed][/Y], vraisemblablement du fait d'une spécification de plan erronée (inconsistante, incomplète).

{\large {\tt ActivityStillOpenAtSimulationEnd}} : \\
Avertissement lancé par la méthode [Y]CheckStillOpenAtSimulationEnd[/Y], lorsqu'une activité est encore ouverte au moment de la fin de la simulation. Cette situation est soit justifiable, soit la trace d'une spécification incorrecte du plan.
*/




//@{

void PlanFailure(Activity* A, char* str);

void PlanFailure(Activity* A, Activity* B, char* str);

void UpdateStatusFailedInLoop(C_StateId, Activity*, C_UpDown, Activity*); 

void ActivityStillOpenAtSimulationEnd(Activity*);
//@}

//---------------------------------------------------------------------------
/**@name Erreurs sur une activité de transfert 

{\large {\tt TransferActivityAssignedWithNotTransferOperation}} : \\
Exception levée quand on attache à une activité de transfert une spécification d'opération définie par une classe d'opération autre qu'une opération de transfert :

[P]
\begin{verbatim}
+ I operationSpecification pOTSpec
    operationClassId = MY_UNIT_OPERATION;  // non opération de transfert
;
+ I transferActivity pA
    operationSpecAttribute = pOTSpec;  // future TransferActivityAssignedWithNotTransferOperation
    //...
;
\end{verbatim}
[/P] 

{\large {\tt NonElementTransferActivityFromNotCapacitatedResource}} : \\
Avertissement émis quand on tente un transfert à partir d'une source qui n'est pas une ressource à capacité et que le transfert ne porte pas sur les éléments.\\

En effet, un transfert peut avoir comme source :\\
\begin{itemize}

\item une ressource à capacité. On transfère alors :
\begin{itemize}

    \item soit tout ou partie de la contenance d'une ressource à capacité. \\
	La ressource (une [C]SingleCapacitatedResource[/C], par exemple) possède un descripteur [C]ResourceCapacitySupply[/C] qui pointe sur un objet représentant sa capacité en tant que 'source'. Cette capacité ([C]ResourceCapacity[/C]) est caractérisée par un descripteur [C]DescriptorIdAttribute[/C] dont la valeur est le symbole d'un descripteur numérique de la ressource (par exemple [C]Volume[/C]), et par une valeur maximale pour ce descripteur : la contenance (par exemple 100), et une valeur courante entre 0 et la valeur maximale)\\
	L'instance de [C]TransferInformation[/C] attachée à l'activité de transfert désigne ce descripteur (par son descripteur [C]TransferQuantityId[/C] : la nature de ce qu'on transfère) et la quantité à transférer à partir de cette ressource 'source'.

    \item soit des éléments (entre 0 et le nombre d'éléments) de cette entité. \\
    Pour que le moteur fasse cela, le descripteur [C]DescriptorIdAttribute[/C] de la [C]ResourceCapacity[/C] attachée à la ressource doit alors avoir la valeur C_ELEMENTS (au lieu de, par exemple, VOLUME).
\end{itemize}

\item une ressource qui n'est pas à capacité : 
\begin{itemize}

\item le transfert ne peut alors porter que sur les éléments (entre 0 et le nombre d'éléments) de cette ressource. Pour que le moteur fasse cela, le descripteur [C]TransferInformationAttribute[/C] de l'activité de transfert doit être une [C]TransferInformation[/C] dont le descripteur [C]TransferQuantityId[/C] a la valeur C_ELEMENTS.\\
Naturellement dans ce cas, aucune [C]ResourceCapacity[/C] n'entre en jeu pour la 'source'.
\end{itemize}

\end{itemize}

La présente exception est levée dans le dernier cas, si C_ELEMENTS n'est pas la valeur
de [C]TransferQuantityId[/C] de la [C]TransferInformation[/C] de l'activité.
*/
//@{ 

void TransferActivityAssignedWithNotTransferOperation(PrimitiveActivity*, UserClassId);

void NonElementTransferActivityFromNotCapacitatedResource(pEntityTab* ooList);

//@}
         // / Exception levée quand ...
void ChangingElementTabOfActivityWithinUpdatingStatusLoopOnElements(Activity* pAct, pEntityTab);

//@} // fin erreurs / activités


//===========================================================================
/**@name Erreurs sur les opérations */
//@{ 
//---------------------------------------------------------------------------
/**@name Opération mal construite  

{\large {\tt BadProportionalityMode}} : \\
Exception levée lors du développement d'une spécification de ressource propre d'opération, lorsque le mode de proportionalité est incompatible avec la fonction de sélection.\\
Les incompatibilités sont :\\
- PROPORTIONAL avec la fonction de sélection MAX\\
- PROPORTIONAL avec la fonction de sélection ALL\\

\begin{verbatim}
+ I operationResourceSpecification pOtRscSpec
    entitySpecReferenceClassId = MY_TOOL;
    proportionalityMode = PROPORTIONAL;
    selectorFctId = ALL;
;
+ I myUnitOperation pOT
    requiredResources << pOTRscSpec;  // future BadProportionalityMode
    //...
;
\end{verbatim}
[/P] 

{\large {\tt BadEntityResourceClass}}  \\
Exception levée lors de l'initialisation du processus d'exécution d'une [C]QuantityGranulatedOperation[/C] ou d'une [C]TimeGranulatedOperation[/C], lorsqu'une des entités-ressource trouvées dans la spécification d'objet opéré de l'instruction n'est pas instance (directe ou indirecte) de la classe-entité de l'opération.

[P] 
\begin{verbatim}
+ I operatedObjectSpecification pOtRscSpec
entitySpecReferenceClassId = MY_ENTITY; // future BadEntityResourceClass
    selectorFctId = ANYONE;
    selectorFctArg = 1;
;
+ I myUnitOperation pOT
    operatedEntityId = OTHER_ENTITY;   // non sous-classe de MY_ENTITY
    //...
;
\end{verbatim}
[/P] 

{\large {\tt NoStateTransitionProcedure}} : \\
Exception levée lorsqu'on n'a pas spécifié de procédure de transition d'état.\\

Que le code soit ou non géréré par Solfege, il doit faire appel à la méthode [C]AssignBody[/C], argumenté (i) par le symbole de classe de la méthode prédéfinie [Y]StateTransitionProcedure[/Y] et (ii) par un pointeur sur une fonction de signature appropriée :

[P] 
\begin{verbatim}
bool myUnitOperation_stateTransitionProcedure_Body(EntityMethod* pM) {
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  Entity* pOtherEntity = pM->GetEntityArgValue(OPERATION_TARGET);
  bool result = TRUE;
  // ... le code du changement d'état
  return result;
}

MyUnitOperation::MyUnitOperation() {
  SetMetaClass(MY_UNIT_OPERATION);
  //SetParentClassId(UNIT_GRANULATED_OPERATION); // pour memoire
  AddInstanceToClass();
  SetIntConstValue(OPERATED_ENTITY_ID, OTHER_ENTITY);
  AssignBody(STATE_TRANSITION_PROCEDURE, 
             myUnitOperation_stateTransitionProcedure_Body)
  // ...
  ;
};
\end{verbatim}
[/P] 

{\large {\tt NoInformationForOperationQuantityDescId}} : \\
Exception levée lorsqu'on recherche sans succès le symbole de la classe du descripteur sur lequel est basée la vitesse d'une opération de type QUANTITY_GRANULATED. 

[P] 
\begin{verbatim}
MyIntQuantityOperation::MyOperation() {
  SetMetaClass(MY_INT_QUANTITY_OPERATION);
  //SetParentClassId(INT_QUANTITY_GRANULATED_OPERATION); // pour memoire
  // ...
  //SetIntConstValue(QUANTITY_DESCRIPTOR_ID, // future ...
  //                 UN_NOMBRE);             // NoInformationForOperationQuantityDescId
  SetIntConstValue(INT_QUANTITY_SPEED, 4);
  // ...
  ;
};

\end{verbatim}
[/P] 

{\large {\tt BadActivityClosingThreshold}} : \\
Exception levée lorsqu'on tente d'affecter un seuil de fermeture inférieur à 1 pour une opération de type QUANTITY_GRANULATED ou TIME_GRANULATED.\\

Le seuil de fermeture est le degré de progression de l'effet de l'opération (descripteur [C]AchievementDegree[/C]) en dessous duquel le passage de l'activité à l'état CLOSED est interdit.\\

Or, pour les [C][Int Float]QuantityGranulatedOperation[/C] et les [C]TimeGranulatedOperation[/C], si le degré de progression augmente de manière monotone, le complet changement d'état est réalisé dès le premier pas de progression. Par exemple,\\
- si l'effet sur l'entité de classe [C]A[/C] est de changer la valeur de son descripteur [C]UnEtat[/C] de 0 à 1, \\
- et si la vitesse de l'opération est basée sur un autre descripteur [C]Surface[/C], de valeur 10\\
- et si la vitesse est 2\\
alors,\\
- [C]UnEtat[/C] passe de 0 à 1, et le degré de progression passe à 0.2, au premier pas\\
- puis le degré de progression passe à 0.4, 0.6, ... 1.0 lors des 4 pas suivants.\\

Il en résulte qu'arrêter une telle opération avant qu'elle ne soit terminée génèrerait une incohérence : un effet complet ([C]UnEtat[/C] = 1) mais une progression incomplète ([C]AchievementDegree[/C] < 1).\\

Avec une [C]UnitGranulatedOperation[/C], le degré de progression avance à chaque 'pas' d'un ratio 'nombre d'unités N'/'nombre total d'unités', et l'état ne change à chaque 'pas' que sur N unités identifiables. C'est pourquoi arrêter l'opération avant qu'elle ne soit terminée ne génère pas l'incohérence.
 

{\large {\tt PermanentOperationWithMoreThanOneOperatedObject}} : \\
Avertissement lancé quand une opération de type QUANTITY_GRANULATED ou TIME_GRANULATED, souhaitée permanente, a été spécifiée comme portant sur plus d'un seul objet opéré. Le mécanisme de progression de l'opération est tel que les objets après le premier déclaré ne seront pas opérés.\\

En effet, l'effet ([Y]StateTransitionProcedure[/Y]) de l'opération permanente est alors répété tant que l'activité n'est pas passée à l'état CLOSED, par ses propres spécifications. Et il est toujours répété sur le premier objet opéré puisque, par convention, le degré de progression de l'opération reste à 0  et qu'il n'ya donc pas lieu de passer à l'objet opéré suivant.
 
*/
//@{
void BadProportionalityMode(Operation*, C_ProportionalityMode, char*);

void BadEntityResourceClass(Operation*, UserClassId, UserClassId);  

void NoStateTransitionProcedure(Operation*);

void NoInformationForOperationQuantityDescId(Operation* pOt);   // + JPR 100705

void BadActivityClosingThreshold(Operation*, float);

void PermanentOperationWithMoreThanOneOperatedObject(Operation*);

//@}

//---------------------------------------------------------------------------
/**@name Exécution anormale ou en échec 

{\large {\tt InterruptedAtActivityEndDate}} : \\
Avertissement émis quand une opération est interrompue avant sa terminaison (valeur donc inférieure à 1 pour le descripteur [C]AchievementDegree[/C]), parce que la date de fermeture au plus tard de l'activité primitive a été atteinte.

{\large {\tt InterruptedAtSimulationEndDate}} : \\
Exception levée quand une opération est interrompue avant sa terminaison (valeur donc inférieure à 1 pour le descripteur [C]AchievementDegree[/C]), parce que la date de fin de la simulation a été atteinte.\\
 
Ces deux situations sont généralement non désirées, et la description du système décisionnel doit alors être révisée quand l'avertissement est lancé. Elle peut aussi correspondre à un cas particulier justifiable.

*/
//@{ 
void InterruptedAtActivityEndDate(int, Operation*); 

void InterruptedAtSimulationEndDate(int, Operation*); 
//@}

//---------------------------------------------------------------------------
/**@name Erreurs sur la spécification de ressources propres 

{\large {\tt NoInformationForOperationResourceClass}} : \\
Exception levée lorsqu'on recherche sans succès le symbole de la classe de référence d'une spécification de ressource propre d'opération, parce que ni symbole ni nom de classe de référence n'ont été attribués à la spécification.  
[P]
\begin{verbatim}
+ I operationResourceSpecification pOTRscSpec
    //operationClassId = MY_ENTITY;  // future ...
    //operationClassName = myEntity; // ... NoInformationForOperationResourceClass
    //...
;
+ I myOperation pOT
    requiredResources << pOTRscSpec;
    //...
;
\end{verbatim}
[/P]      

Une autre situation qui lève cette exception est celle où on a attribué un nom de classe de référence, mais que le symbole ne peut pas être inféré parce qu'aucune instance de cette classe n'a encore été créée (typiquement par absence d'invocation de [Y]CreateMetaClass[/Y] pour cette classe) (voir la page 'Défaut de spécification d'une activité').  
*/
//@{ 
void NoInformationForOperationResourceClass(OperationResourceSpecification*, char*);

void NoInformationForOperationResourceClass(OperationResourceSpecification*);
//@}

//@} // fin opérations

//===========================================================================
/**@name Erreurs sur les ressources */
//@{ 
//---------------------------------------------------------------------------
/**@name Ressource mal construite 

{\large {\tt ResourceNotYetAttachedToARB}} : \\
Exception levée lorsqu'on recherche le bloc activités-ressources auquel est attachée une ressource qu'on immobilise ou qu'on mobilise, alors qu'on n'a pas réalisé cet attachement.\\

Lors de cette (im)mobilisation, le moteur de simulation programme un nouvel événement [C]MakeInstructionList[/C] au même instant (si non déjà présent pour une autre raison). Or le service qui effectue la programmation ([Y]PostMakeInstructionListEvent[/Y]) réclame le bloc activités-ressources en argument.\\

Dans la situation d'exception, vérifier dans la base de connaissances ou les fichiers de données externes que la ressource désignée a été attachée à un pool ([C]ResourcePoolConnjunction[/C]) et que le pool a été attaché à une instance de [C]ActivitiesResourcesBlock[/C].

{\large {\tt InstantiateHeterogeneousResourceNotAllowedInAllocation}} : \\
Exception levée lorsqu'on tente d'expanser une spécification d'ensemble d'entités ([C]EntitySpec[/C]) de mode INSTANTIATE sur une sous-classe de [C]HeterogeneousAggregatedResource[/C]. \\

Lors de l'allocation de cette ressource à une activité, l'expansion de spécification d'ensemble d'entités peut générer plusieurs instances de la ressource agrégée (par exemple {\small {\tt pAggRsc1}}, {\small {\tt pAggrRsc2}}, {\small {\tt pAggrRsc3}}, si on a adressé le message [P]pES->IntQuantity(3);[/P] à la spécification). \\
Or, le code du constructeur de la ressource agrégée a le droit de faire référence, comme composants, à toujours les mêmes instances de ressources (par exemple, {\small {\tt pRsc1}} et {\small {\tt pRsc2}}). Dans cette éventualité, la même instance d'entité ({\small {\tt pRsc1}}, par exemple) est composant de plus d'une instance d'entités (ici {\small {\tt pAggRsc1}}, {\small {\tt pAggrRsc2}}, {\small {\tt pAggrRsc3}}), ce qui est interdit, par convention, dans la couche \BD.
*/
//@{  
void ResourceNotYetAttachedToARB(Resource*);

void InstantiateHeterogeneousResourceNotAllowedInAllocation(EntitySpec*);  // + JPR 111027
//@} 

/**@name Appel à des services non disponibles 

{\large {\tt CapacityCheckingOnDiscreteStateResource}} : \\
Avertissement lancé lors d'un test de disponibilité lié à la capacité d'une ressource, alors que cette ressource est "à état discret".

{\large {\tt DiscreteStateCheckingOnCapacitatedResource}} : \\
Avertissement lancé lors d'un test de disponibilité lié à la capacité d'une ressource, alors que cette ressource est "à état discret".

*/
//@{  
void CapacityCheckingOnDiscreteStateResource(Resource*);  // + JPR 120130 
 
void DiscreteStateCheckingOnCapacitatedResource(Resource*);  // + JPR 120130
//@} 

//---------------------------------------------------------------------------
/* *@name Erreur sur la spécification pour une opération 

Exception levée lors du développement d'une spécification de ressource propre d'opération : aucune spécification d'ensemble d'entités n'a été indiquée.

Cette situation est normalement impossible : par construction, le descripteur ResourceEntitySpecAttribute a une valeur par défaut qui est une ES nouvellement instanciée
*/
// @{
void NullEntitySpecInOperationResourceSpec(OperationResourceSpecification*);  
// @} 

//---------------------------------------------------------------------------
/**@name Mobilisation/immobilisation suspecte 

{\large {\tt BadStatusForMobilizationImmobilization}} : \\
Avertissement émis lorsqu'on tente ...
\begin{itemize}
\item d'immobiliser une ressource indisponible (entre autres causes : par une précédente immobilisation), \\
\item ou au contraire de mobiliser une ressource disponible (entre autres causes : par une précédente mobilisation). 
\end{itemize}
*/
//@{
void BadStatusForMobilizationImmobilization(Resource*, char*);   
//@}
//---------------------------------------------------------------------------
/**@name Erreurs sur la définition d'une contrainte   

{\large {\tt UnexpectedClassIdForInconsistencyCondition}} : \\
Exception levée dans deux situations différentes :
\begin{itemize}
\item on a créé une condition d'inconsistance liée aux ressources (instance de la classe [C]ResourceSharingViolationCondition[/C]), et on ne lui a pas donné, comme valeur du descripteur [C]ConstrainedEntityClassId[/C] le symbole d'une sous-classe de ressources ([C]Resource[/C]).
\item on a créé une condition d'inconsistance liée aux activités (instance de la classe [C]ActivityInconsistencyCondition[/C]), et on ne lui a pas donné, comme valeur du descripteur [C]ConstrainedEntityClassId[/C] le symbole d'une sous-classe de d'activité primitive ([C]PrimitiveActivity[/C]).
\end{itemize}   

{\large {\tt VoidARIC}} : \\
Exception levée lorsque le moteur de simulation rencontre une [C]ActivitiesResourcesInconsistentCommitment[/C] 
\begin{itemize}
\item sans corps attribué à sa méthode [Y]HoldingCondition[/Y]
\item et sans éléments de type [C]ActivityInconsistencyCondition[/C] ni [C]ResourceSharingViolationCondition[/C].  
\end{itemize}

L'absence d'éléments est une situation envisageable lorsqu'on intègre dans la [Y]HoldingCondition[/Y] la totalité de la sémantique de la contrainte.

{\large {\tt NoOccurrenceCrossDomain}}  \\
Exception levée lorsque le moteur de simulation rencontre une [C]ResourceSharingViolationCondition[/C] ou une [C]ActivityInconsistencyCondition[/C] sans valeur pour le descripteur [C]OccurrenceCrossDomainAttribute[/C].

{\large {\tt VoidOccurrenceCrossDomain}} : \\
Exception levée lorsque le moteur de simulation rencontre un [C]OccurrenceCrossDomain[/C] sans élément (de type [C]OccurrenceDomain[/C]).  \\

Pour localiser les deux dernières erreurs, la construction correcte se fait typiquement de la manière suivante :  
  
[P]
\begin{verbatim}
MyViolationCondition::MyViolationCondition() {
  // sous-classe de RESOURCE_SHARING_VIOLATION_CONDITION
  // ...
  Entity* pOccurrenceDomain = new MyDomainViolation();
  Entity* pOccurrenceCrossDomain = new OccurrenceCrossDomain();
  pOccurrenceCrossDomain->AddElement(pOccurrenceDomain);

  SetEntityConstValue(OCCURRENCE_CROSS_DOMAIN_ATTRIBUTE,
					  pOccurrenceCrossDomain);
};
\end{verbatim}
[/P]  

L'absence du [P]SetEntityConstValue[/P] provoque la première erreur.\\
L'absence du [P]AddElement[/P] provoque la seconde erreur.\\
*/
//@{
void UnexpectedClassIdForInconsistencyCondition(Entity* pCondition, int newValue, char* str);

void VoidARIC(Entity* pARIC);

void NoOccurrenceCrossDomain(Entity* pIC);

void VoidOccurrenceCrossDomain(Entity* pIC, Entity* pOCD);
//@}
//---------------------------------------------------------------------------
/**@name Erreurs spécifiques aux capacités  

{\large {\tt InexistantResourceCapacity}} : \\
Exception levée dans les situations suivantes :
\begin{itemize}
\item pour opérer le changement d'état par une opération de transfert ([C]<typed>TransferOperation[/C]), le moteur de simulation trouve que la ressource 'source' n'a pas été dotée d'une capacité multiple (une instance de [C]MultipleResourceCapacity[/C], valeur du descripteur [C]ResourceCapacitySupply[/C]) ;
\item on recherche sans succès, dans une capacité multiple ([C]MultipleResourceCapacity[/C]), un élément de capacité (instance de [C]ResourceCapacity[/C]) avec une valeur donnée pour le descripteur [C]DescriptorIdAttribute[/C]. On rappelle que cette valeur est le symbole d'un descripteur de la ressource ou bien le symbole C_ELEMENTS.
\end{itemize}

{\large {\tt BadCapacityId}} : \\
Exception levée dans les situations suivantes : 
\begin{itemize}
\item on consomme une ressource à capacité entière, et la capacité de la ressource n'est pas une instance, directe ou indirecte, de la classe [C]IntegerResourceCapacity[/C] ;
\item on consomme une ressource à capacité 'flottante', et la capacité de la ressource n'est pas une instance, directe ou indirecte, de la classe [C]FloatResourceCapacity[/C].
\end{itemize}

{\large {\tt IntegerStockLevelOutOfBounds}} : \\
Exception levée lorsqu'on tente de modifier le niveau courant d'un stock à niveau discret (dont la capacité est une instance de [C]IntegerStockCapacity[/C]), en dépassant les bornes inférieures ou supérieures autorisées (les descripteurs [C]IntLowerLevel[/C] et [C]IntUpperLevel[/C] de la capacité). 

{\large {\tt FloatStockLevelOutOfBounds}} : \\
Exception levée lorsqu'on tente de modifier le niveau courant d'un stock à niveau continu (dont la capacité est une instance de [C]FloatStockCapacity[/C]), en dépassant les bornes inférieures ou supérieures autorisées (les descripteurs [C]IntLowerLevel[/C] et [C]IntUpperLevel[/C] de la capacité).
*/
//@{ 
void InexistantResourceCapacity(MultipleResourceCapacity*, Resource*, UserDescriptorId, char*);    

void BadCapacityId(Resource*, UserDescriptorId, char*); 
 
      //Exception levée lors de l'établissement de la demande en capacité d'une opération de transfert, quand on a spécifié l'option C_ELEMENTS alors que la ressource en jeu n'est pas un stock atomique discret. 
//void BadTransferInfoForNonStock(Resource*);  // unused

void IntegerStockLevelOutOfBounds(SingleStock*, UserDescriptorId, int, int, int); 

void FloatStockLevelOutOfBounds(SingleStock*, UserDescriptorId, float, float, float);
//@}

//@} // fin ressources


//===========================================================================
/**@name Erreurs sur buts et critères 

{\large {\tt NoGoalEvaluationByActivity}} : \\
Exception levée lorsqu'on a besoin d'évaluer l'atteinte d'un but par la mise en oeuvre d'une activité,       mais que la méthode [Y]IntFutureValueByActivity[/Y] de l'instance de [C]IntGoal[/C] ou [Y]IFloatFutureValueByActivity[/Y] de l'instance de [C]FloatGoal[/C] n'a pas été dotée d'un corps.\\ \label{defNoGoalEvaluationByActivity} 

Cette assignation d'un corps se fait explicitement par un appel [P]pGoal->AssignBody([INT FLOAT]_FUTURE_VALUE_BY_ACTIVITY, f);[/P] , où {\small {\tt f}} est une fonction spécifique à l'application, renvoyant un entier ou un flottant, et argumenté par un pointeur sur une [C]EntityMethod[/C]. Par exemple :

[P]
\begin{verbatim}
int my_intFutureValueByActivity_Body(EntityMethod* pM){
  int result = D_INTMAX;
  Entity* pGoal = pM->DescribedEntity();
  BasicEntity* pSingle = pM->GetEntityArgValue(SINGLE_ACTIVITY_SET_ARGUMENT);
  pIntIntervalTab* domain = pM->GetIntDomainArgValue(TIME_DOMAIN_ARGUMENT);
  // ... le calcul de la valeur d'atteinte du but
  return result;  
};
\end{verbatim}
[/P]
  

{\large {\tt NoGoalEvaluationByActivityList}} : \\
Exception levée lorsqu'on a besoin d'évaluer l'atteinte d'un but par la mise en oeuvre d'un jeu d'activités, mais que la méthode [Y]IntFutureValueByActivityList[/Y] de l'instance de [C]IntGoal[/C] ou [Y]IFloatFutureValueByActivityList[/Y] de l'instance de [C]FloatGoal[/C] n'a pas été dotée d'un corps.\\ \label{defNoGoalEvaluationByActivityList} 

Cette assignation d'un corps peut se faire de deux manières :
\begin{itemize}
\item en donnant une valeur au descripteur [C]ActivityActionMode[/C] du but (C_TAKE_MIN, C_TAKE_MEAN, C_TAKE_MAX) : c'est alors une des fonctions prédéfinies [Y]min mean max]StandardIntGoal_[int float]FutureValueByActivityList_Body[/Y] qui est assigné comme corps ;
\item explicitement par un appel [P]pGoal->AssignBody([INT FLOAT]_FUTURE_VALUE_BY_ACTIVITY_LIST, f);[/P]
, où {\small {\tt f}} est une fonction spécifique à l'application, de signature identique aux fonctions prédéfinies nommées ci-dessus. Par exemple :

[P]
\begin{verbatim}
int my_intFutureValueByActivityList_Body(EntityMethod* pM) {
  int result = D_INTMAX;
  Entity* pGoal = pM->DescribedEntity();

  IntEntityMethod* pMsub = (IntEntityMethod*)(pGoal->GetMethod(INT_FUTURE_VALUE_BY_ACTIVITY));
  if(! pMsub->HasBodyAssigned())
    NoGoalEvaluationByActivity((Goal*)pGoal);   // exception

  pEntityTab* pSingleList = pM->GetEntityTabArgValue(SINGLE_LIST_ARGUMENT);
  pIntIntervalTab* domain = pM->GetIntDomainArgValue(TIME_DOMAIN_ARGUMENT);
  int nbSingle = pSingleList->size();
  for (int k=0;k<nbSingle;k++) {
    BasicEntity* pSingle = pSingleList->get_nth(k);
    pMsub->SetArgValue(SINGLE_ACTIVITY_SET_ARGUMENT, pSingle);
    pIntIntervalTab* domainCopy = new pIntIntervalTab(*domain);
    pMsub->SetArgValue(TIME_DOMAIN_ARGUMENT, domainCopy);
    int result_k = pMsub->Body(pMsub);
    // ... le calcul de la contribution de cette activité
    // à la valeur d'atteinte du but par le jeu
  }
  return result;  
};
\end{verbatim}
[/P]
\end{itemize}
*/
//@{
void NoGoalEvaluationByActivity(Goal* pGoal);

void NoGoalEvaluationByActivityList(Goal* pGoal);

//@} // fin buts et critères


//===========================================================================
/**@name Erreurs sur les fichiers  

{\large {\tt InvalidLowerBoundForPrediction}} : \\
Exception levée lorsqu'on cherche à réaliser une prédiction sur la base d'un fichier de données séquentielles, avec une borne inférieure invalide.\label{nvalidBoundForPrediction} 
  
[P]
\begin{verbatim}{
  // ...
  SumUpPredictionIntFieldValue(MY_FILE, A_FLD, 0, 3);  // au moins 1
  // ...  
}
\end{verbatim}
[/P]   

{\large {\tt InvalidUpperBoundForPrediction}} : \\
Exception levée lorsqu'on cherche à réaliser une prédiction sur la base d'un fichier de données séquentielles, avec une borne supérieure invalide.  
  
[P]
\begin{verbatim}{
  // ...
  pManager->SetIntConstValue(PREDICTION_SCOPE, 10); // 10 enregistrements prévisibles
  SumUpPredictionIntFieldValue(MY_FILE, A_FLD, 1, 15);  // au plus 10
  // ...  
}
\end{verbatim}
[/P]

{\large {\tt InvalidLowerBoundForMemory}} : \\
Exception levée lorsqu'on cherche à réaliser une mémorisation sur la base d'un fichier de données séquentielles, avec une borne inférieure invalide.\label{invalidBoundForMemory}   
  
[P]
\begin{verbatim}{
  // ...
  SumUpMemoryIntFieldValue(MY_FILE, A_FLD, -1, 3);  // au moins 0
  // ... 
} 
\end{verbatim}
[/P]    

{\large {\tt InvalidUpperBoundForMemory}} : \\
Exception levée lorsqu'on cherche à réaliser une mémorisation sur la base d'un fichier de données séquentielles, avec une borne supérieure invalide.  
  
[P]
\begin{verbatim}{
  // ...
  pManager->SetIntConstValue(PREDICTION_SCOPE, 10); // 10 enregistrements mémorisés
  SumUpPredictionIntFieldValue(MY_FILE, A_FLD, 0, 10);  // au plus 9
  // ...  
}
\end{verbatim}
[/P]     

{\large {\tt InvalidIndexForPrediction}} : \\
Exception levée lors d'un traitement avec un indice invalide, sur la table de prédiction basée sur un fichier de données séquentielles.        

{\large {\tt InvalidIndexForMemory}} : \\
Exception levée lors d'un traitement avec un indice invalide, sur la table de mémorisation basée sur un fichier de données séquentielles. 


*/ // + JPR 081027
//@{
void InvalidLowerBoundForPrediction(BufferizedDataFile* pF, int lb, int xmin);

void InvalidUpperBoundForPrediction(BufferizedDataFile* pF, int ub, int xmax);

void InvalidIndexForPrediction(BufferizedDataFile* pF, int index);

void InvalidLowerBoundForMemory(BufferizedDataFile* pF, int lb, int xmin);

void InvalidUpperBoundForMemory(BufferizedDataFile* pF, int ub, int xmax);

void InvalidIndexForMemory(BufferizedDataFile* pF, int index);
//@}


//===========================================================================
/* *@name Messages d'erreur divers */
// @{ 

         // / Exception levée lorsqu'on utilise une fonction de création de système avec une liste d'arguments erronnée, pour la classe dont le symbole est passé en argument.\label{defBadActConstArg}
//void BadSystemConstructionArguments(C_EntityClassId id);

         // / Exception levée lorsqu'on tente d'instancier une entité à partir d'une chaîne de caractère qui ne correspond pas à un nom de classe d'entité.\label{BadStringForNewEntity}
//void BadStringForNewEntity(char *s);

         // / Exception levée lors d'une copie d'un [C]InstructionSet[/C]. Le second argument indique la cause d'erreur.
//void BadInstructionSetCopy(InstructionSet*, char*); 

// @} // fin divers


//@} // fin fonction erreur




#endif  // _C_COREEXCEPTION_HH
