//============================================================================
// FILE: C_Resource.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des ressources
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_RESOURCE_HH
#define _C_RESOURCE_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "Entity.h"

#include "C_CoreTypes.h"
#include "C_ResourceCapacity.h"
#include "C_ResourceSet.h"
#include "C_TransferInformation.h"

// EXTERNAL REFERENCES

/**@name Les ressources

Une ressource est une entité matérielle requise par une activité primitive ([C]PrimitiveActivity[/C]) pour qu'elle soit exécutable.\\

L'identification des ressources en jeu dans un système de production permet de distinguer trois ensembles de connaissances :
\begin{itemize}
\item celles intrinsèques à l'activité, telle son rôle dans le contrôle du système ;
\item celles intrinsèques aux ressources, telles les causes, les mécanismes et la dynamique de la disponibilité/indisponibililité ;
\item celles sur la relation activités/ressources, telles la réquisition de ressources par une activité et les conditions de l'allocation des ressources à uneactivité.
\end{itemize}

La présente section documente les deux derniers ensembles de connaissances.\\

Du point de vue structurel, une ressource est une entité ([C]Entity[/C]) comme une autre, au sens où elle possède des descripteurs ([C]Descriptor[/C]) et des méthodes ([C]Method[/C]). Au-delà, elle est le sujet ou l'objet de mécanismes spécifiques dans le fonctionnement global du système :

\begin{itemize}
\item l'alternance entre sa disponibilité et son indisponibilité, causée soit par sa dynamique propre (disponibilité intrinsèque) ou bien résultant de sa mobilisation par une activité et de son degré de partageabilité avec d'autres activités ;
\item une fois disponible, son allocation (ou non) à une ou plusieurs activités qui la requièrent en un instant donné, compte tenu de contraintes posées sur cette allocation.
\end{itemize}

Les liens en haut de la présente page documentent ces differents aspects :
\begin{itemize}
\item la structure, en distinguant les ressources atomiques des agrégées,
\item les différents critères de disponibilité d'une ressource
\item l'expression de contraintes sur la disponibilité
\item l'expression de la capacité d'une ressource à satisfaire une demande quantitative
\item les structures de données qui portent le scénario de l'allocation
\end{itemize}
 */
//@{   // fermeture dans C_AvailabilityConstraint

// /X///////////////X/////////////////////////////X////////////////////////////
//
// RESSOURCE : CLASSE DE BASE
//
// /X///////////////X/////////////////////////////X////////////////////////////
/**@name La classe de base des ressources 
\begin{itemize} \item Directive d'inclusion : 
[P]\#include C_Resource.h[/P]
\end{itemize}

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]Resource[/C]} & {\tiny RESOURCE} & {\tiny resource} & {\tiny } & {\tiny }
\end{tabular}

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny C} & {\tiny int} & {\tiny [C]IsSingle[/C] IS_SINGLE isSingle} & {\tiny 1} & {\tiny {\em vrai} seulement pour les [C]SingleResource[/C]}\hline
{\tiny C} & {\tiny int} & {\tiny [C]IsPermanent[/C] IS_PERMANENT isPermanent} & {\tiny 1} & {\tiny  {\em faux} si ressource permanente}\hline
{\tiny V (1) (2)} & {\tiny int} & {\tiny [C]AvailabilityStatus[/C] AVAILABILITY_STATUS availabilityStatus} & {\tiny 1} & {\tiny état booléen de disponibilité}
\end{tabular}

{\large {\tt Ressource permanente}} : Dans le tableau ci-dessus, on qualifie de permanente un un objet, une personne, un dispositif, présent matériellement dans le système durant toute la période de simulation.A l'inverse, une ressource construite occasionnellement pour une durée limitée est non permanente. C'est le cas par exemple d'une équipe de travail constitué en appelant un sous-ensemble du pool des ouvriers présents. cette équipe est dissoute après la terminaison de l'activité, et ses éléments à nouveau disponibles pour constituer une équipe différente. Lors de leur dissolution, le moteur de simulation récupère la mémoire allouée à la ressource non permanente.

  {\bf ** Moniteurs :}\\

\begin{tabular}{ccl}
 & {\tiny } & {\tiny événement} & {\tiny }\hline
 & {\tiny (1)} & {\tiny SetValue} & {\tiny propage vers les resources 'conteneur' l'indisponibilité de la ressource décrite}\\
 & {\tiny (2)} & {\tiny GetValue} & {\tiny . [C]OperatedObject[/C] : renvoie la disponibilité de la [C]ResourceEntity[/C] \newline . autre [C]SingleResource[/C] dotée d'une [C]AvailabilityTimetable[/C] : renvoie la disponibilité sur cette base \newline . autre [C]SingleResource[/C] : renvoie la valeur de [C]AvailabilityStatus[/C] \newline . ressource agrégée : renvoie {\em vrai} si et seulement si tous les éléments ou composants sont disponibles}
\end{tabular}
  
  {\bf Méthodes :}

*/ 
//@{        
        ///
class Resource : public Entity
{
public:
          // Constructeur de base.
  Resource();
          // Constructeur par copie.
  Resource(const Resource&);
          // Destructeur.
  ~Resource();

  
/**@name [SC]Accès aux ressources "conteneurs" ou incluses[/SC] */
//@{  
         /// Renvoie l'instance de [C]ResourceSet[/C] qui contient la ressource réceptrice comme élément à une profondeur quelconque et qui est composant d'un (et d'un seul par construction) [C]ActivitiesResourcesBlock[/C].
  ResourceSet* GetARBResourcePool();

         /// Renvoie l'instance de [C]ResourceSet[/C] qui contient la ressource réceptrice comme élément à une profondeur quelconque et qui est composant d'un (et d'un seul par construction) [C]OperatingSystem[/C].
  ResourceSet* GetOSResourcePool();

         /// Renvoie la liste des ressources (simples ou agrégées) qui sont éléments ou composants à une profondeur quelconque de la ressource réceptrice de ce message. \index{méthodes!GetInvolvedResourceTab}
  pEntityTab* GetInvolvedResourceTab();
//@}

/**@name [SC]Tests de disponibilité[/SC] \\\\
Pour les services ci-dessous, lorsque la ressource est une ressource agrégée, le test est réalisé récursivement sur les éléments ou composants, et il est vérifié si aucun test n'échoue.
*/
//@{  
         /// Renvoie {\em vrai} si la ressource satisfait la contraintes de disponibilité liée au temps qui lui est attachée.\index{méthodes!CheckTimeAvailabilityConstraint}
Boolean CheckTimeAvailabilityConstraint();  // _no swig_ 

         /// Renvoie {\em vrai} si la ressource satisfait toutes les contraintes de disponibilité liées à son état qui lui sont attachées.\index{méthodes!CheckStateAvailabilityConstraints}
Boolean CheckStateAvailabilityConstraints();  // _no swig_ 

         /// Renvoie {\em vrai} si la ressource satisfait toutes les contraintes de disponibilité liées à sa capacité qui lui sont attachées.\index{méthodes!CheckCapacityAvailabilityConstraints}
Boolean CheckCapacityAvailabilityConstraints();  // _no swig_  

         /// Renvoie {\em faux} si la valeur du descripteur [C]AvailabilityStatus[/C] est {\em faux}. Sinon, renvoie {\em vrai} si la ressource satisfait toutes les contraintes de disponibilité liées à son état, et la contrainte de disponibilité liée au temps, qui lui sont attachées. \index{méthodes!IsTimeAndStateAvailable}
Boolean IsTimeAndStateAvailable();  // _no swig_
//@}

};

//@}// fin classe de base Resource

// suite dans C_SingleResource.h



#endif  // _C_RESOURCE_HH
