//============================================================================
// FILE: File.h
//
// TYPE: C++ CLASS
// DESCRIPTION: entête de la classe générique des fichiers
// 
// AUTHORS: Feller (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _FILE_HH
#define _FILE_HH

// ---------------
// SYSTEM INCLUDES
// ---------------

// ----------------
// PROJECT INCLUDES
// ----------------
#include "CoreTypes.h"

typedef int UserFileId;  // transféré de CoreTypes.h 021022

class LexicalInputFile;
class SerialDataFile;
class SequentialDataFile;
class FactualDataFile;
class OutputFile;

/**@name CDPFORTEXLes fichiers
Le comportement du système simulé est déterminé par deux ensembles d'informations :
\begin{itemize}
\item {\large {\tt Les connaissances générales}} sur la structure du système et les processus en jeu. Pour l'essentiel, ces informations sont codées dans les constructeurs des classes d'entités et de processus. Les constructeurs sont eux-mêmes invoqués dans des modules supervisés par un module particulier, connu sous le nom de programme principal (c'est la fonction [Y]main()[/Y]). Ce programme fixe en outre tout ou partie des directives de la simulation : typiquement et sommairement, les instructions de lecture des données en entrée, le lancement de la simulation et la récupération des résultats. Toutes ces informations sont compilées, c'est-à-dire qu'elles ne sont pas modifiables par l'utilisateur. Elles constituent, solidairement, le simulateur.

\item {\large {\tt Les données spécifiques à un cas d'étude}} faisant l'objet d'une simulation du comportement du système. Ces données sont modifiées par l'utilisateur d'une simulation à l'autre. Puisque le simulateur compilé est fermé, ces données spécifiques sont placées en dehors de lui, sur des fichiers externes.
\end{itemize}

A tout moment du développement d'un simulateur, le modélisateur doit décider si une connaissance est générale à son domaine ou bien spécifique à un cas d'étude.\\
C'est notamment le cas de la structure du système à simuler : faut-il la fixer par le code des constructeurs du système et de ses composants ou éléments ? Ou bien laisse-t-on l'utilisateur du simulateur la décrire au cas par cas dans un fichier externe ?\\
C'est aussi le cas des critères d'analyse du comportement du système : l'évolution de quelles variables d'état est un critère général ? L'évolution de quelles autres n'est intéressant que dans un cas d'étude particulier.\\

Noter que chaque connaissance ne fait normalement l'objet d'une déclaration externe que si le développeur a choisi de ne pas les inclure dans la partie compilée. Si des informations sont placées à la fois dans la partie compilée et sur fichier externe, cette dernière forme annulera et remplacera la première.\\ 

Chaque type de connaissance "externalisée" fait l'objet d'un {\em fichier}\index{fichier} différent, avec un format spécifique. On peut les classer en trois ensembles :

\begin{enumerate} 

\item les données en entrée de la simulation,  qui sont elles-mêmes de quatre types :\index{données en entrée de la simulation} 

\begin{itemize}
\item
{\large {\tt les paramètres du système}}, c'est-à-dire la valeur de constantes intervenant, n'importe où, dans le code du simulateur : par exemple, la latitude du lieu d'étude, un paramètre d'ajustement d'une courbe de croissance, etc. Ils sont déclarés dans un {\em fichier de paramètres} unique, exploité en tout début du programme principal par un appel à [Y]ParseParameterFile[/Y]. Cette exploitation génère la création d'instances (une par paramètre) de la classe [C]Parameter[/C]. 
\noLF
Les valeurs affectées (celles figurant dans le fichier externe) sont alors accessibles par les services [Y]Get..ParameterValue[/Y].\index{paramètres du système}\index{fichier de paramètres}\index{fonctions!ParseParameterFile}

\item 
{\large {\tt les aspects structurel, fonctionnel et dynamique}} du système simulé : quelles entités sont composants ou éléments de telle autre, quelle est la valeur initiale de tel descripteur de telle entité ; quels événements programme-t-on {\em ex ante}, quand et avec quelle priorité, quels processus sont en jeu dans ces événements, sur quelles entités vont-ils porter.  Ces déclarations sont faites dans un unique {\em fichier de la structure} présenté dans le lien ' Les fichiers en entrée de type déclaratif', et exploité par un appel à [Y]ParseStructureFile[/Y] avant l'invocation de la méthode [Y]Run[/Y] (que celle-ci soit programmée dans la fonction [Y]main()[/Y] ou déclarée dans le fichier des directives de simulation (voir ci-dessous)). 
\noLF
La lecture et l'interprétation de ces informations provoque l'invocation de services de \D, notamment les constructeurs des classes de l'application, les services d'affectation de valeurs, etc.\index{structure du système simulé}\index{fichier de la structure}\index{fonctions!ParseStructureFile}

\item 
{\large {\tt les paramètres de la simulation}} elle-même : les dates de début et de fin, l'entité qui tient le rôle d'"entité simulée", le nom du répertoire où sont classés les différents fichiers externes, etc. Ces informations sont déclarés dans un fichier unique exploité par un appel à [Y]ParseSimulationFile[/Y], avant l'invocation de la méthode [Y]Run[/Y], de quelque manière qu'elle soit réalisée.\index{spécification de la simulation}\index{fonctions!ParseSimulationFile}

\item
{\large {\tt la dynamique de l'environnement}} du système simulé. L'état de l'environnement évolue dans le temps, et on dispose d'un vecteur de valeurs en plusieurs instants de la simulation. Une option d'externalisation est que chaque vecteur constitue un {\em enregistrement} d'un fichier\index{enregistrement (d'un fichier)} ; une simulation accède alors à une séquence d'enregistrements. L'enregistrement peut comporter certains {\em champs}\index{champ (d'un enregistrement)} qui définissent le moment auquel les valeurs se rapportent. Ces champs peuvent etre ignorés, ou bien remplir la fonction de clé d'accès\index{clé (d'un enregistrement)} pour un accès indexé aux données, ou bien encore permettre une vérification de concordance de temps entre la donnée accédée et l'instant courant de la simulation.\\
\noLF
Les descripteurs dynamiques de l'environnement figurent dans des {\em fichiers en série d'enregistrements} externes.\index{fichiers en série d'enregistrements} A chacun, le développeur du simulateur fait correspondre une instance de la classe [C]SerialDataFile[/C], typiquement en début de simulation, avant l'appel au [Y]Run[/Y].
\end{itemize}

De façon générale, les informations dans le {\em fichier des paramètres du système} peuvent être exploitées dans les autres fichiers en entrée. Par exemple, le fichier de déclaration de la structure du système pourra faire référence à une valeur de paramètre pour fixer le nombre d'éléments à ajouter à une certaine entité. De même, ce fichier pourra contenir des informations utiles à l'exploitation d'un fichier en série d'enregistrements (par exemple, le nom physique du fichier).

\item {\large {\tt Les "spécifications de sortie"}} : les "sorties" de la simulation sont une trace sélective du comportement du système simulé. C'est l'utilisateur qui établit la sélection sous la forme de spécifications écrites dans un fichier externe au simulateur. Ce fichier de spécifications de sorties est présenté avec la classe [C]OutputSpec[/C] (voir page 'Les spécifications de sortie'). \Db propose des services pour lire ce fichier (par un appel à [Y]ParseOuputSpecFile[/Y] normalement au début du programme principal), l'interpréter et générer automatiquement les sorties sur un ou plusieurs autres fichiers qui seront exploitables après la fin de la simulation (voir la classe [C]OutputSpec[/C]).\index{spécifications de sortie}\index{fonctions!ParseOuputSpecFile}
 
\item {\large {\tt Les directives de simulation}} : il s'agit des actions que doit entreprendre le moteur pour initialiser une simulation, afficher la structure initiale du système, exécuter la simulation et exploiter ses sorties. Ces directives de calcul figurent dans un unique {\em fichier de directives} de simulation présenté dans le lien ' Les fichiers en entrée de type déclaratif'.\\
Bien noter ici encore que le développeur du simulateur peut décider de rendre générale tout ou partie de la série d'actions à entreprendre, et la coder dans le programme compilé (plus précisément dans la fonction [Y]main()[/Y]). 
\noLF
Quel que soit l'ensemble des directives externalisées sur fichier, sa lecture et son interprétation sont lancées, dans le programme principal, par une unique instruction [Y]ParseDirectiveFile[/Y].\index{fichiers de directives}\index{directives de simulation}\index{fonctions!ParseDirectiveFile}
 
\end{enumerate}

\vspace{5mm}
{\bf Remarques :} 
\begin{itemize}
\item Le mot {\em fichier} peut désigner, selon le contexte, l'une ou l'autre des deux entités bien distinctes suivantes : 
  \begin{itemize}
  \item le lieu physique où les données sont stockées, avec une certaine organisation (c'est la notion classique de fichier),
  \item la structure de données, instance d'une sous-classe de [C]File[/C], qui encapsule les informations sur le lieu physique et l'organisation des données.
  \end{itemize}
\item Aucune classe propre au domaine ne peut être une sous-classe directe de la classe générique [C]File[/C] et aucune instance directe de cette classe ne doit être créée.
  \item La présente version ne propose de services prédéfinis que pour la lecture de données. Quant aux services d'écriture sur fichier, ceux qui exécutent les spécifications de sorties de la simulation sont inclus dans les méthodes de [C]OutputSpec[/C] et inaccessibles directement. Au-delà de ça, les procédures répondant à des besoins spécifiques dans tel ou tel domaine d'application sont à programmer entièrement, avec les outils de plus bas niveau.
\end{itemize}
*/
//@{

// ----------------
// Enumérations utilisées par des fonctions externes aux classes.
// ----------------

enum InternalFileId {OUT_FILE = -2, IN_FILE=-1};

enum AccesTypeId {
  READ  = 1,
  WRITE = 2
};

enum FileClassType {			// Type de classe de DIESE
  GENERIC_FILE = 0,
  LEXICAL_INPUT_FILE,
  SERIAL_DATA_FILE,
  SEQUENTIAL_DATA_FILE,
  FACTUAL_DATA_FILE,
  TEXT_OUTPUT_FILE=5 
};
/**@name Enumération des modes d'écriture sur les fichiers
APPEND=1, NEW=2\index{OutputMode}\label{defAppendNew}\\

Liste exhaustive des modes d'écriture dans un fichier, utilisés par les spécifications de sortie (voir page 'Les spécifications de sortie') et pour les fichiers de sortie banalisés.\\

Une spécification de sortie en mode NEW sur un fichier <file> crée le fichier <file> puis écrit les informations à partir de son début.\\

Une spécification de sortie en mode APPEND sur le fichier <file> écrit les informations en commençant après le dernier enregistrement déjà présent. 
*/
//@{
enum OutputMode {
   APPEND=1,
   NEW=2
};
//@}


//----------------------------------------------------------------------------
// File
//----------------------------------------------------------------------------
/**@name La classe de base pour représenter les fichiers 
\begin{itemize} \item Directive d'inclusion :
[P]
\#include File.h
[/P]
\end{itemize}
*/
//@{
  /// 
class File
{
public:
// =============================
// Exceptions
// =============================

/**@name LIFECYCLE */
//@{
	/// Constructeur de base. 
  File();
	/// Constructeur. N'est normalement appelé que par les constructeurs des sous-classes prédéfinies. Le premier argument correspond au symbole de classe, le second correspond au type d'accès (READ pour un accès en lecture seulement ou WRITE pour un accès en écriture seulement). Le troisième argument est 0 pour un acces READ, et NEW ou APPEND selon que l'écriture doit débuter au début ou en fin du fichier.
  File (UserFileId, AccesTypeId, OutputMode); // _no swig_

	// Destructeur superficiel. 
  ~File();		 // _no swig_
//@}

  // OPERATORS

         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const File&) const {return TRUE;};
  Boolean         operator > (const File&) const {return TRUE;};


/**@name [SC]Identification[/SC] */
//@{
	/// Renvoie le symbole de classe.\index{méthodes!ClassSymbol - File}
  UserFileId	ClassSymbol();
	/// Affecte le symbole de classe.
  void   	ClassSymbol(UserFileId);	

	

	// Affiche les attributs d'une instance de cette classe
  void          Display(); // _no swig_	
		
//@}

  /**@name [SC]Ouverture et fermeture[/SC] */
//@{		

	/// Ouverture du fichier, en fonction du mode d'accès déclaré lors de la construction.\index{méthodes!Open}
  void          Open();		

	/// Fermeture du fichier. Cette méthode s'assure que le fichier est bien ouvert, avant de le fermer.\index{méthodes!Close}
  void          Close();
//@}




/**@name [SC]Lien avec le fichier physique[/SC] */
//@{     

	/// Renseigne le nom complet du fichier. Le client de ce service doit se charger de la désallocation éventuelle de la chaîne passée en argument.\index{méthodes!AssignFullName}
  void		AssignFullName(char*);  

	/// Renseigne le répertoire physique du fichier. C'est le chemin d'accès au fichier relativement au répertoire dans lequel la simulation est exécutée. Le client de ce service doit se charger de la désallocation éventuelle de la chaîne passée en argument. Si le nom simple a été spécifié, il est assemblé à ce répertoire, pour être assigné au nom complet.\index{méthodes!AssignPhysicalPath}
  void		AssignPhysicalPath(char*);
  
	/// Renseigne le nom simle du fichier. Le client de ce service doit se charger de la désallocation éventuelle de la chaîne passée en argument. Si le répertoire a été spécifié, il est assemblé à ce nom simple, pour être assigné au nom complet.\index{méthodes!AssignSimpleName}
  void		AssignSimpleName(char*); // _no swig_ 

        /// Renvoie le nom physique du répertoire du fichier. 
  char*         GetPhysicalPath() const; // _no swig_ 

        /// Renvoie le nom complet du fichier. 
  char*         GetFullName() const; // _no swig_   

	// Renvoie le nom simple du fichier. 
  char*         GetSimpleName() const; // _no swig_ 

	// Affiche le nom de répertoire, le nom simple et le nom complet du fichier. 
  void          DisplayNames() const; // _no swig_ 

        /// Renvoie vrai si le répertoire du fichier est spécifié. 
  Boolean       PathSpecified() const; // _no swig_ 
        /// Renvoie vrai si le nom complet du fichier est spécifié. 
  Boolean       FullNameSpecified() const; // _no swig_ 
        /// Renvoie vrai si le nom complet du fichier est spécifié. 
  Boolean       SimpleNameSpecified() const; // _no swig_ 
        /// Renvoie vrai si le nom du fichier est spécifié, soit par le nom complet, soit par le répertoire et le nom simple. 
  Boolean       FileNameSpecified() const; // _no swig_ 
        // Renvoie le nom complet du fichier, éventuellement reconstitué à partir du répertoire et du nom simple. Le client doit désallouer lui-m\^eme la mémoire occupée par la chaîne renvoyée (par [P]delete[];[/P]).\index{méthodes!GetFileName} 
  char*         GetFileName() const; // _no swig_

	/// Renvoie le pointeur sur le fichier physique. \index{méthodes!GetPhysicalFile}
  FILE*         GetPhysicalFile() const;		

	// Renvoie le pointeur sur la seule instance de la sous-classe de [C]File[/C] dont le symbole de classe est passé en argument.
  static File*	GetFile(UserFileId); // _no swig_ 	

	// Renvoie le pointeur sur la seule instance de la sous-classe de [C]SerialDataFile[/C] dont le symbole de classe est passé en argument. 
  static SerialDataFile* GetSerialDataFile(UserFileId);
//@}

    // comptage
    static int   getNbConst() {return _nbConst;}
    static int   getNbDest() {return _nbDest;}


  static pFileTab    mpInstanceTab;	// Tableau des instances de File
  static int	     mInstanceNumber;	// le nombre d'instance

  FileClassType	ClassType();		// Renvoie le type de classe.

protected :

  int static      _nbConst;
  int static      _nbDest;
  FILE*		mpFile;			// Pointeur du fichier physique
  UserFileId	mClassSymbol;
        
  FileClassType mClassType;	  	// Type de classe de DIESE

  char*		mPhysicalPath;	  // Nom du répertoire (path)   //+ JPR 060323
  char*		mFullName;	  // Nom complet du fichier
  char*		mSimpleName;	  // Nom simple du fichier      //+ JPR 020321

  int		mOpenMode;	  // Lecture = 1, ecriture = 2
  int		mOutputMode;	  // APPEND= 1, NEW = 2

  enum FileStatus {
    OPENED   = 1,
    CLOSED  = 2
  }		mFileStatus;

}; // _end class_

// ---------------
// EXTERNAL FUNCTIONS
// ---------------
/**@name Fonction de désallocation 
\\
{\large {\tt void DeleteFileInstances(void)}}

Suppression en mémoire de toutes les instances créées pour toutes les sous-classes de [C]File[/C].\\
Réalise la fermeture des fichiers physiques correspondants.\\
Remarquer qu'en conséquence tous les fichiers sont fermés en même temps (en fait tous les fichiers en série d'enregistrements, puisque les fichiers de paramètres du système et de la simulation, les fichiers de description de la structure et le fichier de directives sont fermés, et l'instance supprimée, juste après leur exploitation).\index{fonctions!DeleteFileInstances}
*/
//@{
void   DeleteFileInstances(void); // _no swig_

//@} // fin fonctions

//@} // fin classe de base

//----------------------------------------------------------------------------
// LexicalInputFile
//----------------------------------------------------------------------------
/**@name Les fichiers en entrée de type déclaratif 
\\
Il s'agit des fichiers placés en arguments de la ligne appel du simulateur exécutable (par exemple [P]main[/P]). Dans une forme simple, la ligne d'appel est :
 
\begin{verbatim}
[P]./main  sim1.par  sim1.sim  sim1.osp  sim1.str  sim1.dir[/P]
\end{verbatim}

Les cinq arguments font l'objet, dans cet ordre, d'un des liens du haut de la page.
*/
//@{


/**@name La classe des fichiers déclaratifs 
\\
Les services, ici documentés ou non, sont exploités par d'autres services prédéfinis dans \D (les interpréteurs des fichiers). Le développeur ne les utilise normalement pas directement.
*/
//@{
  /// \label{classLexicalInputFile} 
class LexicalInputFile : public File  // _no swig_
{
public:
  /**@name LIFECYCLE */
//@{
	/// Constructeur du fichier. Ce constructeur est appelé automatiquement par les fonctions de lecture et d'interprétation des fichiers de déclarations.
  LexicalInputFile();

	/// Destructeur du fichier (instance de la classe [C]LexicalInputFile[/C] et fermeture du fichier ; le fichier en tant que lieu physique n'est évidemment pas touché). 
  ~LexicalInputFile();
//@}

	//
  int GetCurrentRecordIndex();
  LexicalInputFile* GetFileFrom();

	//
  void SetCurrentRecordIndex(int);
	//
  void SetFileFrom(LexicalInputFile*);

 protected:
  int currentRecordIndex;
  LexicalInputFile* pFileFrom;

}; // _end class_

//@} // fin classe


/**@name Le fichier des paramètres du système
\\
\index{fichier de paramètres}
Ce fichier est lu (et son contenu exploité) par un appel à la fonction [Y]ParseParameterFile[/Y], avant l'exécution de la simulation. Cette fonction prend en argument le nom physique du fichier.
\\
Cet appel peut figurer dans le programme principal ... : 
\begin{verbatim}
[P]int main(int argc,  char* argv[]) {
  ...
  ParseParameterFile(argv[1]);
  ...
  pCurrentSim->Run();
  ...
}[/P]
\end{verbatim}

... auquel cas le nom du fichier est passé en argument de la ligne de commande :

\begin{verbatim}
[P]./main sim1.par ...[/P]
\end{verbatim}

L'appel peut aussi résulter d'une directive de simulation écrite sur un fichier externe (voir le lien 'Les fichiers des directives de la simulation'), auquel cas le nom du fichier sera un des éléments de la directive.\\ 
 */
//@{
/**@name Contenu du fichier des paramètres
Le fichier de paramètres est une suite de lignes, dont la syntaxe est la suivante\label{parameterFile} :

\begin{verbatim}
[P]<fichier>               ::= <ligne>*
<ligne>                 ::= <ligne_trace> | <ligne_commentaire> | <ligne_donnees>
<ligne_trace>           ::= TRACE <mode_trace> <fin>
<mode_trace>            ::= PARSING | NO_PARSING
<ligne_commentaire>     ::= <symbole_commentaire> <any>
<any>                   ::= .*

<ligne_donnees>         ::= <mot_clé>? <identifiant_1> <identifiant_2> 
                            <symbole_affectation> <valeur> <fin>
<fin>                   ::= ; 

<mot_clé>               ::= <chaine_maj> 
<identifiant_1>         ::= <chaine_id> 
<identifiant_2>         ::= <chaine_id> 
<symbole_affectation>   ::= <- | <= 
<symbole_commentaire>   ::= // | # 
<valeur>                ::= <entier> | <reel> | <chaine | <path>

<chaine>                ::= [a-zA-Z%] [a-zA-Z0-9'_/%.*:]* |
                            "[a-zA-Z%/.~][a-zA-Z0-9'_/%.*: ]*"
<chaine_id>             ::= [a-zA-Z][a-zA-Z0-9] | "[a-zA-Z][a-zA-Z0-9]"
<chaine_maj>            ::= [A-Z][A-Z0-9\_]*
<path>                  ::= <path_linux> | <path_windows>
<item_path_l>           ::= [a-zA-Z0-9][a-zA-Z0-9\_\.]*
<item_path_w>           ::= [a-zA-Z0-9][a-zA-Z0-9\_\.\ ]*
<path_linux>            ::= (~ | .. | .)?/(<item_path_l>/)*
<path_windows>          ::= (~ | .. | . | [a-zA-Z]:)?(/ | \)(<item_path_w>(/ | \))*

<entier>                ::= -?[0-9] | MIN | MAX
<reel>                  ::= -?[0-9]*[.,][0-9]+ | EPS | DOUBLE_EPS[/P]
\end{verbatim}

C'est le développeur qui choisit les mots_clés et les chaînes <identifiant_1> et <identifiant_2> (30 caractères au plus)\label{NDLR : MINMAJSPC dans ParserParam.lex.l}. Il est bon qu'ils et elles soient "parlantes" à l'utilisateur. Par exemple, la ligne :
\begin{verbatim}
[P]voiture positionInitiale   <-  0.5;[/P]
\end{verbatim} 
exploitée par un appel [P]GetRealParameterValue("voiture","positionInitiale")[/P], est plus parlante que la ligne suivante, qui renverra la même valeur avec un appel [P]GetRealParameterValue("a","b")[/P] :
\begin{verbatim}
[P]a b   <-  0.5;[/P]
\end{verbatim}
Noter que les chaînes contenant des caractères accentués sont interdites dans les identifiants, ainsi que la présence d'espaces, de '_' et de '-'. Les chaîne peuvent être entourées de guillemets. Pour les répertoires (valeurs de paramètres de type chaîne de caractères), y compris les chemins Windows contenant des espaces, ne jamais les entourer de guillements. 

Le fichier peut donc contenir des lignes telles que :

\begin{verbatim}
[P]// Paramètres des processus continus[/P]
\end{verbatim}

\begin{verbatim}
[P]voiture positionInitiale   <-  0.5;[/P]
\end{verbatim} 
pour affecter la valeur flottante 0,5 au paramètre identifié par les deux chaînes "voiture" et "positionInitiale" (noter la {\em présence} obligatoire du point pour un paramètre à valeur flottante)\label{formatValeurParam}, 

\begin{verbatim}
[P]ENTITE voiture nbPassagers   <-  4;[/P]
\end{verbatim} 
pour affecter la valeur entière 4 au paramètre identifié par les deux chaînes "voiture" et "nbPassagers" (noter l'{\em absence} obligatoire du point pour un paramètre à valeur entière), et :
 
\begin{verbatim}
[P]FICHIER "FichierToboggan" NAME      <- fdonnees.txt;
FICHIER "FichierToboggan" FORMAT    <- "%d %*s %lf";[/P]
\end{verbatim}
pour affecter la chaîne de caractères "fdonnees.txt" comme valeur du paramètre identifié par les deux chaînes "FichierToboggan" et "NAME", puis la chaîne de caractères "\%d \%*s \%lf" comme valeur du paramètre identifié par les deux chaînes "FichierToboggan" et "FORMAT". Noter qu'on peut ou non entourer les chaînes de guillemets. L'exploitation du premier paramètre pourra être réalisée par une séquence telle que :
\begin{verbatim}
[P]  SequentialDataFile* p = new SequentialDataFile();
  p->AssignFullName(GetStringParameterValue("FichierToboggan", "NAME");[/P]
\end{verbatim}

Un exemple de fichier de paramètres est le suivant :
\begin{verbatim}
[P]TRACE NO_PARSING;
# début paramètres entiers
ENTITE voiture nbPassagers <- 4;
voiture cv <= 7;
voiture quotaKm <= MAX;
TRACE PARSING;
// début paramètres réels
voiture "positionInitiale"   <-  0.5;
voiture "vitesse maxi"   <-  80.;
voiture "conso"   <-  EPS;   // la plus petite valeur en simple précision
"LOG" "DIR" <- c:\docsNsets\durand\KBS\Ma base\exec\Mon log\ "";
//"LOG" "DIR" <- /cygdrive/c/docsNsets/durand/KBS/MaBase/exec/Mon log/;[/P]
\end{verbatim}

L'{\em inclusion} d'un fichier à l'intérieur d'un autre est commandée par :
\begin{verbatim}
[P]INCLUDE <chemin>;[/P] 
\end{verbatim}
où <chemin>} est le nom complet (absolu ou relatif) du fichier inclus.\\
La dernière ligne du fichier inclus doit être :
\begin{verbatim}
[P] 
END INCLUDE;[/P] 
\end{verbatim}

Ce système d'inclusion est, en complément de la notion de groupe de paramètres, une autre voie de structuration et de clarification d'un grand ensemble de paramètres.
*/
/**@name Transformation de paramètres en variables globales 
\\
Pour améliorer la vitesse d'accès à la valeur d'un paramètre, surtout dans le cas où la simulation comporte de nombreux accès, on peut remplacer l'appel à, par exemple, [P]GetIntParameterValue("voiture", "conso")[/P] par une invocation d'une variable globale, appelée par exemple [Y]gIntParam_voiture_conso_[/Y]. \\

A cette fin, il faut :
\begin{itemize} 
\item déclarer ces variables (en {\em extern}) dans le code du simulateur (fichier [F]UserTypes.h[/F]) : \\
\begin{verbatim}
[P]  extern int gIntParam_voiture_conso_;
[/P]
\end{verbatim}

\item leur donner une valeur : ces initialisations sont réalisées de la manière suivante, dans une fonction dont le nom est obligatoirement [Y]InitParameterGlobalVariables[/Y]. \\
\begin{verbatim}
[P]void InitParameterGlobalVariables() {
  gIntParam_voiture_conso_ = GetIntParameterValue("voiture", "conso");
  ...
};[/P]
\end{verbatim}
\end{itemize} 

Parce que le moteur de simulation invoque impérativement cette fonction, elle doit être présente dans le code de l'application, même si elle ne comporte aucune initialisation de variable. On comprend que cette fonction est invoquée par le moteur de simulation juste après la lecture du fichier des paramètres du système.\\

Noter que le code de l'application peut contenir des appels [P]GetIntParameterValue("voiture", "conso");[/P], qu'on ait ou non initialisé la variable correspondante [Y]gIntParam_voiture_conso_[/Y].\\

Si le paramètre pris en exemple était de type flottant ou chaîne de caractères, la dénomination suggérée pour la variable serait [Y]gRealParam_voiture_conso_[/Y] ou [Y]gStringParam_voiture_conso_[/Y], respectivement.\\

Noter enfin que l'outil Solfege, associé à la bibliothèque \D, prend entièrement en charge le codage de [Y]InitParameterGlobalVariables[/Y], et le remplacement des appels aux [Y]Get...ParameterValue[/Y] par la variable globale correspondante. Les appels aux [Y]Set...ParameterValue[/Y] sont conservés et complétés par une surchargement de la valeur de la variable globale correspondante.
*/
//@} 



/**@name Le fichier des spécifications de simulation
\\
Ce fichier est lu (et son contenu exploité) par un appel à la fonction [Y]ParseSimulationFile[/Y], avant l'exécution de la simulation. Cette fonction prend en argument le nom physique du fichier.\\
Cet appel peut figurer dans le programme principal ... : 
\begin{verbatim}
[P]int main(int argc,  char* argv[]) {
  ...
  ParseParameterFile(argv[1]);
  ParseSimulationFile(argv[2]);
  ...
  pCurrentSim->Run();
  ...
}[/P]
\end{verbatim}

..., auquel cas le nom du fichier peut être passé en argument de la ligne de commande :

\begin{verbatim}
[P]./main  sim1.par  sim1.sim ...[/P]
\end{verbatim}
\index{fichier de spécifications de simulation}

L'appel peut aussi résulter d'une directive de simulation écrite sur un fichier externe (voir le lien 'Les fichiers des directives de la simulation'), auquel cas le nom du fichier sera un des éléments de la directive.\\ 

Le fichier des spécifications de simulation est une suite de phrases, dont la syntaxe est la suivante :

\begin{verbatim}
[P]<fichier>      ::= <phrase>*
<phrase>       ::= <ligne_trace> | <comment> | <set_ivar> 
                   <init_rand>   |  <init_sim>   |  <open_file>

<ligne_trace>  ::= TRACE <mode_trace> <fin>
<mode_trace>   ::= PARSING | NO_PARSING

<comment>      ::= <l_comment> | <ls_comment>
<l_comment>    ::= <symb_comment> <any>
<symb_comment> ::= // | # 
<any>          ::= .* 
<ls_comment>   ::= '/''*'(<any>\n)*<any>'/''*'

<set_ivar>     ::= <ref_ivar> = <int_expr> <fin> 
<entier>       ::= [0-9]+
<int_expr>     ::= <entier> | <ref_ivar> | <ref_iparam> |
                   <int_expr> <op> <int_expr> |
                   (<int_expr>) |
                   RANDOM(<int_expr>)
<op>           ::= < | <= | > | >=
<ref_ivar>     ::= {i<entier>}
<ref_iparam>   ::= '<'pi'>' '<' <param_str> <param_str> <'>'
<param_str>    ::= <chaine> | <multichaine>

<init_rand>    ::= INITRAND <entier> <fin> |
                   INITRAND <ref_iparam> <fin> |
                   INITRAND <fin>

<init_sim>     ::= INITIALISATION SIMULATION <spec>+ <fin>
<spec>         ::= <spec_unite>|<spec_step>|<spec_beg>|<spec_end>|
                   <spec_in>|<spec_out>

<spec_unite>   ::= UNITE_TPS <unite_tps> <fin>
<unite_tps>    ::= DAY ...
<spec_step>    ::= NB_UNITE_TPS <entier> <fin>
<spec_beg>     ::= DATE_DEBUT <date> <fin>
<spec_end>     ::= DATE_FIN <date> <fin>
<date>         ::= <entier> <mois> <entier> [<entier> <entier> <entier>] 
<mois>         ::= JAN .... DEC  
<spec_in>      ::= IN_DIR <dir_path> <fin>
<spec_out>     ::= OUT_DIR <dir_path> <fin>
<dir_path>     ::= "[(~ | . | .. |)/<etage>]*"
<etage>        ::= <chaine_rep>/
<chaine>       ::= [a-zA-Z][a-zA-Z0-9\_\.]*
<chaine_rep>   ::= [a-zA-Z][a-zA-Z0-9\_\.\:]*
<multichaine>  ::= "(<chaine>" ")*<chaine>"

<open_file>    ::= OUVERTURE <file_type> <spec_file>+ <fin>
<file_type>    ::= FICHIER_SERIEL|FICHIER_SEQUENTIEL
<spec_file>    ::= <spec_nom>|<spec_fkey>|<spec_fdata>|<spec_f>|
                   <spec_com>|<spec_skip>
<spec_nom_s>   ::= NOM_SIMPLE_FICHIER <simple_name> <fin>
<simple_name>  ::= "<chaine>" | <ref_svar>
<ref_svar>     ::= "<chaine>"
<spec_nom_c>   ::= NOM_COMPLET_FICHIER <full_name> <fin>
<full_name>    ::= "<dir_path><simple_name>"
<spec_fkey>    ::= FORMAT_CLES <format> <fin> 
<spec_fdata>   ::= FORMAT_DONNEES <format> <fin> 
<spec_f>       ::= FORMAT_CHAMPS <format> <fin>
<spec_com>     ::= LIGNES_ENTETE <entier> <fin>
<spec_skip>    ::= SKIP_ENTETE <fin>
<format>       ::= "<chaine>"

<fin>          ::= ;[/P] 
\end{verbatim}

\begin{itemize}
\item  {\small {\tt <ligne_trace>}} permet d'activer ou de désactiver une fonction de trace des actions de lecture/inter\-prétation. 
   
\item {\small {\tt <init_rand>}} initialise une série de nombres pseudo-aléatoires par un appel à [Y]srand(<entier>)[/Y] ou [Y]srand(<valeur de paramètre>)[/Y], ou bien par un appel à [Y]srand((int)getpid())[/Y] si l'entier n'est pas fourni ou est égal à 0. Utile pour provoquer des séries différentes (par la fonction [Y]RandomInteger[/Y]) lors de simulations successives. On rappelle que si aucune directive {\small {\tt <init_rand>}} n'est donnée, la génération de la série aléatoire commence en utilisant la valeur 1.

\item {\small {\tt <init_sim>}} encapsule une ou plusieurs des spécifications suivantes pour la simulation. On rappelle qu'une instance de la classe [C]Simulation[/C], laquelle est la valeur de la variable globale [Y]pCurrentSim[/Y] (voir pages 'Variables globales' et 'La classe des simulations'), a été créée par un appel à [Y]InitSimulation[/Y] au début de la fonction [Y]main[/Y]. 

\item {\small {\tt <spec_unite>}} spécifie l'unité de temps naturelle (minute, seconde, \ldots) en laquelle est mesurée la longueur de l'unité de temps simulé (voir page 'Types et fonctions pour la gestion du temps simulé'). Les indications valides pour {\small {\tt <spec_unit>}} sont celles de l'énumération {\em TimeUnit} (voir page 'Unités de temps simulé'). Il n'y a pas de valeur par défaut : cette spécification est donc nécessaire.

\item {\small {\tt <spec_step>}} spécifie la longueur de l'unité de temps simulé, avec l'unité de mesure ci-dessus. Comme ci-dessus, il n'y a pas de valeur par défaut.

\item {\small {\tt <spec_beg>}} et {\small {\tt <spec_end>}} déterminent les instants de début et de fin au plus tard de la simulation. Dans la {\small {\tt <date>}}, les {\small {\tt <entier>}} représentent, dans l'ordre, le jour, l'année puis, éventuellement, l'heure, la minute et la seconde.
Les indications valides pour {\small {\tt <mois>}} sont celles de l'énumération {\em ShortMonthName} (voir page 'Constantes, structures et fonctions pour les dates et le temps en général').
Les années avant 2000 doivent être données avec leurs 4 chiffres (e.g. 1997). Celles à partir de 2000 peuvent être données avec leurs deux derniers chiffres (e.g. 03 pour 2003).

\item {\small {\tt <spec_out>}} désigne, entre guillemets et relativement au répertoire dans lequel est lancée la commande d'exécution, le répertoire dans lequel seront éventuellement créés les fichiers de sortie selon les indications portées dans le fichier de spécifications de sortie. Cette valeur sera celle renvoyée par la méthode [Y]GetOutDir[/Y] de la classe [C]Simulation[/C].

\item {\small {\tt <spec_in>}} désigne, entre guillemets et relativement au répertoire dans lequel est lancée la commande d'exécution, le répertoire dans lequel seront éventuellement recherchés les fichiers d'entrée. Cette valeur sera celle renvoyée par la méthode [Y]GetInDir[/Y] de la classe [C]Simulation[/C].

\item le mot-clé RANDOM, suivi d'une expresion entre parenthèses, renvoie un nombre entier aléatoire positif inférieur à la valeur de l'expression. 

\item {\small {\tt <open_file>}} déclare les propriétés d'un fichier en série d'enregistrements. Noter bien que ce genre de spécifications peuvent \^etre écrites dans le {\en fichier de la structure du système}. On peut indiquer ici :
\begin{itemize}
\item le ou les formats des champs de clé et des champs de donnée, ou le format global (clés et données); 
\item le nombre de lignes d'entête qui ne sont pas des lignes de données ({\small {\tt <spec_com>}}) et demander de les sauter immédiatement ({\small {\tt <spec_skip>}}), c'est-à-dire pendant la lecture du fichier des paramètres de simulation (on peut aussi avoir programmé ce saut ailleurs dans le simulateur, et donc ne pas le demander ici). ; 
\item quant au nom du fichier physique ({\small {\tt <spec_nom_*>}}), on peut indiquer le nom complet ou le nom simple. Dans le second cas, le fichier sera recherché dans le répertoire désigné dans la spécification {\small {\tt <spec_in>}}.
\end{itemize}
\end{itemize}

Des exemples valides pour {\small {\tt <dir_path>}} sont : [P]"/home/anyuser/dir"[/P],[P]"~/dir/"[/P], [P]"dir/"[/P], [P]"../dir/"[/P], [P]"./dir/"[/P]. Si la chaîne entre les guillemets (non compris) est une spécification absolue (commençant par un {\small {\tt /}}), elle constitue directement le chemin complet du répertoire des fichiers de sortie. Si c'est une spécification relative, elle est interprétée relativement au répertoire courant.\\

Toute fin de ligne après {\small {\tt //}} ou {\small {\tt \#}} est ignorée, de même que tout passage entre /$*$ et $*$/.\\

Un exemple de fichier de spécification de la simulation est le suivant :
\begin{verbatim}
[P]//TRACE NO_PARSING;
{i1} = 97;
INITIALISATION SIMULATION
     UNITE_TPS MINUTE;
     NB_UNITE_TPS 1;
     //UNITE_TPS SECOND;
     //NB_UNITE_TPS 30;
     DATE_DEBUT 1 JAN {i1}  
                <pi><Simulation "heure deb"> <pi><Simulation "minut deb"> 0;
     DATE_FIN   1 JAN {i1} 
                <pi><Simulation "heure fin"> <pi><Simulation "minut fin"> 0;
     OUT_DIR "../out/minute/";
     OUVERTURE FICHIER_SEQUENTIEL FICHIER_METEO
         NOM_PHYSIQUE  "../in/meteo_alenya.15min.txt";
         FORMAT_CHAMPS "%d %d %d %d %lf %lf %lf %lf %lf";
         LIGNES_ENTETE 1;
         //SKIP_ENTETE;  // le saut est programmé ailleurs
     ;
;[/P]
\end{verbatim}
*/


/**@name {\it Le fichier des spécifications de sortie} (voir le chapitre dédié du sommaire)
*/


/**@name Le fichier de description de la structure du système
\\
Ce fichier est lu (et son contenu exploité) par un appel à la fonction [Y]ParseStructureFile[/Y]. Cette fonction prend en argument le nom physique du fichier. Cet appel peut figurer dans le programme principal ... : 
\begin{verbatim}
[P]int main(int argc,  char* argv[]) {
  ...
  ParseParameterFile(argv[1]);
  ParseSimulationFile(argv[2]);
  ...
  ParseStructureFile(argv[4]);
  ...
  pCurrentSim->Run();
  ...
}[/P]
\end{verbatim}

... auquel cas le nom du fichier peut être passé en argument de la ligne de commande. Par exemple :
\begin{verbatim}
[P]./main  sim1.par  sim1.sim  sim1.osp  sim1.par ...[/P]
\end{verbatim}

L'appel peut aussi résulter d'une directive de simulation écrite sur un fichier externe (voir plus haut la section sur le fichier des directives), auquel cas le nom du fichier sera un des éléments de la directive.\\ 

Quoi qu'il en soit, l'appel à [Y]ParseStructureFile[/Y] doit figurer après l'appel à [Y]ParseParameterFile[/Y] si la mise en place d'un élément de la structure requiert que des paramètres du système soient déjà connus en mémoire (donc accessibles par [Y]Get..ParameterValue[/Y]).\\

De même, il doit figurer avant l'appel à [Y]Run[/Y], laquelle requiert naturellement l'instanciation préalable du système.\\  

Le fichier de description de la structure n'est pas présenté ici de façon formelle, sur le modèle utilisé pour les autres fichiers, mais en considérant successivement les différents sens qu'on souhaite donner aux expressions.

\begin{itemize}
\item La {\em trace} des opérations de lecture/interprétation est initiée par : 
\begin{verbatim}
[P]TRACE PARSING;[/P]
\end{verbatim}
La trace est arrêtée par :  
\begin{verbatim}
[P]TRACE NO_PARSING;[/P]
\end{verbatim}

\item L'{\em inclusion} d'un fichier est commandée par :
\begin{verbatim}
[P]INCLUDE <chemin>;[/P] 
\end{verbatim}
où <chemin>} est le nom complet (absolu ou relatif) du fichier inclus.\\
La dernière ligne du fichier inclus doit être :
\begin{verbatim}
[P] 
END INCLUDE;[/P] 
\end{verbatim}

\item On crée une variable locale temporaire entière (ou réelle, ou à valeur chaîne de caractères), et on lui affecte une valeur résultat de l'évaluation de l'expression entière {\small {\tt <int_expr>}} (ou réelle {\small {\tt <real_expr>}}, où à valeur chaîne de caractères {\small {\tt <str_expr>}}), par :
\begin{verbatim}
[P]{i1} = <int_expr>;
{x9} = <real_expr>;
{s1} = <str_expr>;[/P]
\end{verbatim}
Les indices valides pour {\small {\tt i.}}, {\small {\tt x.}} et {\small {\tt s.}} sont les entiers de 1 à 10.\\
Les expressions {\small {\tt <int_expr>}} et {\small {\tt <real_expr>}} sont construites avec les règles énoncées pour {\small {\tt <int_expr>}} dans la section consacrée au fichier de spécification de la simulation. Les expressions {\small {\tt <str_expr>}} sont une chaîne de caractères, la référence à un paramètre à valeur chaîne de caractères (avec le préfixe {\small {\tt <ps>}}) ou bien plusieurs {\small {\tt <str_expr>}} séparées par l'opérateur {\small {\tt ^}} signifiant la concaténation.

\item On demande la création d'une {\em nouvelle instance d'entité} de la classe de nom {\small {\tt nom_c}} par :
\begin{verbatim}
[P]+ I nom_c nom_i;  // ou + I nom_c nom_i,; ou encore + I nom_c,;[/P] 
\end{verbatim}
{\small {\tt nom_c}} et {\small {\tt nom_i}} sont des chaînes de caractères simples ({\em e.g} abc) ou multiple, encadrées alors par des guillemets ({\em e.g} "abc dfe"). Les caractères autorisés sont les lettres et le caractère '_'  et, sauf pour le premier caractère, les chiffres, '.' et ':'. \\ 
Cette requête invoque un constructeur de l'entité codé dans le fichier [F]UserFunction.cc[/F]. Ce constructeur est déterminé dans la fonction [Y]App_NewEntityFromString[/Y] (voir page 'Fonctions globales/Création d'instances de classes') en fonction de la chaîne de caractères passée en argument (ici {\small {\tt nom_c}}).\\
Comme le constructeur invoqué n'attribue généralement pas de nom d'instance, un nom d'instance par défaut sera attribué par \Db (voir page 'La classe des entités sans descripteurs ni méthodes/Identification de l'entité').\\
Ce nom par défaut est remplacé par un nom choisi, {\em e.g.} {\small {\tt nom_i}}, si on écrit :
\begin{verbatim}
[P]+ I nom_c nom_i;[/P]
\end{verbatim}

\item On demande l'ajout à {\small {\tt nom_i}} d'un {\em nouveau composant} de la classe de nom {\small {\tt nom_cc.}}, et d'éventuel nom d'instance {\small {\tt nom_ic.}} par l'une des notations de l'exemple suivant :
\begin{verbatim}
[P]+ I nom_c nom_i,
     + C nom_cc1, ;               // noter la virgule
     + C nom_cc2 nom_ic2 ;
     + C nom_cc3 nom_ic33, ;[/P]
;\end{verbatim}
Noter que de façon générale il n'y a aucune contrainte de retour à la ligne ou d'indentation. Dans le dernier exemple, le {\small {\tt ;}} final (qui clôt la déclaration de {\t nom_i}) peut suivre le précédent sur la même ligne.\ \

\item De même, on demande l'ajout à {\small {\tt nom_i}} d'un {\em nouvel élément} de la classe de nom {\small {\tt nom_ce}} par :
\begin{verbatim}
[P]+ I nom_c nom_i, 
     + E nom_ce1, ;
     + E nom_ce2 nom_ie2 ;
     + E nom_ce3 nom_ie3, ;;[/P]
\end{verbatim}

\item On fait {\em référence à une entité déjà créée}, de classe {\small {\tt nom_c}} et nommée {\small {\tt nom_i}}, en écrivant une des lignes suivantes :
\begin{verbatim}
[P]<I><nom_c nom_i>
<I><nom_c nom_i,>
<I><nom_c,>
<I><,nom_i>[/P]
\end{verbatim}
Si on ne désigne l'entité que par un nom de classe, c'est la première entité de la classe à avoir été instanciée qui sera désignée.

On pourra désigner cette entité comme étant l'"entité simulée" (cf. page 'La classe des simulations'), par :
\begin{verbatim}
[P]ENTITE_SIMULEE <I><nom_c nom_i>;[/P]
\end{verbatim}

\item On demande pour {\small {\tt nom_i}} l'{\em ajout d'un composant déjà créé} de nom de classe {\small {\tt nom_cc}} par :
\begin{verbatim}
[P]+ I nom_c nom_i, 
     <- C <I><nom_cc,>;;[/P]
\end{verbatim}

\item On demande pour ce composant l'{\em ajout d'éléments déjà créés} de nom d'instance {\small {\tt nom_ie1}} et {\small {\tt nom_ie2}} par :
\begin{verbatim}
[P]+ I nom_c nom_i, 
     <- C <I><nom_cc,>
          <- E <I><,nom_ie1>; 
          <- E <I><,nom_ie2>;
     ;
;[/P]
\end{verbatim}

\item On établit la {\em valeur de descripteurs} de noms de classe {\small {\tt d1}}, \ldots de l'entité {\small {\tt nom_i}} par les expressions suivantes, selon que la valeur est, dans l'ordre, un nombre entier ({\small {\tt d11}}), un nombre réel ({\small {\tt d12}}), une fonction entière ou réelle de la valeur d'une variable locale temporaire ({\small {\tt d13}}), une chaîne de caractères ({\small {\tt d14}}), une date ({\small {\tt d15}}, noter les guillemets), la valeur d'un paramètre du système, d'abord entier ({\small {\tt d21}}), puis réel ({\small {\tt d22}}) et à valeur chaîne de caractères ({\small {\tt d23}}):
\begin{verbatim}
[P]+ I nom_c nom_i, 
     d11 = 1;
     d12 = 0.5;
     d12 = EPS;
     d12 = <int_expr>/<int_expr>;
     d13 = {i1}*3+{x2};
     d14 = str1^str2;
     d15 = "21/08/2008";
     d21 = <pi><str1 str2>;
     d22 = <pr><str1 str3>/2;
     d23 = <ps><str1 str2>;
     d7 = + I ,nom_i1; ;
     d8 << <I><,nom_i2>;
     d8 << 5 15 18;
     d9 << (0 <int_expr>);
;[/P]
\end{verbatim} 
 Les descripteurs ({\small {\tt d7}}), ({\small {\tt d8}}), ({\small {\tt d9}}) et ({\small {\tt d10}}) sont respectivement de type 'entité', 'tableau d'entité', tableau d'entiers et 'tableau d'intervalles d'entiers'. Les éléments de valeur à droite du symbole d'affectation {\small {\tt <<}} sont ajoutées à la fin de la liste des élements de valeur déjà présents.\\
Ces manières d'affecter une valeur à un descripteur s'appliquent dans tous les contextes de déclaration d'entité (nouvelle, déjà créée, composant ou élément à quelque niveau que ce soit)\\
   On voit en outre que les spécifications de construction de la structure et les affectations de valeurs peuvent être entremêlées. La valeur du descripteur ({\small {\tt d7}}) est construite au moment de l'affectation (remarquer des deux ";" successifs).

\item On peut {\em affecter des fonctions prédéfinies} comme corps aux méthodes (instances de [C]Method[/C]) des entités. Dans la présente version de \D, seulement deux fonctions sont prédéfinies, toutes deux de type {\em bool}, l'une retournant toujours {\em vrai}, l'autre retournant toujours {\em faux}. On réalise cela par les lignes suivantes, où {\small {\tt a_predicate}} et {\small {\tt b_predicate}} sont les noms de classe des instances de [C]Method[/C] attachées à l'entité {\small {\tt nom_i}} :
\begin{verbatim}
[P]+ I nom_c nom_i,
     (BOOLEAN) a_predicate -> ALWAYS_TRUE; 
     (BOOLEAN) b_predicate -> ALWAYS_FALSE;	     
;[/P]
\end{verbatim} 

 \item La structure d'une entité est affichée par une requête commençant par le groupe de mots {\small {\tt AFFICHER STRUCTURE}} suivi d'une référence à une entité déjà créée (voir plus haut) et terminée par {\small {\tt ;}}. 

\item Les valeurs des descripteurs d'une entité sont affichées par une requête commençant par le groupe de mots {\small {\tt AFFICHER VALEURS DESCRIPTEURS}} suivi d'une référence à une entité déjà créée (voir plus haut) et terminée par {\small {\tt ;}}. 

\item La valeur d'un descripteur d'une entité est affichée par une requête commençant par le groupe de mots {\small {\tt AFFICHER VALEUR INT}} (ou FLOAT, DOUBLE ou STRING) suivi d'une référence à une entité déjà créée (voir plus haut) et terminée par {\small {\tt ;}}.   

\item On demande la création d'une {\em nouvelle instance de spécification d'ensemble d'entités} de nom de classe {\small {\tt nom_c}} par :
\begin{verbatim}
[P]+ SpE nom_c;[/P]
\end{verbatim}

\item Des spécifications complémentaires peuvent être exprimées sur la spécification d'ensemble d'entités de la manière suivante, à titre d'exemples indépendants: 
\begin{verbatim}
[P]REFCLASS_ID = VOITURE;
REFCLASS_NAME = voiture;
EFFECTIF = 2;
QUANTIFICATEUR = 1;  // pour NFIRST, ou 2 pour ALLENT
MODE = 1; // pour RETURN_INSTANCES et 2 pour INSTANTIATE
REF_ENTITY = <I><, maVoiture>;[/P]
\end{verbatim}
  

\item On demande la création d'une {\em nouvelle instance de processus} (continu ou ponctuel) de nom de classe {\small {\tt nom_c}} et de nom d'instance {\small {\tt nom_i}} (noter que les deux informations sont requises et que, comme pour les entités, elles ont la forme d'une chaîne simple ou multiple) par :
\begin{verbatim}
[P]+ P nom_c nom_i;[/P]
\end{verbatim}
Cette requête invoque un constructeur du processus codé dans le fichier [F]UserFunction.cc[/F]. Ce constructeur est déterminé dans la fonction [Y]App_NewProcessFromString[/Y] (voir page 'Fonctions globales/Création d'instances de classes') en fonction de la chaîne de caractères passée en argument (ici {\small {\tt nom_c}}).\\
Si ce constructeur n'affecte pas explicitement de valeur au {\em pas} ou au {\em délai de ré-examen}, la valeur {\em 1} leur est attribuée par défaut.

\item On fait {\em référence à un processus déjà créé}, de classe {\small {\tt nom_c}} et nommé {\small {\tt nom_i}}, en écrivant :
\begin{verbatim}
[P]<P><nom_c nom_i>[/P]
\end{verbatim} 
ou, en n'utilisant que le nom d'instance (noter la position de la virgule) :
\begin{verbatim}
[P]<P><,nom_i>[/P]
\end{verbatim} 

\item Des spécifications complémentaires peuvent être exprimées sur un processus. Dans la déclaration suivante, portant sur le processus nommé {\small {\tt nom_i}}, et successivement, on affecte une valeur au {\em pas}, au {\em délai de ré-examen}, on désigne l'entité qui est l'objet du processus ({\small {\tt nom_io}}) : 
\begin{verbatim}
[P]<P><,nom_i>
     PAS = 2;
     DELAI = 5;
     ENTITE_CIBLE = <I><,nom_io>; ;[/P]
\end{verbatim}
Si le processus est un processus continu, on peut, au même niveau que les déclarations ci-dessus, programmer un événement d'initialisation : 
\begin{verbatim}
[P]<P><,nom_i>
     INIT_EVT
          DATE_OCCUR = 0;
          //DATE_OCCUR = 21/08/2008;
          //DATE_OCCUR = 21/AOU/2008 12/0/0;
          //DATE_OCCUR = 21/AOU/2008 <pi><"occur" "heure">/0/0;
          PRIORITE = 1;
     ;[/P]
\end{verbatim}
ou bien un événement d'initialisation et de poursuite, ou encore de poursuite simplement, en remplaçant le mot-clé {\small {\tt INIT_EVT}} par {\small {\tt INIT_PRCD_EVT}} ou {\small {\tt PRCD_EVT}}. Bien noter que cette programmation doit être la dernière spécification complémentaire du processus.\\

Si le processus est un processus de lecture de fichier séquentiel, on peut, au même niveau que les déclarations ci-dessus, déclarer le symbole de classe du fichier à lire, et l'une ou les autres des informations sur le nom du fichier à lire :
\begin{verbatim}
[P]FILE_ID = FICHIER_METEO;
REPERTOIRE_FICHIER "/home/durand/mesBases/in/";
//NOM_SIMPLE_FICHIER "meteo1.txt";
NOM_COMPLET_FICHIER "/home/durand/mesBases/in/meteo1.txt";[/P]
\end{verbatim} 

On peut aussi écrire :
\begin{verbatim}
[P]REPERTOIRE_FICHIER {s1};[/P]
\end{verbatim} 

... si {s1} est une variable locale (de type 'chaîne de caractères', vu sa lettre-préfixe) à laquelle on a préalablement affecté la valeur appropriée :
\begin{verbatim}
[P]{s1} = "/home/durand/mesBases/in/";[/P]
\end{verbatim} 

ou
\begin{verbatim}
[P]{s1} = IN_DIR "in/";[/P]
\end{verbatim} 

... si on a utilisé la directive IN_DIR dans le fichier des spécifications de la simulation.

\item On demande la création d'une {\em nouvelle instance d'événement} de nom de classe {\small {\tt nom_c}} et de nom d'instance {\small {\tt nom_i}} (noter que les deux informations sont requises et que, comme pour les entités, elles ont la forme d'une chaîne simple ou multiple) par :
\begin{verbatim}
[P]+ V nom_c nom_i;[/P]
\end{verbatim}
Cette requête invoque un constructeur d'événement codé dans le fichier [F]UserFunction.cc[/F]. Ce constructeur est déterminé dans la fonction [Y]App_NewEventFromString[/Y] (voir page 'Fonctions globales/Création d'instances de classes') en fonction de la chaîne de caractères passée en argument (ici {\small {\tt nom_c}}). L'événement est inséré dans l'agenda d'événement de l'instance de [C]Simulation[/C] [Y]pCurrentSim[/Y] (voir page 'Variables globales').\ \
Si ce constructeur n'affecte pas explicitement de valeur à la {\em date d'occurence} ou à la {\em priorité}, les valeurs {\em 0} et {\em 99} leur sont attribuées respectivement par défaut.

\item On fait {\em référence à un événement déjà créé}, de classe {\small {\tt nom_c}} et nommé {\small {\tt nom_i}}, en écrivant :
\begin{verbatim}
[P]<V><nom_c nom_i>[/P]
\end{verbatim} 
ou, en n'utilisant que le nom d'instance :
\begin{verbatim}
[P]<V><,nom_i>[/P]
\end{verbatim} 

\item Des spécifications complémentaires peuvent être exprimées sur un événement. Dans la déclaration suivante, portant sur l'évenement nommé {\small {\tt nom_i}}, et successivement, on affecte une valeur à la {\em date d'occurence} (par tirage au sort uniforme entre deux bornes, ou directement (ici : 20), ou par une date calendaire), à la {\em priorité}, à la {\em probabilité d'occurrence} et, pour le cas où l'événement est autogénérable, une valeur aux délais minimum et maximum d'occurrence de l'événement autogénéré : 
\begin{verbatim}
[P]<V><,nom_i>
     //MIN_DATE_OCCUR = 0;
     //MAX_DATE_OCCUR = 30;
     DATE_OCCUR = 20;
     //DATE_OCCUR = 21/JAN/2008); // ou 21/JAN/2008 0/0/0)
     //DATE_OCCUR = 21/JAN/-9);   // millésime = celui à l'instantiation
     PRIORITE = 1;
	 PROBABILITE = 0.667;
     MIN_DELAY_NEXT = 5;
     MAX_DELAY_NEXT = 10;
;[/P]
\end{verbatim}
Au même niveau que les déclarations ci-dessus, on peut définir des directives en termes d'actions sur des processus  (et on le fait normalement de la manière suivante, en faisant obligatoirement référence à un ou des processus déjà créés) : 
\begin{verbatim}
[P]<V><,nom_i>
     PROCESS_ACTION = {<P><,nom_ip1> (INIT PROCEED),
                       <P><nom_cp, nom_ip2> (EXEC)};
;[/P]
\end{verbatim}
On peut aussi déclarer quelle sera l'entité touchée par un des processus gérés par l'événement (en faisant obligatoirement référence, d'une manière ou d'une autre, à une entité déjà créée). Par exemple : 
\begin{verbatim}
[P]<V><,nom_i>
     ENTITE nom_cp = <I><,nom_io>;
     //I nom_cp = <I><,nom_io>;   // forme equivalente;
     //ENTITE_CIBLE nom_cp = <I><,nom_io>;   // forme equivalente
;[/P]
\end{verbatim}
Si l'événement gère un processus de lecture de fichier séquentiel, on peut, au même niveau que les déclarations ci-dessus, déclarer l'une ou les autres des informations sur le nom du fichier à lire, avec les m\^emes expressions qu'au niveau du processus. Les spécifications sur l'événement surchargent celles faites sur le processus et sur le fichier lui-m\^eme.
 
\item On demande la création d'une {\em nouvelle instance de fichier en série d'enregistrements} de symbole de classe {\small {\tt symb_c}} par :
\begin{verbatim}
[P]+ SDF symb_c;[/P]
\end{verbatim}
Cette requête invoque un constructeur de [C]SequentialDataFile[/C] codé dans le fichier [F]UserFile.cc[/F]. Ce constructeur est déterminé dans la fonction [Y]App_NewSequentialDataFileFromString[/Y] (voir page 'Fonctions globales/Création d'instances de classes') en fonction de la chaîne de caractères passée en argument (ici {\small {\tt symb_c}}).\\

Ce paragraphe d'instantiation de fichier peut inclure les informations sur le chemin d'accès (répertoire, nom simple, nom complet) avec les m\^emes expressions que dans la déclaration d'un processus de lecture. Ce qu'on écrit ici vient surcharger les instructions de m\^eme effet placées dans le constructeur.\\

On peut aussi ajouter la directive d'ouverture du fichier, da la manière simple suivante :
\begin{verbatim}
[P]OUVERTURE;[/P]
\end{verbatim}
\item A tout moment dans la séquence de spécifications, on peut demander {\em l'affichage du contenu de l'agenda} d'évenements, par {\small {\tt AFFICHER AGENDA;}}

\item Toute fin de ligne après {\small {\tt //}} ou {\small {\tt \#}} est ignorée, de même que tout passage entre /$*$ et la fin d'une ligne commençant par $*$/.

\item Toute spécification de structure {\small {\tt <spec>}} peut être exprimée conditionnellement. {\small {\tt <spec>}} peut être simple ou complexe et inclure une spécification conditionnelle : 
\begin{verbatim}
[P]SI <condition> <spec> FIN SI
SI <condition> <spec> SINON <altSpec> FIN SI[/P]
\end{verbatim}
avec :
\begin{verbatim}
[P]<condition>         ::= <num_expr> <op_num> <num_expr> 
                    ::= <str_expr> <op_str> <num_expr> 
<op_num>            ::= < | <= | > | >= 
<op_str>            ::= == | !=[/P]
\end{verbatim}

\item Certaines spécifications de structure peuvent être itérées, sous le contrôle d'une variable entière. Il s'agit de la déclaration/affectation d'une variable, des spécifications de création d'entité, de processus et d'événement. Il s'agit aussi de l'ajout d'éléments aux entités. Noter qu'on ne peut pas itérer l'ajout de composants et l'affectation de valeurs aux descripteurs, ni l'affectation de valeurs aux attributs des processus et des événements. On peut imbriquer plusieurs boucles, sous le contrôle de plusieurs variables différentes. On peut ainsi écrire :
\begin{verbatim}
[P]{x1} = 4.0;
POUR i1 = (1) TO (3*<int_expr>)
    + I classeA,
        POUR i2 = (1) A (4)
            + E classeB inst{i2}
                x =: {x1};
            ;
        FIN POUR
    ;
FIN POUR
;[/P]
\end{verbatim}
Attention ! L'itérateur i2 ne peut pas servir d'opérateur arithmétique dans la mise en oeuvre de la boucle. On ne peut pas écrire, par exemple :
\begin{verbatim}
[P]...
            + E classeB inst{i2}
                x =: {x1}*{i2};
            ;
...[/P]
\end{verbatim}
\end{itemize}
*/

/**@name Le fichier des directives de simulation
\\
index{fichier des directives de simulation}
Ce fichier est lu (et son contenu exploité) par un appel à la fonction [Y]ParseDirectiveFile[/Y]. Cette fonction prend en argument le nom physique du fichier. Cet appel ne peut figurer que dans le programme principal.  
\begin{verbatim}
[P]int main(int argc,  char* argv[]) {
  ...
  ParseParameterFile(argv[1]);
  ParseSimulationFile(argv[2]);
  ...
  ParseDirectiveFile(argv[5]);
  //pCurrentSim->Run();
  ...
}[/P]
\end{verbatim}

Le nom du fichier peut figurer dans l'instruction (et donc compilé) ou, plus généralement, être passé en argument de la ligne de commande :
\begin{verbatim}
[P]./main  sim1.par  sim1.sim  sim1.osp  sim1.str  sim1.dir[/P]
\end{verbatim}

Dans le programme principal, l'appel à [Y]ParseDirectiveFile[/Y] doit figurer après l'appel à [Y]ParseParameterFile[/Y] si une directive (notamment celle qui lance l'éxécution par [Y]Run[/Y]) requiert que des paramètres du système soient déjà connus en mémoire (donc accessibles par [Y]Get..ParameterValue[/Y]).\\

De même, il doit figurer après l'appel à [Y]ParseSimulationFile[/Y] si est déclarée une directive d'éxécution (par [Y]Run[/Y]), laquelle requiert l'instanciation préalable d'une instance de [C]Simulation[/C].\\ 

Le fichier des directives de simulation est une suite de phrases, dont la syntaxe est la suivante\label{directiveFile} :

\begin{verbatim}
[P]<fichier>         ::= <phrase>*
<phrase>          ::= <ligne_trace> | <ligne_chrono> | <ligne_message> |
                      <parse_param> | <parse_os>     | <parse_struct>  | 
                      <init_rand>   | <init_simu>    | <run_simu>      |   
                      <recup_mem>   | <display_os>   | <report_mem>    |
                      <ligne_exit>

<ligne_trace>     ::= TRACE <mode_trace> <fin>
<mode_trace>      ::= PARSING | NO_PARSING | ACTION | NEWDEL

<ligne_chrono>    ::= CHRONO <instr_chrono> <fin> |
                      CHRONO DISPLAY [<message>] <fin>
<instr_chrono>    ::= INIT | START | PAUSE | CLOSE 
<message>         ::= <submessage>*
<submessage>      ::= <chaine> | <multichaine> | <format> | ENDL | TAB | CLOCK
<chaine>          ::= [a-zA-Z][a-zA-Z0-9\_\.]*
<multichaine>     ::= "<chaine_m>"
<chaine_m>        ::= [a-zA-Z(\ )*][a-zA-Z0-9\_\.\ ]*
<format>          ::= "<chaine_f>"
<chaine_f>        ::= [a-zA-Z0-9\+\-\*\/\,\_\.\:\!\?\%\'\"\ \=\\]*

<ligne_message>   ::= MESSAGE <message> <fin> 

<parse_param>     ::= LECTURE PARAMETRE dir_path <fin>
<dir_path>        ::= "[/]<etage>*"
<etage>           ::= <chaine>/ | ../
<chaine>          ::= [a-zA-Z][a-zA-Z0-9\_\.]* 

<parse_os>        ::= LECTURE OUTPUTSPEC dir_path <fin>

<parse_struct>    ::= LECTURE STRUCTURE dir_path <fin>

<init_rand>       ::= INITRAND <entier> <fin> |
                      INITRAND <fin>  

<init_simu>       ::= INIT SIMULATION dir_path <fin> 

<run_simu>        ::= RUN [<entier>] <fin>

<recup_mem>       ::= DELETE <spec_recup> <fin>
<spec_recup>      ::= <spec_recup_entite> | 
                      MONITOR | PARAMETER | OUTPUTSPEC | FICHIER | SIMULATION | 
                      ALL
<spec_recup_ent>  ::= ENTITE [<nom_classe>] INSTANCE |
                      ENTITE [<nom_classe>] CLASSE 
<nomclasse>       ::= <chaine> | <multichaine>

<display_os>      ::= DISPLAY OUTPUTSPEC <spec_display_os> <fin>
<spec_display_os> ::= <spec_dspl_ent> | 
                      PROCESS | CPROCESS | DPROCESS | EVENT 
<spec_dspl_ent>   ::= ENTITE [<nom_classe>]

<report_mem>      ::= RAPPORT MEMOIRE <fin> 

<ligne_exit>      ::= EXIT <fin> 

<fin>             ::= ;[/P]
\end{verbatim}

\begin{itemize}
\item {\small {\tt <ligne_trace>}} permet d'activer ou de désactiver une fonction de trace des actions de lecture/inter\-prétation. L'option ACTION donne la valeur TRACE_ACTION_ON à la variable globale [P]gTraceActionMode[/P]. L'option NEWDEL donne la valeur TRACE_NEWDEL_ON à la variable globale [P]gTraceNewDelMode[/P] (voir page 'Fonctions globales/Interprétation des arguments de la commande de simulation').

\item {\small {\tt <ligne_chrono>}} permet de gérer un chronomètre. L'option INIT crée le chronomètre et initialise le compteur de temps à zéro. L'option START déclenche le chronomètre après la création ou un arrêt provisoire. L'option PAUSE l'arrête provisoirement et l'option CLOSE l'arrête définitivement. L'option PAUSE calcule le temps écoulé depuis le dernier déclenchement et ajoute cette valeur au compteur de temps. Bien noter que donner deux instructions PAUSE successives sans instruction START entre les deux a le même effet qu'avoir donné seulement la seconde instruction PAUSE. A une option INIT doit correspondre une et une seule option CLOSE.  
\noLF
L'option DISPLAY affiche la valeur du compteur de temps, c'est-à-dire le temps cumulé sur les couples 'déclenchement/arrêt' successifs. On peut inclure un message dans la ligne affichée, avant l'indication du temps. Ce message peut comprendre des tabulations (option TAB), des retours à la ligne (option ENDL) ou enfin l'indication de la valeur de l'horloge de la simulation (option CLOCK).

\item {\small {\tt <ligne_message>}} affiche un message sur la sortie standard, construit comme ceux associés à un chronométrage.

\item {\small {\tt <parse_param>}} demande la lecture/interprétation d'un fichier des paramètres du système. 

\item {\small {\tt <init_rand>}} initialise une série de nombres pseudo-aléatoires par un appel à [Y]srand(<entier>)[/Y] ou [Y]srand(<valeur de paramètre>)[/Y], ou bien par un appel à [Y]srand((int)getpid())[/Y] si l'entier n'est pas fourni ou est égal à 0. Utile pour provoquer des séries différentes (par la fonction [Y]RandomInteger[/Y]) lors de simulations successives. Les tirages aléatoires effectués avant l'interprétation de cette ligne (notamment ceux dans le fichier de la structure du système) ne sont pas gérés par cette graine. La graine peut aussi être initialisée dans le fichier des paramètres de la simulation. (voir page 'Le fichier des spécifications de simulation').

\item {\small {\tt <init_simu>}} demande la lecture/interprétation d'un fichier de spécification de la simulation. 
  
\item {\small {\tt <parse_struct>}} demande la lecture/interprétation d'un fichier de description de la structure du système.

\item {\small {\tt <parse_os>}} demande la lecture/interprétation d'un fichier de spécifications de sortie. Cette demande doit figurer après la spécification de la simulation, si celle-ci détermine (par le mot-clé {\small {\tt OUT_DIR}}) le répertoire d'accueil des fichiers de sortie.

\item {\small {\tt <run_simu>}} provoque l'invocation de la méthode [Y]Run[/Y] sur la simulation courante.

\item {\small {\tt <recup_mem>}} provoque, selon le contenu de {\small {\tt <spec_recup>}}, l'invocation des fonctions suivantes : [Y]DeleteMonitors[/Y], [Y]DeleteParameters[/Y], [Y]DeleteOutputSpecs[/Y], [Y]DeleteFileInstances[/Y]. \\
Avec le mot-clé {\small {\tt SIMULATION}}, on invoque [P]delete pCurrentSim;[/P].\\
Avec les mots-clés {\small {\tt ENTITE}} et {\small {\tt CLASSE}}, on invoque [Y]DeleteEntityClasses[/Y], argumenté ou non avec le symbole de la classe dont le nom est {\small {\tt <nom_classe>}}.\\
Avec les mots-clés {\small {\tt ENTITE}} et {\small {\tt INSTANCE}}, on invoque [Y]DeleteEntityInstances[/Y], argumenté ou non avec le symbole de la classe dont le nom est {\small {\tt <nom_classe>}}. \\
Avec le mot-clé {\small {\tt ALL}}, on invoque [P]DeleteAll();[/P].

\item {\small {\tt <display_os>}} provoque, selon le mot-clé, l'invocation d'une des fonctions de transfert sur fichier des informations sauvegardées en mémoire au titre des spécifications de sortie (voir page 'Les spécifications de sortie/Fonctions'). \ \
Avec le mot-clé {\small {\tt ALL}}, on invoque [/P]DisplayAllOutputSpecs()[/P].

\item {\small {\tt <report_mem>}} provoque l'invocation de la fonction [Y]MemoryReport[/Y] 

\item {\small {\tt <ligne_exit>}} fait que les lignes suivantes sont ignorées, comme un commentaire.
\end{itemize}

Un exemple de fichier de directives est le suivant :

\begin{verbatim}
[P]TRACE PARSING;
//TRACE NEWDEL;
//TRACE ACTION;

CHRONO INIT;
CHRONO START;

LECTURE PARAMETRE "parametres";
INIT SIMULATION "initSim";
LECTURE STRUCTURE "structure";
LECTURE OUTPUTSPEC "outspec.std";

RUN 1000;

MESSAGE ENDL + "temps simule = " + HORLOGE + ENDL;
CHRONO DISPLAY ENDL + "duree de la simulation = "; MESSAGE ENDL;
   
DISPLAY OUTPUTSPEC EVENT;
DISPLAY OUTPUTSPEC DPROCESS;
DISPLAY OUTPUTSPEC CPROCESS;
DISPLAY OUTPUTSPEC ENTITE entity;
MESSAGE ENDL + "fin sauvegarde des sorties" + ENDL;

DELETE PARAMETRE;  
DELETE OUTPUTSPEC; 
DELETE ENTITE INSTANCE;
DELETE MONITOR;
DELETE SIMULATION;
RAPPORT MEMOIRE;
DELETE ENTITE CLASSE;
MESSAGE ENDL + "fin nettoyage" + ENDL;
CHRONO CLOSE;[/P]
\end{verbatim}

Noter que le nom de classe {\small {\tt entity}} est celui de la classe de base [C]Entity[/C]. On demande alors la sauvegarde d'informations pour toutes les classes d'entités
*/

//@}  // fin fichiers déclaratifs (LexicalFile)


//----------------------------------------------------------------------------
// SerialDataFile
//----------------------------------------------------------------------------
/**@name Les fichiers de données en série d'enregistrements
Une simulation peut exploiter, en entrée, un ou plusieurs fichiers de données constitués d'une série d'enregistrements dont chacun, au-delà du premier, est associé (explicitement ou non) à un instant postérieur à l'instant auquel est associé le précédent.

Ces fichiers sont de l'un des deux types suivants, correspondant à deux sous-classes de [C]SerialDataFile[/C] :
\begin{itemize}
\item les fichiers de données séquentielles (classe [C]SequentialDataFile[/C]) : la série d'enregistrements correspond de manière bijective à une série d'intervalles temporels tous de la même longueur. C'est le cas typique d'un fichier de données climatiques journalières. Les enregistrements peuvent contenir un ou des champs identifiant explicitement l'intervalle temporel (typiquement, l'indication du jour). Ce n'est pas obligatoire parce que c'est la dynamique du processus de lecture qui attache un intervalle de temps à chaque enregistrement lu : le processus est lancé en un instant connu et lit un enregistrement à chaque pas de longueur fixe.
\item les fichiers de données factuelles (classe [C]FactualDataFile[/C]) : les enregistrements correspondent chacun à un instant explicitement précisé par un ou plusieurs de ses champs. Ils sont obligatoirement rangés dans le fichier dans l'ordre croissant des instants. Le fichier est doté d'une méthode ([Y]GetRecordClockValue[/Y]) dont le corps doit être écrit par l'utilisateur, et qui renvoie l'instant auquel est associé l'enregistrement. 
\end{itemize}

Pour chaque fichier de données, le développeur doit donc créer une sous-classe de [C]SequentialDataFile[/C] ou de [C]FactualDataFile[/C], et insérer dans le code du simulateur la création d'une instance de la sous-classe concernée\\

 La classe [C]SerialDataFile[/C] posséde comme attributs :

\begin{enumerate}
\item un attribut qui spécifie les {\em champs}\index{champ (d'un enregistrement)} utiles de l'enregistrement\index{enregistrement (d'un fichier)}, dans l'ordre de leur position sur la ligne. Ces champs sont de deux types : les champs de clé\index{clé (d'un enregistrement)} et les champs de données proprement dit\index{données (d'un enregistrement)}. La valeur de cet attribut [Y]mpFieldTab[/Y] (de type {\em pFieldTab}, voir page 'Constantes symboliques générales/Champ d'un enregistrement de fichier') est un tableau de couples {\em type du champ}-{\em valeur du champ} ;
\item une spécification de {\em format de lecture}\index{format (d'un enregistrement)} ; c'est l'attribut [Y]mFormat[/Y] ;
\item une {\em fonction de lecture} \index{fonction de lecture} exploitant le format ci-dessus et affectant une valeur aux éléments de {\em mpFieldTab}.
\item seulement pour la sous-classe [C]FactualDataFile[/C], une {\em fonction de renvoi de date} \index{fonction de renvoi de date} qui sait reconnaître l'instant associé à l'enregistrement parmi ses champs, et qui le renvoie.
\end{enumerate}

En outre, [C]SerialDataFile[/C] hérite de [C]File[/C] un attribut (de nom [Y]mpFile[/Y]) dont la valeur, attribuée par le moteur au moment de l'ouverture du fichier, est un pointeur sur une structure FILE de C.\\[5mm]

{\bf Remarques :}
\begin{itemize}
\item 
Les enregistrements d'un fichier de données doivent tous comporter le même nombre de champs, séparés par un ou plusieurs espaces ou tabulations. 
\item 
Ces champs doivent avoir le même type et la même signification dans tous les enregistrements. 
\item 
Par contre, pour chaque enregistrement, un champ peut avoir une longueur variable (en nombre de caractères).  
\end{itemize}

  Par exemple, pour la ligne de référence suivante\ldots
\begin{verbatim}
     0 Ut = -1 Xtp1 = -0.501177 Vtp1 = -0.00117684
\end{verbatim}

  \ldots les lignes suivantes sont correctement écrites :
\begin{verbatim}
     2   cUt = -1  Xtp1 = -0.507017   Vtp1 = -0.00349537
     300 Ut = -100 Xtp1 = -0.37       Vtp1 = -0.1968
     41  Ut = -1   Xtp1 = -0.6        Vtp1 = -0.005737
\end{verbatim}

  \ldots alors que l'ensemble de lignes suivant ne l'est pas :
\begin{verbatim}
     1 Ut = -1 Xtp1 = -0.503522      = -0.00234488 (manque la chaîne "Vtp1")
     2 Ut = -1 Xtp1 = -0.507017 Vtp1   -0.00349537 (manque la chaîne "=")
       Ut = -1 Xtp1 = -0.517346 Vtp1 = -0.00570937 (manque le premier champ)
\end{verbatim}
\begin{itemize}
\item La valeur de l'attribut {\small {\tt mFormat}} peut être affectée de 2 façons différentes (voir illustration plus bas) :
\begin{enumerate}
\item par la méthode [Y]AssignFormat[/Y]

\item par les méthodes [Y]AssignKeyFormat[/Y] et [Y]AssignDataFormat[/Y], auquel cas le format de lecture sera la concaténation des 2 formats partiels. Noter cependant que les champs lus sous le format de clé n'ont pas, pour la présente classe, de rôle particulier dans l'accès aux enregistrements. Par contre, l'utilisateur peut les exploiter pour vérifier l'identification des valeurs des données lues.
\end{enumerate}

La spécification de format est une chaîne de caractères de même syntaxe que la chaîne de contrôle des instructions d'entrée/sortie de C. En particulier, les sous-chaînes "\%d","\%s","\%f" et "\%lf" captent respectivement sur la ligne lue un entier, une chaîne de caractères, un flottant et un double. 

\item La valeur de l'attribut {\em fonction de lecture} est affectée par la méthode [Y]AssignReadNextEnrgtMethod[/Y].\\ 
La lecture d'un enregistrement se fait par un appel à la méthode [Y]ReadNextEnrgt[/Y]. Pour récupérer ensuite la valeur d'un des champs de l'enregistrement, il suffit d'appeler l'une des fonctions suivantes : [Y]GetIntFieldValue[/Y], [Y]GetFloatFieldValue[/Y], [Y]GetDoubleFieldValue[/Y] ou [Y]GetStringFieldValue[/Y], selon le type de la valeur du champ.  

\item Pour la sous-classe [C]FactualDataFile[/C], la valeur de l'attribut {\em fonction de lecture} est affectée par la méthode [Y]AssignGetRecordClockValueMethod[/Y].\\ 
La recherche de l'instant associé à l'enregistrement se fait par un appel à la méthode [Y]GetRecordClockValue[/Y]. La valeur renvoyée doit être une valeur de l'horloge de la simulation. 
\end{itemize}


\ \vspace{10mm}
{\bf Exemple de réalisation et d'exploitation d'un fichier de données :} \\ 

{\small {\tt 1) dans [F]UserType.h[/F]}}

\begin{verbatim}
[P]         // Symboles de classe des fichiers en série d'enregistrements.
enum App_FileId {F_TOBOGGAN = 11};[/P]
\end{verbatim}

{\small {\tt 2) dans [F]UserFile.h[/F]}}

\begin{verbatim}
[P]// Fonction de lecture spécifique du fichier de données.
//
Toboggan_ReadNextEnrgt(SequentialDataFile*);
 
// Enumération de tous les champs d'un enregistrement. Les symboles
// sont arbitraires. Les valeurs entières (positives ou nulles)  
// correspondantes sont optionnelles.
// 
enum  F_Toboggan_Fields {
  COMMAND, POSITION, NUMLIGNE, VITESSE
};[/P]
\end{verbatim}

{\small {\tt 3) dans [F]UserFile.cc[/F]}}

\begin{verbatim} 
[P]#include "UserFile.h"
 
  void CreationFichierToboggan()
//---------------------------------------------------------------------------- 
// Fonction de création du fichier de données.
//----------------------------------------------------------------------------
{
	// création de l'instance
  SequentialDataFile* pF = 
    new SequentialDataFile(F_TOBOGGAN, READ);

	// création du tableau des attributs correspondant aux champs
	// respecter l'ordre des champs sur la ligne
  pF->AddField(NUMLIGNE, INT  );
  pF->AddField(COMMAND,  INT  );
  pF->AddField(VITESSE,  FLOAT);
  pF->AddField(POSITION, FLOAT);
 
	// affectation d'une fonction de lecture 
  pF->AssignReadNextEnrgtMethod(Toboggan_ReadNextEnrgt);

	// désignation du nom physique du fichier ; réalise l'ouverture 
  pF->AssignFullName("data_toboggan.txt");

	// affectation d'un format de lecture lu en paramètre
  pF->AssignFormat(GetStringParameterValue("f_toboggan", "format");
};
 
int Toboggan_ReadNextEnrgt(SequentialDataFile* pF) 
//----------------------------------------------------------------------------
// Fonction de lecture d'un enregistrement.
// Retourne : le nombre de champs lus ou 'EOF'
//----------------------------------------------------------------------------
{
  return fscanf(pF->GetPhysicalFile(),
           pF->GetFormat(), 
           &pF->mpFieldTab[NUMLIGNE]->mFieldValue.intValue,
           &pF->mpFieldTab[COMMAND] ->mFieldValue.intValue,
           &pF->mpFieldTab[VITESSE] ->mFieldValue.floatValue,
           &pF->mpFieldTab[POSITION]->mFieldValue.floatValue);
};[/P]
\end{verbatim}

{\small {\tt 4) dans les fonctions utilisant les champs d'un enregistrement :}}

\begin{verbatim}
[P]CreationFichierToboggan();
while(ReadNextEnrgt(F_TOBOGGAN) != EOF) {
  float MyValue = GetFloatFieldValue(F_TOBOGGAN, VITESSE));
  ...
}[/P]
\end{verbatim}
*/

//@{

/**@name CDPFORTEXLa classe de base
\begin{itemize} \item Directive d'inclusion :
[P]
\#include File.h
[/P]
\end{itemize}
*/
//@{
  /// 
class SerialDataFile : public File 
{
public:
/**@name LIFECYCLE */
//@{
	/// Constructeur de base. 
  SerialDataFile(); // _no swig_

	/// Constructeur. Le premier argument est un symbole associé au fichier, et doit correspondre à une valeur entière strictement positive. Le second argument est le type d'accès (READ ou WRITE).
  SerialDataFile(UserFileId, AccesTypeId); // _no swig_

	// Destructeur du fichier de données séquentielle (instance de la classe [C]SerialDataFile[/C] et fermeture du fichier ; le fichier en tant que lieu physique n'est évidemment pas touché). 
  ~SerialDataFile(); // _no swig_
//@}

/**@name  [SC]Manipulation du format[/SC] */
//@{
	/// Assigne le format complet d'un enregistrement. \index{méthodes!AssignFormat}
  void   AssignFormat(char*); // _no swig_

	/// Assigne le format complet d'un enregistrement. Le premier argument correspond à la clé, le second correspond aux données. 
  void   AssignFormat(char*, char*); // _no swig_

	/// Assigne le format de la clé d'un enregistrement. \index{méthodes!AssignKeyFormat}
  void   AssignKeyFormat (char*); // _no swig_

	/// Assigne le format des données d'un enregistrement. \index{méthodes!AssignDataFormat}
  void   AssignDataFormat(char*); // _no swig_

	/// Renvoie le format complet d'un enregistrement.
  char*  GetFormat(); // _no swig_

	/// Renvoie le format de la clé d'un enregistrement.
  char*  GetFormatKey(); // _no swig_

	/// Renvoie le format des données d'un enregistrement.
  char*  GetFormatData(); // _no swig_
//@}

/**@name  [SC]Manipulation des champs de l'enregistrement[/SC] */
//@{

	/// Ajoute un champ pour un enregistrement en spécifiant son type. Le premier argument, valué seulement pour mémoire, n'est pas exploité dans la méthode. Les valeurs possibles pour l'énumération {\em ValueTypeId} sont celles listées page 'Constantes symboliques générales/Types de valeurs', sauf le type P_ENTITY.\index{méthodes!AddField (SerialDataFile)}
  void   AddField(int, ValueTypeId); // _no swig_ 	

	/// Renvoie la valeur de type entier du champ dont le symbole est passé en argument, dans l'enregistrement courant du fichier. \index{méthodes!GetIntFieldValue}
int      GetIntFieldValue        (int);
	 
	/// Renvoie la valeur de type flottante du champ dont le symbole est passé en argument, dans l'enregistrement courant du fichier. \index{méthodes!GetFloatFieldValue}
float    GetFloatFieldValue      (int);

	/// Renvoie la valeur de type flottante en double précision du champ dont le symbole est passé en argument, dans l'enregistrement courant du fichier. \index{méthodes!GetDoubleFieldValue}
double   GetDoubleFieldValue     (int);

	/// Renvoie la chaîne de caractère, valeur du champ dont le symbole est passé en argument, dans l'enregistrement courant du fichier. \index{méthodes!GetStringFieldValue}
char*    GetStringFieldValue     (int);

	/// Affiche le contenu de l'enregistrement courant.\index{méthodes!Display (SerialDataFile)}
  void   Display    ();	 // _no swig_
//@}

/**@name  [SC]Gestion de l'entête du fichier[/SC] */
//@{

        /// Détermine le nombre de lignes à partir du début du fichier qui ne sont pas des enregistraments de données. La valeur par défaut est {\em 0}.\index{méthodes!SetNbTopCommentLines}
  void   SetNbTopCommentLines(int); // _no swig_

        /// Renvoie le nombre de lignes à partir du début du fichier qui ne sont pas des enregistraments de données.\index{méthodes!GetNbTopCommentLines}
  int    GetNbTopCommentLines(); // _no swig_ 

        /// Saute {\em n} enregistrements (lignes) dans le fichier physique associé au récepteur de ce message. Le fichier doit être ouvert. Si le fichier {\em f} vient juste d'être ouvert (aucune lecture réalisée), un appel [P]f->Skip(3);[/P] positionne la prochaine lecture sur le quatrième enregistrement.\index{méthodes!Skip}
  void   Skip(int n);
//@}


  pFieldTab	mpFieldTab;	        // les champs en vigueur d'un enreg.


  protected: 

  int		mFieldTabSize;     	// le nombre de champs
  char*		mFormatKey;	     	// Format de la clé   d'un enreg.
  char*		mFormatData;	     	// Format des données d'un enreg.
  char*		mFormat;	     	// Format complet     d'un enreg.
  int       mNbTopCommentLines;     // + JPR 021220

}; // _end class_ (Serial)
//@}

/**@name CDPFORTEXLa classe des fichiers séquentiels
\begin{itemize} \item Directive d'inclusion :
[P]
\#include File.h
[/P]
\end{itemize}
*/
//@{
  /// 
class SequentialDataFile : public SerialDataFile 
{
public:
/**@name LIFECYCLE */
//@{
	/// Constructeur de base. 
  SequentialDataFile(); // _no swig_

	/// Constructeur. Le premier argument est un symbole associé au fichier, et doit correspondre à une valeur entière strictement positive. Le second argument est le type d'accès (READ ou WRITE).
  SequentialDataFile(UserFileId, AccesTypeId); // _no swig_

	// Destructeur du fichier de données séquentielle (instance de la classe [C]SequentialDataFile[/C] et fermeture du fichier ; le fichier en tant que lieu physique n'est évidemment pas touché). 
  ~SequentialDataFile(); // _no swig_
//@}

/**@name [SC]Désignation de la procédure de lecture[/SC] */
//@{

        /// Réalise la liaison entre une fonction écrite par l'utilisateur et la méthode prédéfinie connue pour lire séquentiellement un fichier. \index{méthodes!AssignReadNextEnrgtMethod}
  void   AssignReadNextEnrgtMethod(int (* f)(SequentialDataFile*));// _no swig_

	// Fonction de lecture séquentielle d'un fichier. Cette fonction fait appel à une fonction écrite par l'utilisateur, et liée par la méthode [Y]AssignReadNextEnrgtMethod[/Y] décrite ci-dessus.
  int    ReadNextEnrgt(void); // _no swig_

//@}
  protected: 	

  int   (*CoreReadNextEnrgt) (SequentialDataFile*); // _no swig_

}; // _end class_ (SequentialDataFile)
//@}

/**@name CDPFORTEXLa classe des fichiers factuels
\begin{itemize} \item Directive d'inclusion :
[P]
\#include File.h
[/P]
\end{itemize}
*/
//@{
  /// 
class FactualDataFile : public SerialDataFile 
{
public:
/**@name LIFECYCLE */
//@{
	/// Constructeur de base. 
  FactualDataFile(); // _no swig_

	/// Constructeur. Le premier argument est un symbole associé au fichier, et doit correspondre à une valeur entière strictement positive. Le second argument est le type d'accès (READ ou WRITE).
  FactualDataFile(UserFileId, AccesTypeId); // _no swig_

	// Destructeur du fichier de données séquentielle (instance de la classe [C]FactualDataFile[/C] et fermeture du fichier ; le fichier en tant que lieu physique n'est évidemment pas touché). 
  ~FactualDataFile(); // _no swig_
//@}

/**@name [SC]Désignation de la procédure de lecture[/SC] */
//@{

        /// Réalise la liaison entre une fonction écrite par l'utilisateur et la méthode prédéfinie connue pour lire séquentiellement un fichier. \index{méthodes!AssignReadNextEnrgtMethod}
  void   AssignReadNextEnrgtMethod(int (* f)(FactualDataFile*));// _no swig_

	// Fonction de lecture séquentielle d'un fichier. Cette fonction fait appel à une fonction écrite par l'utilisateur, et liée par la méthode [Y]AssignReadNextEnrgtMethod[/Y] décrite ci-dessus.
  int    ReadNextEnrgt(void); // _no swig_ 

        /// Réalise la liaison entre une fonction écrite par l'utilisateur et la méthode prédéfinie connue pour rechercher et renvoyer la valeur d'horloge associée à l'enregistrement lu sur le fichier de données factuelles.\index{méthodes!AssignGetRecordClockValueMethod}
  void   AssignGetRecordClockValueMethod(int (* f)(FactualDataFile*));// _no swig_

//@}

/**@name  [SC]Manipulation des champs de l'enregistrement[/SC] */
//@{
	/// Invoque la méthode de même nom attachée à la classe [C]SerialDataFile[/C], et la complète en ajoutant un champ aux deux structures dont l'une reçoit les données lors de leur lecture, et l'autre les données en transition entre la précédente et la structure des données en cours.\index{méthodes!AddField (FactualDataFile)}
  void   AddField(int, ValueTypeId); // _no swig_

	/// Renvoie la valeur de type entier du champ dont le symbole est passé en argument, dans l'enregistrement courant du fichier. C'est la dernière valeur physiquement lue qui est renvoyée, avant éventuelle affectation comme valeur en cours.\index{méthodes!GetIntFieldReadValue}
int      GetIntFieldReadValue        (int);
	 
	/// Renvoie la valeur de type flottante du champ dont le symbole est passé en argument, dans l'enregistrement courant du fichier. C'est la dernière valeur physiquement lue qui est renvoyée, avant éventuelle affectation comme valeur en cours.\index{méthodes!GetFloatFieldReadValue}
float      GetFloatFieldReadValue        (int);
	 
	/// Renvoie la valeur de type flottante double précision du champ dont le symbole est passé en argument, dans l'enregistrement courant du fichier. C'est la dernière valeur physiquement lue qui est renvoyée, avant éventuelle affectation comme valeur en cours.\index{méthodes!GetDoubleFieldReadValue}
double      GetDoubleFieldReadValue        (int);	
	 
	/// Renvoie la valeur 'chaîne de caractères' du champ dont le symbole est passé en argument, dans l'enregistrement courant du fichier. C'est la dernière valeur physiquement lue qui est renvoyée, avant éventuelle affectation comme valeur en cours.\index{méthodes!GetStringFieldReadValue}
char*      GetStringFieldReadValue        (int);


//@}

/* *@name  ACCESS */
// @{

    // 
void      IsBufferEmpty(Boolean);
    // 
Boolean   IsBufferEmpty();
    // 
void      IsDataEmpty(Boolean);
    // 
Boolean   IsDataEmpty();

    // 
void      SetReaderProcess(Process*);
    // 
Process*  GetReaderProcess();

    // 
void      ReadTabToBufferTab();
    // 
void      BufferTabToDataTab();
    // 
void      ReadTabToDataTab();

// @}

  pFieldTab	mpReadFieldTab;		// les champs d'un enreg. lu
  pFieldTab	mpBufferFieldTab;	// les champs en attente vers FieldTab


  int   (*GetRecordClockValue) (FactualDataFile*); // _no swig_ 

  protected: 	
  int   (*CoreReadNextEnrgt) (FactualDataFile*); // _no swig_

  Boolean   mIsBufferEmpty;     // + JPR 091215
  Boolean   mIsDataEmpty;     // + JPR 101012

  Process*  mpReaderProcess;

}; // _end class_ (Factual)
//@}

// ---------------
// EXTERNAL FUNCTIONS
// ---------------
/**@name Fonctions 
\\
{\large {\tt int    ReadNextEnrgt           (UserFileId)}} :

\DTab4 Fonction de lecture du prochain enregistrement d'un fichier en série d'enregistrements. 
Cette fonction fait appel à une fonction écrite par l'utilisateur, et liée par la méthode [Y]AssignReadNextEnrgtMethod[/Y]. La valeur retournée est le nombre de champs lus, ou bien EOF (c'est-à-dire -1) si la fin du fichier a été atteinte.\index{fonctions!ReadNextEnrgt}

{\large {\tt int    GetIntFieldValue        (UserFileId, int)}} :

\DTab4 Renvoie la valeur de type entier du champ dont le symbole\ldots
 est passé en deuxième argument, dans l'enregistrement courant du fichier de symbole {\em UserFileId}. \index{fonctions!GetIntFieldValue}

{\large {\tt float  GetFloatFieldValue      (UserFileId, int)}} :

\DTab4 Renvoie la valeur de type flottante du champ dont le symbole\ldots
 est passé en deuxième argument, dans l'enregistrement courant du fichier de symbole {\em UserFileId}. \index{fonctions!GetFloatFieldValue}

{\large {\tt double GetDoubleFieldValue     (UserFileId, int)}} :

\DTab4 Renvoie la valeur de type flottante en double précision du champ dont le symbole\ldots
 est passé en deuxième argument, dans l'enregistrement courant du fichier de symbole {\em UserFileId}. \index{fonctions!GetDoubleFieldValue}

{\large {\tt char*  GetStringFieldValue     (UserFileId, int)}} :

\DTab4 Renvoie la chaîne de caractère, valeur du champ dont le symbole\ldots
 est passé en deuxième argument, dans l'enregistrement courant du fichier de symbole {\em UserFileId}. \index{fonctions!GetStringFieldValue}

{\large {\tt SequentialDataFile* GetSequentialDataFile(UserFileId)}} :

\DTab4 Renvoie un pointeur sur l'instance de fichier séquentiel dont le symbole est passé en argument.\index{méthodes!GetSequentialDataFile}

*/
//@{
int    ReadNextEnrgt           (UserFileId);

int    GetIntFieldValue        (UserFileId, int);

float  GetFloatFieldValue      (UserFileId, int);

double GetDoubleFieldValue     (UserFileId, int);

char*  GetStringFieldValue     (UserFileId, int);

SequentialDataFile* GetSequentialDataFile(UserFileId);

	// Cette méthode n'est réalisée nulle part : obsolète ?
void   AssignFileReadNextEnrgt (UserFileId, int (* function)(SequentialDataFile*)); // _no swig_

	// 
OutputFile* GetOutputFile(char* className); // _no swig_

//@}  // fin fonctions SDF

//@}  // finSDF

//----------------------------------------------------------------------------
// OutputFile
//----------------------------------------------------------------------------
/**@name Les fichiers de sortie banalisés
\\
Cette classe permet de créer et de gérer des fichiers de sortie à écriture séquentielle. L'utilisateur de \Db crée une instance de cette classe pour chaque fichier externe désiré, puis utilise librement les instructions d'écriture sur fichier fournies par C++.  \\
Il est important de noter qu'un pointeur sur l'instance de la classe [C]FILE[/C] correspondant au fichier physique est renvoyé par la méthode [Y]GetPhysicalFile[/Y]. Ainsi, les instructions suivantes permettent de créer et d'écrire sur le fichier [F]myFile.txt[/F] dans le répertoire père du répertoire courant : \\

\begin{verbatim}
[P]  OutputFile* f = new OutputFile("../myFile.txt", NEW);
  FILE* pF = f->GetPhysicalFile();
  fprintf(pF,"\%s","un texte quelconque");[/P]
\end{verbatim}
*/
//@{
  /// 
class OutputFile : public File  // _no swig_
{
public:
  /**@name  LIFECYCLE */
//@{
	/// Constructeur. Le premier argument est le nom physique du fichier. Ce nom est concaténé par le présent constructeur avec un éventuel préfixe renvoyé par le message [Y]GetOutDir[/Y] adressé à la simulation courante.\label{defOutDir}. Le second argument est le mode d'écriture (APPEND ou NEW : voir la section 'Enumération des modes d'écriture sur les fichiers').
  OutputFile  (char*, OutputMode);

	// Destructeur du fichier (instance de la classe [C]OutputFile[/C] et fermeture du fichier ; le fichier en tant que lieu physique n'est évidemment pas touché). 
  ~OutputFile();
//@}

/* *@name  ACCESS */
// @{
  /*
	/// Affecte un mode d'écriture. \index{méthodes!SetOutputMode}
  void         SetOutputMode(OutputMode);

	/// Renvoie le mode d'écriture. \index{méthodes!GetOutputMode}
  OutputMode   GetOutputMode() const;	
  */


   static pFileTab  InstanceTab();
// @}

  void   Display    ();

  static pFileTab  mpInstanceTab;
	
  /*
  OutputMode     mMode;	
  */     	
};  // _end class_

//@} // fin classe


//@}  // fin fichiers


#endif  // _FILE_HH
