//============================================================================
// FILE: C_File.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe générique des fichiers séquentiels
//              objets de mémoire et de prédiction (C-DIESE)
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_FILE_HH
#define _C_FILE_HH


#include "C_CoreTypes.h"

/**@name Les fichiers de données pour la gestion
\label{classeFichierMemo}\index{données pour la gestion}
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_File.h} \end{itemize}
*/
//@{

// ----------------
// Enumérations utilisées par des fonctions externes aux classes.
// ----------------

enum C_FileClassType {			// Type de classe de CDIESE
  BUFFERIZED_DATA_FILE=6
};

//===========================================================================
//
// FILE
//
//===========================================================================
/**@name Les fichiers de données séquentielles mémorisées et prédites 
La classe {\bf BufferizedDataFile} est une spécialisation de {\bf SequentialDataFile} incluse dans la couche \D. Elle permet de maintenir au cours d'une simulation une structure de données incluant deux types d'information : les valeurs qu'ont prises certaines données séquentielles dans le passé, et des données interprétables comme des predictions sur la valeur que prendront les mêmes données dans le futur.\\

La compréhension de la présente documentation nécessite de bien connaître la notion de fichier de données séquentielles, présentée dans le chapitre 'Les fichiers de données en série d'enregistrements' de la documentation de la couche BASIC DIESE.\\

Pour chaque fichier de données, le développeur doit créer une sous-classe de {\bf BufferizedDataFile}, et insérer dans le code du simulateur la création d'une instance de la sous-classe concernée. La classe {\bf BufferizedDataFile} posséde par héritage de tous les attributs de {\bf SequentialDataFile}, plus les attributs spécifiques suivants :
\begin{itemize}
\item un tableau de M élements, chacun étant la "copie" d'un enregistrement du fichier séquentiel : celui correspondant à l'instant simulé courant, et M-1 autres pour celui correspondant à l'instant précédent, etc.
\item un tableau de P élements, chacun étant la "copie" d'un enregistrement du fichier séquentiel : celui correspondant à l'instant simulé suivant le courant, et P-1 autres pour celui correspondant à l'instant encore suivant, etc. On comprend que le moteur de simulation est amené, pour l'exploitation de ce type de fichier, à "prendre de l'avance" sur les autres processus.
\end{itemize}

La taille de chacun de ces tableaux est spécifiée dans deux méthodes spécifiques des  {\bf BufferizedDataFile} :
\begin{itemize}
\item {\bf BFSetMemoryScope}, pour laquelle le modélisateur doit coder un corps et l'affecter à la méthode par un appel à \bf AssignSetMemoryScopeMethod}. Ce corps renvoie la taille du tableau de la mémoire sur les instants simulés précédents.
\item {\bf BFSetPredictionScope}, pour laquelle le modélisateur doit coder un corps et l'affecter à la méthode par un appel à \bf AssignSetPredictionScopeMethod}. Ce corps renvoie la taille du tableau de la prédiction sur les instants simulés suivants. 
\end{itemize}

Ces méthodes sont à exploiter, pour la création des deux tableaux, dans l'initialisation du processus continu dédié à la lecture d'un fichier séquentiel de ce type : {\bf ReadBufferizedDataFileProcess}.

Pour le même processus, la méthode d'avancée exploite la méthode {\bf ReadNextEnrgt}, héritée de la classe mère {\bf SequentialdataFile}, mais redéfinie spécifiquement à ce niveau pour tenir compte de l'existence des tableaux de mémoire et de prédiction.  En effet, ce n'est plus sur le fichier physique que sont recherchées les valeurs champs pour l'instant courant, mais sur le tableau des prédictions. Plus précisément, l'élément de ce tableau qui représente la prédiction la plus proche est copié dans l'élement du tableau de la mémoire qui représente l'instant courant (par écrasement de l'élement de ce tableau représentant la mémoire la plus lointaine). Ensuite, la méthode {\bf ReadNextEnrgt} procède à une lecture physique d'un nouvel enregistrement et écrase l'élément du tableau des prédicion représentant la prédiction la plus proche, qui vient de passer en mémoire.

Alors que le corps de la méthode ReadNext d'un {\bf SequentialDataFile} devait être codé par le modélisateur, le corps de la méthode ReadNext d'un {\bf BufferizedDataFile} est entièrement prédéfini par le moteur de la couche CONTROL DIESE, pour assurer les tâches ci-dessus. Il reste au modélisateur à coder le corps d'une nouvelle méthode, spécifique des {\bf BufferizedDataFile}, {\bf BFScanNextEnrgt} qui scanne l'élément du tableau des prédiction qui représente la prédiction la plus proche. Ce corps a un contenu identique à celui de la méthode ReadNext d'un {\bf SequentialDataFile} (à structure d'enregistrement identique), à ceci prèe que l'appel à fscanf (lecture du fichier physique) est remplacé par un appel à sscanf (scan d'une chaîne de caractères en mémoire). En reprenant l'example donné dans la documentation des {\bf SequentialDataFile}, cela donne le code suivant :
 
\begin{verbatim}
int Toboggan_ScanNextEnrgt(BufferizedDataFile* pF, char* str)
//---------------------------------------------
// Fonction de scanning d'un enregistrement en mémoire.
//---------------------------------------------
{
  int i;
  i = sscanf (str,
           pF->GetFormat(), 
           &pF->mpFieldTab[NUMLIGNE]->mFieldValue.intValue,
           &pF->mpFieldTab[COMMAND] ->mFieldValue.intValue,
           &pF->mpFieldTab[VITESSE] ->mFieldValue.floatValue,
           &pF->mpFieldTab[POSITION]->mFieldValue.floatValue);
  return i;
};
\end{verbatim}

... où l'argument str est valué dans le corps de méthode ReadNext avec l'élément approprié du tableau des prédictions.\\

En résumé, le modélisateur d'un {\bf BufferizedDataFile} doit :
\begin{itemize}
\item créer une sous-classe de {\bf SequentialDataFile}
\item coder et affecter un corps pour les méthodes spécifiant les portéee de la mémoire et de la prédiction
\item coder et affecter un corps pour la méthode de scanning d'un enregistrement
\end{itemize}

Pour des compléments sur le codage du processus de lecture, se reporter au chapitre dédié.
*/
//@{
          ///
//----------------------------------------------------------------------------
class BufferizedDataFile : public SequentialDataFile
{
public:

/**@name  LIFECYCLE */
//@{ 
          //  Constructeur de base.
  BufferizedDataFile();
          //  Constructeur de base.
  BufferizedDataFile(UserFileId, AccesTypeId);
          //  Destructeur.
  ~BufferizedDataFile();
//@} 

/**@name  OPERATIONS */
//@{ 

        /// Réalise la liaison entre une fonction écrite par l'utilisateur et la méthode prédéfinie connue pour scanner en mémoire un enregistrement déjà lu sur un fichier de données séquentielles. \index{méthodes!AssignScanNextEnrgtMethod}
  void   AssignScanNextEnrgtMethod(int (* f)(BufferizedDataFile*, char*));// _no swig_ // + maquettage 081027

	// Fonction de scanning en mémoire d'un enregistrement déjà lu sur un fichier. Cette fonction fait appel à une fonction écrite par l'utilisateur, et liée par la méthode {\bf AssignScanNextEnrgtMethod} décrite ci-dessus.
  int    ScanNextEnrgt(char*); // _no swig_ // + maquettage 081027

        /// Réalise la liaison entre une fonction écrite par l'utilisateur et la méthode prédéfinie connue pour déterminer la portée de la mémoire associée au fichier.  C'est-à-dire le nombre de pas de simulation passés pour lesquels on aura accès aux valeurs enregistrées dans le fichier.\index{méthodes!AssignSetMemoryScopeMethod}
  void   AssignSetMemoryScopeMethod(int (* f)(UserFileId));// _no swig_ // + maquettage 081027

	// Fonction de détermination de la portée de la mémoire associée au fichier. Cette fonction fait appel à une fonction écrite par l'utilisateur, et liée par la méthode {\bf AssignSetPredictionScopeMethod} décrite ci-dessus.
  int    SetMemoryScope(UserFileId); // _no swig_ // + maquettage 081027

        /// Réalise la liaison entre une fonction écrite par l'utilisateur et la méthode prédéfinie connue pour déterminer la portée de la prédiction associée au fichier.  C'est-à-dire le nombre de pas de simulation passés pour lesquels on aura accès aux valeurs enregistrées dans le fichier.\index{méthodes!AssignSetPredictionScopeMethod}
  void   AssignSetPredictionScopeMethod(int (* f)(UserFileId));// _no swig_ // + maquettage 081027

	// Fonction de détermination de la portée de la prédiction associée au fichier. Cette fonction fait appel à une fonction écrite par l'utilisateur, et liée par la méthode {\bf AssignSetPredictionScopeMethod} décrite ci-dessus.
  int    SetPredictionScope(UserFileId); // _no swig_ // + maquettage 081027

           //
  void DisplayMemoryTab();
           //
  void DisplayPredictionTab();

           /// Les méthodes suivantes renvoient la somme des valeurs mémorisées pour le champ donné en premier argument, entre les bornes (comprises) spécifiées par les deux arguments suivants. La borne 0 correspond à l'enregistrement lu au pas courant de la simulation, la borne 1 correspond au pas précédent, etc.
  int SumUpMemoryIntFieldValue(int fieldId, int lb, int ub);
           /// 
  float SumUpMemoryFloatFieldValue(int fieldId, int lb, int ub);
           /// 
  double SumUpMemoryDoubleFieldValue(int fieldId, int lb, int ub);

           /// Les méthodes suivantes renvoient la somme des valeurs prédites pour le champ donné en premier argument, entre les bornes (comprises) spécifiées par les deux arguments suivants. La borne 0 (qui correspondrait au pas courant) est interdite. La borne 1 correspond à l'enregistrement qui sera lu au pas suivant de la simulation, etc.
  int SumUpPredictionIntFieldValue(int fieldId, int lb, int ub);
           /// 
  float SumUpPredictionFloatFieldValue(int fieldId, int lb, int ub);
           /// 
  double SumUpPredictionDoubleFieldValue(int fieldId, int lb, int ub);
           /// 
//@}  

/**@name  ACCESS */
//@{ 

           /// \index{méthodes!IsControlled}
  TableauString* GetMemoryTab();
           /// \index{méthodes!IsControlled}
  TableauString* GetPredictionTab();

           /// \index{méthodes!IsControlled}
  void SetMemoryTab(TableauString*);
           /// \index{méthodes!IsControlled}
  void SetPredictionTab(TableauString*);

           /// 
  char* GetPredictionTabItem(int k);

           /// 
  void SetMemoryTabItem(int k, char*);
           /// 
  void SetPredictionTabItem(int k, char*);

           /// 
  void AddMemoryTabItem(char*);
           /// 
  void AddPredictionTabItem(char*);

           /// 
  int GetMemoryCurrentRecordIndex();
           /// 
  int GetPredictionCurrentRecordIndex();

           /// 
  void SetMemoryCurrentRecordIndex(int k);
           /// 
  void SetPredictionCurrentRecordIndex(int k);

           /// 
  void UpdateMemoryCurrentRecordIndex();
           /// 
  void UpdatePredictionCurrentRecordIndex();

           /// 
  int GetMemoryOldestRecordIndex();
           /// 
  int GetPredictionNearestRecordIndex();

           /// 
  int GetMemoryPreviousRecordIndex(int refIndex);
           /// 
  int GetPredictionNextRecordIndex(int refIndex);

           // 
  Boolean IsEndOfFileReached();
           // 
  void IsEndOfFileReached(Boolean);   
	
  int   (*BFScanNextEnrgt) (BufferizedDataFile*, char*); // _no swig_ 
	
  int   (*BFSetMemoryScope) (UserFileId); // _no swig_ 
	
  int   (*BFSetPredictionScope) (UserFileId); // _no swig_

//@} 

private:

  TableauString* mMemoryTab;
  TableauString* mPredictionTab;

  int mMemoryCurrentRecordIndex;
  int mPredictionCurrentRecordIndex;

  Boolean isEndOfFileReached;

protected:
};
//@}

// ---------------
// EXTERNAL FUNCTIONS
// ---------------
/* *@name Fonctions */
// @{
	//
int BufferizedDataFile_ReadNextEnrgt(SequentialDataFile* pFarg);// _no swig_

	/// Suppression en mémoire de toutes les instances créées pour toutes les sous-classes de {\bf File}, y compris les instances de {\bf BufferizedDadaFile}. Réalise la fermeture des fichiers correspondants. Remarquer qu'en conséquence tous les fichiers sont fermés en même temps (en fait tous les fichiers de données séquentielles, puisque les fichiers de paramètres du système et de la simulation, les fichiers de description de la structure et le fichier de directives sont fermés, et l'instance supprimée, juste après leur exploitation).\index{fonctions!C_DeleteFileInstances}
void   C_DeleteFileInstances(void); // _no swig_

// @} // fin fonctions


//@} /// fin Les fichiers bufferisés

#endif  // _C_FILE_HH

