//============================================================================
// FILE: C_Operation.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe des opérations
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_OPERATION_HH
#define _C_OPERATION_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "C_CoreTypes.h"

 
/**@name Op\'erations sur le syst\`eme pilot\'e 
\label{classeOperation}
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_Operation.h} \end{itemize}

Une opération est une entité qui permet de spécifier la nature et les modalités d'un changement d'état d'un système piloté.\\

Les classes d'opérations dans un domaine d'application sont des instances d'une seule des trois classes {\bf UnitGranulatedOperation}, {\bf QuantityGranulatedOperation} et {\bf TimeGranulatedOperation}, qui sont elles-mêmes des particularisations de la classe générale {\bf Operation}.\\

{\bf descripteurs :} \\
Les descripteurs de type {\bf int} représentent un nombre d'unités de temps simulé, à partir du début de la simulation (voir \Db : Manuel de l'utilisateur). La valeur par défaut est en dernière colonne.\index{date de début (opération)}\index{date de fin (opération)} \\

{\scriptsize
\noindent\begin{tabular}{lllll}
INSTRUCTION\_CONTAINER & instructionContainer & l'instruction 'mère' & {\bf PrimitiveActivity*} &   \\
OPERATED\_ENTITY\_ID & opEntId & symbole de classe & {\bf int} & 0 (O\_ENTITY)  \\
                     &         & de l'entité opérée &          &   \\
STEP & step & pas de progression de l'effet & {\bf int} & 1 \\
PROGRESSION\_MODE & progMode & mode de progression & {\bf C\_ProgressionModeId} & UNIT  \\
SUSPENSION\_THRESHOLD & suspThreshold & seuil de suspension & {\bf int} & 100 \\
PRIORITY\_DEGREE & priorityDegree & degre de priorité & {\bf int} & 99 (la plus faible) \\
REQUIRED\_RESOURCES & reqResources & ressources requises & {\bf pEntityTab*} & NULL  \\
ACHIEVEMENT\_DEGREE & achievementDegree & degré de progression & {\bf float} &   \\
SITUATION & situation & état de l'opération & {\bf C\_StateId} & DORMANT  \\
BEG\_DATE & begDate & date de début & {\bf int} & -9  \\
END\_DATE & endDate & date de fin & {\bf int} & -9  \\
LAST\_RESTARTING\_DATE & restartDate & date de la dernière reprise & {\bf int} & -9  \\
DURATION & & durée de début à fin & {\bf int} & 0  \\
NET\_DURATION & & durée hors suspensions & {\bf int} & 0  \\
\end{tabular}
}

{\bf méthodes :}\\

{\scriptsize 
\noindent\begin{tabular}{llll}
STATE_TRANSITION_PROCEDURE & stateTransitionProcedure & procédure de transition d'état & {\bf EntityPredicate}  \\
FEASIBILITY_CONDITION & feasibilityCondition & condition de faisabilité & {\bf EntityPredicate}  \\
\end{tabular} 
}\\

Le constructeur d'une classe d'opérations dans un domaine d'application doit spécifier le changement d'état du système que provoque l'opération. A cet effet, il doit comporter un appel qui affecte un corps à la méthode de la classe {\bf Operation}, dite 'procédure de transition d'état' dont le symbole est STATE\_TRANSITION\_PROCEDURE :\\

\BTab4 {\tt AssignBody(STATE_TRANSITION_PROCEDURE, f);}

où {\tt f} est une fonction écrite par le développeur.\\

Le corps de {\tt f} exploite normalement son argument pM, qui est la méthode dont {\tt f} est le corps. {\tt f} a donc accès, notamment :\\
\begin{itemize} 
\item à l'opération, par le service {\bf DescribedEntity}, et ainsi à l'activité (valeur du descripteur de symbole INSTRUCTION\_CONTAINER) 
\item à l'entité dont l'opération est censée changer l'état, placée par le moteur dans la liste des arguments de la méthode. {\tt f} peut alors modifier l'état de cette entité, par exemple de la manière suivante : 
\end{itemize}

\BTab4 {\tt bool f (EntityMethod pM) \{ }

\BTab4 {\tt Entity* pObj = pM->GetEntityArgValue(OPERATION_TARGET);}

\BTab4 {\tt float x = pObj->GetFloatValue(TEMPERATURE); }

\BTab4 {\tt x = x * 1.1};

\BTab4 {\tt pObj->SetValue(TEMPERATURE, x); }

\BTab4 {\tt \ldots }

\BTab4 {\tt return TRUE};

\BTab4 {\tt \}; }\\

Le corps de la procédure de transition d'état renvoie TRUE dans le cas général. Seulement pour les opérations de transfert (voir le paragraphe dédié), la valeur renvoyée est FALSE pour indiquer que le transfert n'a pas pu être réalisé comme spécifié.

C'est le moteur de simulation qui se charge d'invoquer la procédure de transition d'état ainsi affectée à l'opération. Il le fait à chaque {\em pas de progression} de l'opération (voir plus loin la définition du {\em pas}). De plus, à chaque pas, le moteur détermine le nombre d'invocations nécessaires et l'entité, valeur de l'argument OPERATION_TARGET, celle dont l'opération change l'état durant ce {\em pas}. Cette détermination est faite de manière spécifique à chacune des trois particularisations de la classe {\bf Operation}, comme décrit ci-après.\\

Pour chaque instruction dans le jeu choisi (voir ...), la méthode de symbole ACT_INSTRUCTION_LIST de l'{\bf OperatingSystem} accède à l'instance d'{\bf Operation} et lui associe un processus continu, instance de la classe prédéfinie {\bf ProceedOperation}, qui va prendre en charge la progression dans le temps de l'effet de l'opération.\\

Une fois construit, le processus est lui-même associé à un événement qui peut être de deux classes différentes, prédéfinies dans \Db :
\begin{itemize}
\item {\bf ProceedFromCurrentState} si l'opération a déjà eu un commencement d'exécution et a été interrompue : le processus est alors associé à une directive PROCEED ;
\item {\bf SetInitialStateAndProceed} dans le cas contraire : il est alors associé à deux directives successives, INIT et PROCEED ;
\end{itemize}

Enfin, cet événement est placé dans l'agenda de la simulation, à la date courante, pour un début ou une reprise immédiate de l'effet de l'opération.\\

Dans le constructeur de {\bf ProceedOperation}, c'est l'opération qui est désignée, par le service {\bf ProcessedEntity} de \D, comme l'entité sur laquelle l'instance de processus va opérer un changement d'état (essentiellement sur la situation, le degré de progression, les dates de début et de fin). Il établit le {\em pas} à la valeur du {\em pas} de l'opération (renvoyé par le message {\tt GetIntValue(STEP)} adressé à l'opération). Il contient enfin l'assignation d'un corps standard aux méthodes d'initialisation, de poursuite et d'arrêt. Ceci est réalisé par les services de \Db :\\

\BTab4 {\tt SetInitialize(f);}

\BTab4 {\tt SetGoOneStepForward(g);}

\BTab4 {\tt SetStop(h);}

où :\\
\begin{itemize}

\item {\tt f} est une fonction d'initialisation dont \CDb fournit trois versions, chacune adaptée à une particularisation d'{\bf Operation}. Le constructeur de {\bf ProceedOperation} choisit la bonne version en fonction du type de l'opération en jeu. Dit de façon sommaire, elles donnent la valeur {\em 0} au descripteur de symbole PROGRESSION\_DEGREE de l'opération, la valeur EXECUTING au descripteur de symbole SITUATION de l'opération, la valeur IN\_COURSE au descripteur de symbole SITUATION de l'instruction. La date de début de l'opération est valuée avec l'instant courant. Le constructeur établit enfin la liste des entités objets de l'effet de l'opération. Ces entités sont des instances de la classe dont le symbole est la valeur du descripteur OPERATED_ENTITY_ID de l'opération. Plus précisément, ce sont celles qu'on trouve à une profondeur quelconque comme élément des entités objet de l'instruction (voir page \pageref{}).\\

\item {\tt g} est une fonction de poursuite du processus dont \CDb fournit trois versions. Le constructeur du processus choisit la version correspondant au type de l'opération en jeu. 
\begin{description} 

\item[UnitGranulatedOperation : ]la version {\bf FullEffectOnUnits} enchaîne (rappel : à l'intérieur d'un {\em pas}) un certain nombre d'invocations (en fait égal à la vitesse {\em n} de l'opération) du corps de la méthode de l'opération de symbole STATE_TRANSITION_PROCEDURE. A chaque invocation, une nouvelle entité de la liste créée dans l'initialisation est placée comme valeur de l'argument OPERATION_TARGET de la méthode STATE_TRANSITION_PROCEDURE. On rappelle que le corps de cette méthode reçoit en paramètre la méthode elle-même, ce qui permet d'accéder à l'entité {\em via} son argument OPERATION_TARGET.

\item[QuantityGranulatedOperation : ]la version {\bf FullEffectOnQuantity} recherche une entité particulière dans la liste des entités objets de l'instruction (voir ci-dessus) : la première telle que la totalité de la valeur d'une certaine dimension  n'a pas été opérée. Cette dimension est celle dont le symbole est la valeur du descripteur QUANTITY_DESCRIPTOR_ID. On dénommme 'quantité' une valeur sur cette dimension. Seulement si cette entité n'a pas du tout été opérée, {\bf FullEffectOnQuantity} invoque le corps de la méthode STATE_TRANSITION_PROCEDURE de l'opération, après avoir désigné l'entité comme valeur de l'argument OPERATION_TARGET. 
\noLF
Dans tous les cas, le degré de progression de l'opération est augmenté du rapport entre la vitesse de l'opération et la quantité totale sur les entités objets de l'instruction. Il y a en tête de la liste un nombre d'entités dites totalement opérées tel que le produit du nombre d'invocations de {\bf FullEffectOnQuantity} (depuis l'initialisation du processus) par la vitesse de l'opération est supérieur ou égal à la somme des quantité sur ce nombre d'entités.

\item[TimeGranulatedOperation : ]si et sulement si le degré de progression de l'effet est nul, {\bf StepEffectOnObject} invoque le corps de la méthode STATE_TRANSITION_PROCEDURE de l'opération successivement sur chaque entité objet de l'instruction (voir ci-dessus), après l'avoir désignée comme valeur de l'argument OPERATION_TARGET. Ensuite et à chaque pas, le degré de progression de l'opération est augmenté de la vitesse de l'opération. 
\end{description}

\item {\tt h} est une fonction standard d'arrêt du processus, invoquée dans les trois méthodes de poursuite lorsque le degré de progression devient 100. Elle donne la valeur TERMINATED au descripteur SITUATION de l'opération. La date de fin de l'opération est valuée avec l'instant courant. Les autres opérations en cours de progression deviennent SUSPENDED et les processus associés recoivent un message {\tt ToBeStopped(TRUE)}. Enfin est programmé pour l'instant courant un événement {\bf C_MakeInstructionListEvent} déclenchant le processus {\bf C_MakeInstructionListProcess} dont la procédure d'exécution invoque le corps de la méthode de symbole MAKE_INSTRUCTION_LIST de l'{\bf OperatingSystem}. Le message {\tt ToBeStopped(TRUE)}, et non le message {\tt Stop()} est adressé aux processus correspondant aux opérations suspendues, pour éviter des exécutions superflues de MAKE_INSTRUCTION_LIST.
\end{itemize}
*/
//@{

 
/**@name Les opérations en général */
//@{

//----------------------------------------------------------------------------
// OPERATION
//----------------------------------------------------------------------------
/**@name La classe de base des opérations */ 
//@{
          ///
class Operation : public Entity 
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  Operation();  // _no swig_
      
//@}
  
/**@name ACCESS */
//@{
    
         /// Renvoie le bloc activités-ressources dans lequel cette opération est incluse.\index{méthodes!GetActivitiesResourcesBlock}
ActivitiesResourcesBlock* GetActivitiesResourcesBlock();  // _no swig_ 
    
         /// Renvoie l'activité primitive dans laquelle cette opération est incluse.\index{méthodes!GetInstructionContainer}
PrimitiveActivity* GetInstructionContainer();  // _no swig_ 
    
         /// Renvoie {\em vrai} si l'opération est permanente, c'est-à-dire si on ne peut pas l'arrêter par l'atteinte de la valeur 1 pour le degré de progression. En fait, ce degré reste {\em epsilon}, qui est la valeur attribuée nécessairement à la {\em vitesse}. Seules les {\bf QuantityGranulatedOperation} et les {\bf TimeGranulatedOperation} peuvent être spécifiées comme permanentes.\index{méthodes!IsPermenent}
Boolean IsPermanent();  // _no swig_ 

//@}
     
/**@name OPERATIONS */
//@{  
           ///
void DeleteInDepth();  // + JPR 110318
void DeleteInDepth1();  // + JPR 110318
void DeleteInDepth2();  // + JPR 110318
void DeleteInDepth3();  // + JPR 110318
void DeleteInDepth4();  // + JPR 110318

         /// Renvoie la date de fin au plus tard de l'{\em instruction} qui porte sur l'opération réceptrice du message.\index{méthodes!MaxEndDate (opération)}
int MaxEndDate(); 

         // Renvoie le pas spécifié ou calculé par la méthode SetUserStep.
int GetStep(); 

//@} 

protected :

}; // _end class_
//@}

/**@name Les opérations spécialisées selon le mode de progression */
//@{

//----------------------------------------------------------------------------
// UNIT OPERATION
//----------------------------------------------------------------------------

/**@name Les opérations progressant unité par unité
La vitesse est un nombre d'entités traitées en un {\em pas} de l'opération.\\
Ces entités sont de la classe dont le symbole est renvoyé par un appel à {\bf GetIntValue(OPERATED\-_ENTITY_ID)}.\\
La vitesse est spécifiée par un appel {\tt SetValue(UNIT_SPEED, x)}, ou bien en donnant un corps à l'instance de {\bf EntityMethod} dont le symbole est SET_UNIT_SPEED.\\

Le symbole de classe est UNIT_GRANULATED_OPERATION.\\
Le nom de classe est "unitOt".\\
*/
//@{
          ///
class UnitGranulatedOperation : public Operation
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  UnitGranulatedOperation();  // _no swig_
//@}
  
/**@name ACCESS */
//@{
           /// Renvoie la vitesse de l'opération. Si un corps a été donné à la méthode dont le symbole est SET_UNIT_SPEED, on prend la valeur retournée par ce corps. Sinon, on prend la valeur du descripteur dont le symbole est UNIT_SPEED.
int GetUnitSpeed();  // _no swig_

           /// Renvoie le nombre d'unités traitées en un pas. Exploite la méthode {\bf GetUnitSpeed()} et module le résultat en fonction le la puissance affectée à la réalisation de l'activité.
  int GetUnitQuantityPerStep();  // _no swig_         // + 100322

//@} 

protected :
}; // _end class_

//@} // fin UNIT

//----------------------------------------------------------------------------
// QUANTITY OPERATION
//----------------------------------------------------------------------------

/**@name Les opérations progressant par une portion de quantité

La vitesse est une quantité (sur la dimension qui est la valeur de QUANTITY_DESCRIPTOR_ID, par exemple une surface, un volume, \ldots) traitée en un {\em pas} de l'opération.\\

La vitesse est spécifiée par un appel {\tt SetValue(QUANTITY_SPEED, x)}, ou bien en donnant un corps à l'instance de {\bf EntityMethod} dont le symbole est SET_QUANTITY_SPEED.\\

Le symbole de classe est QUANTITY_GRANULATED_OPERATION.\\
Le nom de classe est "quantityOt".\\
*/
//@{

/**@name ... la quantité est une valeur entière */
//@{
          /// 
//----------------------------------------------------------------------------
class IntQuantityGranulatedOperation : public Operation
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  IntQuantityGranulatedOperation();  // _no swig_
//@}
  
/**@name ACCESS */
//@{
           /// Renvoie la vitesse de l'opération. Si un corps a été donné à la méthode dont le symbole est SET_INT_QUANTITY_SPEED, on prend la valeur retournée par ce corps. Sinon, on prend la valeur du descripteur dont le symbole est QUANTITY_SPEED.
  int GetIntQuantitySpeed();  // _no swig_

           /// Renvoie la quantité (entière) traitée en un pas. Exploite la méthode {\bf GetIntQuantitySpeed()} et module le résultat en fonction le la puissance affectée à la réalisation de l'activité.
  int GetIntQuantityPerStep();  // _no swig_

//@}

protected :
}; // _end class_

//@}  // fin IntQuantity

/**@name ... la quantité est une valeur réelle */
//@{
          ///
class FloatQuantityGranulatedOperation : public Operation
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  FloatQuantityGranulatedOperation();  // _no swig_
//@}
  
/**@name ACCESS */
//@{
           /// Renvoie la vitesse de l'opération. Si un corps a été donné à la méthode dont le symbole est SET_FLOAT_QUANTITY_SPEED, on prend la valeur retournée par ce corps. Sinon, on prend la valeur du descripteur dont le symbole est QUANTITY_SPEED.
  float GetFloatQuantitySpeed();  // _no swig_

           /// Renvoie la quantité (flottante) traitée en un pas. Exploite la méthode {\bf GetFloatQuantitySpeed()} et module le résultat en fonction le la puissance affectée à la réalisation de l'activité.
  float GetFloatQuantityPerStep();  // _no swig_

//@}

protected :
}; // _end class_

//@}  // fin FloatQuantity

//@}  // fin Quantity

//----------------------------------------------------------------------------
// TIME OPERATION
//----------------------------------------------------------------------------

/**@name Les opérations progressant par une proportion de la réalisation totale
La vitesse est une proportion de la réalisation totale réalisée en un {\em pas} de l'opération.\\
La vitesse est spécifiée par un appel {\tt SetValue(TIME_SPEED, x)}, ou bien en donnant un corps à l'instance de {\bf EntityMethod} dont le symbole est SET_TIME_SPEED.\\
Le symbole de classe est TIME_GRANULATED_OPERATION.\\
Le nom de classe est "timeOt".\\
*/
//@{
          ///
//----------------------------------------------------------------------------
class TimeGranulatedOperation : public Operation
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  TimeGranulatedOperation();  // _no swig_
//@}
  
/**@name ACCESS */
//@{
           /// Renvoie la vitesse de l'opération. Si un corps a été donné à la méthode dont le symbole est SET_TIME_SPEED, on prend la valeur retournée par ce corps. Sinon, on prend la valeur du descripteur dont le symbole est TIME_SPEED.
  float GetTimeSpeed();  // _no swig_

           /// Renvoie le pourcentage de réalisation de l'opération réalisé en un pas. Exploite la méthode {\bf GetTimeSpeed()} et module le résultat en fonction le la puissance affectée à la réalisation de l'activité.
  float GetPercentPerStep();  // _no swig_

//@}

protected :
}; // _end class_

//@} // fin TIME 

//@} // fin 4 spécialisations

//@} // fin les opérations en général

//----------------------------------------------------------------------------
// TRANSFER OPERATION
//----------------------------------------------------------------------------

/**@name Les opérations de transfert */
//@{

/**@name Les opérations de transfert progressant unité par unité */
//@{
          ///
//----------------------------------------------------------------------------
class UnitGranulatedTransferOperation : public UnitGranulatedOperation
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  UnitGranulatedTransferOperation();  // _no swig_
//@}

protected :
}; // _end class_

//@} // fin UNIT Transfer

/**@name Les opérations de transfert progressant par une portion de quantité */
//@{

/**@name ... la quantité est une valeur entière */
//@{
          ///
//----------------------------------------------------------------------------
class IntQuantityGranulatedTransferOperation : public IntQuantityGranulatedOperation
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  IntQuantityGranulatedTransferOperation();  // _no swig_
//@}

protected :
}; // _end class_

//@}


/**@name ... la quantité est une valeur réelle */
//@{
          ///
//----------------------------------------------------------------------------
class FloatQuantityGranulatedTransferOperation : public FloatQuantityGranulatedOperation
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  FloatQuantityGranulatedTransferOperation();  // _no swig_
//@}

protected :
}; // _end class_

//@}

//@} // fin QTY Transfer

//----------------------------------------------------------------------------
/**@name fonctions */
//@{
           /// fonction standard codant le transfert unité par unité
bool unitGranulatedTransferOperation_stateTransitionProcedure_Body(EntityMethod* pM);
           /// fonction standard codant le transfert par quantité entière
bool intQuantityGranulatedTransferOperation_stateTransitionProcedure_Body(EntityMethod* pM);
           /// fonction standard codant le transfert par quantité réelle
bool floatQuantityGranulatedTransferOperation_stateTransitionProcedure_Body(EntityMethod* pM);

  
//@} // fin fonctions
//----------------------------------------------------------------------------

//@} // fin opérations de transfert

//@} // fin opérations sur le système pilot\'e


#endif  // _C_OPERATION_HH
