//============================================================================
// FILE: O_CoreFunction.h
//
// TYPE: C-LIBRARY
// DESCRIPTION: Miscellaneous C functions for Open-Diese
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : 
//=X===============X=============================X============================

#ifndef _O_COREFUNCTIONS_H
#define _O_COREFUNCTIONS_H

// SYSTEM INCLUDES
//
//#include <stdarg.h>
//#include <stdlib.h>

// PROJECT INCLUDES
//
#include "O_CoreTypes.h"

/**@name CDPFORTEXFonctions globales
\begin{itemize} \item Directive d'inclusion : {\tt \#include CoreFunction.h} \end{itemize} 
On présente ici les fonctions, au sens du langage {\bf C}, c'est-à-dire les procédures (avec effets de bord) et les fonctions au sens strict (retournant une valeur) qui ne sont pas attachées à des classes d'objets sous le vocable et avec le r\^ole de "méthodes".\\
Elles sont globales parce qu'on peut les invoquer dans l'ensemble du code, celui de \Db et celui spécifique au domaine d'application.
*/ 
//@{
//===========================================================================

/**@name Fonctions d'initialisation 

Ces deux fonctions ne font pas partie de la bibliothèque \OD. C'est au niveau de l'application qu'elles peuvent être développées. L'interface Solfege propose pour chacune d'elles une version standard, qui peut être exploitée telle quelle ou bien après modification ou augmentation.
*/
//@{
         /**@name InitSimulation
 Fonction d'initialisation pour un simulateur autonome, à invoquer dans la fonction main() du simulateur. Instancie la classe {\bf Simulation], installe la hiérarchie des classes et définit les moniteurs.\index{fonctions!InitSimulation}
		 */
void InitSimulation();

         /**@name InitOpenSimulation
Fonction d'initialisation pour un simulateur couplé, à invoquer dans le programme qui contrôle le simulateur. Outre les tâches listées pour {\bf InitSimulation}, réalise la lecture des fichiers en entrée ('paramètres', 'simulation', 'structure'), et peut être augmentée avec la programmation d'événements qui initialisent l'état du système via des processus.\index{fonctions!InitOpenSimulation}
		 */
void InitOpenSimulation(char*, char*, char*);

         /**@name InitOpenSimulation
Même rôle que la fonction précédente fonction. Réalise la lecture des fichiers en entrée 'paramètres', 'simulation', 'spéc. de sortie' et 'structure'.
		 */
void InitOpenSimulation(char*, char*, char*, char*);

//@}  // fin Fonctions d'initialisation
//===========================================================================
//===========================================================================

/**@name Fonctions de finalisation 

Ces deux fonctions ne font pas partie de la bibliothèque \OD. C'est au niveau de l'application qu'elles peuvent être développées. L'interface Solfege propose pour chacune d'elles une version standard, qui peut être exploitée telle quelle ou bien après modification ou augmentation.
*/
//@{
         /**@name FinalizeSimulation
 Fonction de finalisation pour un simulateur autonome, à invoquer dans la fonction main() du simulateur. Typiquement, elle traite les spécifications de sortie déclarées dans l'initialisation de la simulation, et elle libère la mémoire encore allouée.\index{fonctions!FinalizeSimulation}
		 */
void FinalizeSimulation();

         /**@name FinalizeOpenSimulation
Fonction de finalisation pour un simulateur couplé, à invoquer dans le programme qui contrôle le simulateur. Elle réalise la lecture du fichier en entrée qui contient les directives de fin de simulation (traitement des spécifications de sortie déclarées dans l'initialisation de la simulation, et libèration de la mémoire encore allouée).\index{fonctions!FinalizeOpenSimulation}
		 */
void FinalizeOpenSimulation(char*);

//@}  // fin Fonctions de finalisation 
//===========================================================================

/**@name Fonctions d'instanciation */
//@{
         /// Fonction d'instanciation des classes d'entités prédéfinies dans \ODb (activités, \ldots). Ce sont les constructeurs sans paramètre qui sont invoqués. Cette fonction doit être invoquée dans la fonction {\bf App_NewEntityFromString}, complémentairement au code qui invoque les constructeurs des classes propres à l'application.
Entity* O_NewEntityFromString(char*);

         /// Fonction d'instanciation des classes de processus prédéfinies dans \ODb ({\bf ResourceImmobilizationProcess}, \ldots). Ce sont les constructeurs sans paramètre qui sont invoqués. Cette fonction doit être invoquée dans la fonction {\bf App_NewProcessFromString}, complémentairement au code qui invoque les constructeurs des classes propres à l'application.
Process* O_NewProcessFromString(char*);

         /// Fonction d'instanciation des classes d'événements prédéfinies dans \ODb ({\bf UpdateSituationEvent}, \ldots). Ce sont les constructeurs sans paramètre qui sont invoqués. Cette fonction doit être invoquée dans la fonction {\bf App_NewEventFromString}, complémentairement au code qui invoque les constructeurs des classes propres à l'application.
Event* O_NewEventFromString(char*);

         /// Fonction d'instanciation des classes de spécifications d'ensembles d'entités prédéfinies dans \OD. Il n'y en a pas dans le version courante de \OD.
EntitySpec* O_NewEntitySpecFromString(char*);

         /// Fonction d'instanciation des classes de spécifications de domaines de valeurs prédéfinies dans \OD. Il n'y en a pas dans le version courante de \OD.
DescValueSpec* O_NewDescValueSpecFromString(char*);

         /// Fonction d'instanciation des classes de fichiers séquentiels de nom de classe donné, prédéfinies dans \OD. Il n'y en a pas dans le version courante de \OD.
SerialDataFile* O_NewSerialDataFileFromString(char*);

         /// Fonction d'instanciation des classes de fichiers séquentiels de symbole de classe donné, prédéfinies dans \OD. Il n'y en a pas dans le version courante de \OD.
SerialDataFile* O_NewSerialDataFileFromId(UserFileId);

         /// Fonction qui renvoie le symbole de la sous-classe de {\bf SerialDataFile} dont le nom de classe est passé en argument. Renvoie toujours -9 car il n'y en a pas dans le version courante de \OD.
UserFileId O_SerialDataFileIdFromString(char* s);

//@}  // fin Fonctions d'instanciation 
 
/**@name Fonctions sur la hiérarchie des classes d'entités */
//@{
         /// Fonction d'initialisation des informations sur les sous-classes de {\bf BasicEntity} propres à \OD. Cette fonction doit être invoquée au début du code de toute application (mais après la création d'une instance de {\bf Simulation}). \index{fonctions!O_InstallMetaClasses} 
void O_InstallMetaClasses(); // _no swig_

         ///Fonction d'initialisation des informations sur les sous-classes de {\bf BasicEntity} spécifiques à l'application. Cette fonction doit être invoquée au début du code de toute application (mais après la création d'une instance de {\bf Simulation} et après l'invocation de {\bf C\_InstallMetaClasses}). Noter que ces appels, ainsi qu'un appel à {\bf InstallMetaClass} (voir le manuel de \D) sont automatiquement réalisés si l'initialisation de la simulation est faite dans un fichier de spécifications de la simulation, et si ce fichier est lu par un appel à {\bf C\_ParseSimulationFile}.\index{fonctions!O_App_InstallMetaClasses}\label{commentInstallMetaClasses} 
void O_App_InstallMetaClasses(); // _no swig_  

         ///
void O_MakeClassIdToCharTab(); // _no swig_  
         ///
void O_App_MakeClassIdToCharTab(); // _no swig_  
         ///
void O_MakeDescIdToCharTab(); // _no swig_  
         ///
void O_App_MakeDescIdToCharTab(); // _no swig_  
         ///
void O_MakeMethodIdToCharTab(); // _no swig_    
         ///
void O_MakeArgumentIdToCharTab(); // _no swig_  
         //
//void O_App_MakeMethodIdToCharTab(); // _no swig_ 

         /// Si pObj est une instance d'une classe définie dans \OD, exécute le destructeur de cette classe, sinon appelle la fonction {\bf DeleteEntity}.
void      O_DeleteEntity(BasicEntity* pObj);

///Supprime toutes les instances (celles non encore désallouées au moment de l'appel de cette fonction) des sous-classes "utilisateur" de {\bf BasicEntity} (voir page \pageref{DeleteDOinstances}). Un appel de {\em DeleteEntityInstances} ne doit pas suivre immédiatement un appel de {\bf DeleteEntityClasses}. \index{fonctions!DeleteEntityInstances} 
void         O_DeleteEntityInstances(); // _no swig_  

///Supprime toutes les instances (celles non encore désallouées au moment de l'appel de cette fonction) de la sous-classe "utilisateur" de {\bf BasicEntity} dont l'entier correspondant au symbole de classe est égal à {\em id}.  
void         O_DeleteEntityInstances(UserClassId id); // _no swig_ 

///Supprime toutes les instances (celles non encore désallouées) de toutes les classes, et affiche un rapport par {\bf MemoryReport}. Supprime aussi l'instance de {\bf Simulation} pointée par {\bf pCurrentSim}. Les instances de {\bf BasicEntity} sont désallouées par {\bf O_DeleteEntityInstances}. Les instances de {\bf File} sont désallouées par {\bf C_DeleteFileInstances}.\index{fonctions!O_DeleteAll} 
void         O_DeleteAll(); // _no swig_

///Si l'argument est FALSE, invoque {\tt O_MemoryReport()} seulement si l'option {\tt -m} a été prise dans la ligne de commande. Invoque {\tt O_MemoryReport()} sans condition si l'argument est TRUE. 
void O_MemoryReport (Boolean); // _no swig_ 

///Affiche un rapport sur les constructions des instances des classes prédéfinies de \D, \CD et \OD. On place typiquement un appel à cette fonction en fin de fonction 'main'. Les classes en question ne doivent pas avoir été supprimées par un appel approprié à {\bf DeleteEntityClasses}. Une séquence stantard est : {\tt C_DeleteEntityInstances();} {\tt O_MemoryReport();} {\tt C_DeleteEntityClasses();}\index{fonctions!O_MemoryReport} 
void O_MemoryReport (); // _no swig_

//@}  // fin Fonctions sur la hiérarchie


/**@name Fonctions diverses
\begin{itemize} \item Directive d'inclusion : {\tt \#include O\_CoreFunction.h} \end{itemize}
*/
//@{
          /// Renvoie l'entier correspondant à un token d'énumération dont l'identifiant est le mot passé en argument. Commence par invoquer C_EnumToInt.\index{fonctions!O_EnumToInt}
int O_EnumToInt(char* str);

          ///Interprète les arguments de la commande exécutable. Renvoie FALSE si la commande n'a pas pu être complètement interprétée. Commence par invoquer ParseMainArguments de la couche \D, puis C_arseMainArguments de la couche \CD.\index{fonctions!O_ParseMainArguments}
Boolean O_ParseMainArguments (int argc, char* argv[]); // + JPR 100518 

// Crée une instance de chaque classe devant figurer dans le fichier *.cls, c'est-à-dire celles dont l'utilisateur de Solfege pourra développer des spécialisations. Exploité seulement dans mainMakeCls.cc.
void             O_InstantiateClassesForCls(); // + JPR 090819 

//@}  // fin Fonctions diverses


//@}  // fin Fonctions globales

#endif  // _O_COREFUNCTIONS_H
